/*[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]*/
/* UList.p */
/* Copyright  1984-1989 by Apple Computer Inc. All rights reserved. */

/* Unit UList defines object types TList, a simple dynamic array whose elements are references to
  TObjects, and TSortedList, which maintains the object references in a sequence defined by
  overriding TSortedList.Compare. */
#ifndef  __UList__
#define __UList__  0
#endif
#if  ! __UList__
#define __UList__  1

		/*  Auto-Include the requirements for this unit's interface. */
#ifndef  __UObject__
#include "UObject.h"
#endif

const short kDeletedElement		= 0xFFFFFFFF;

const short kALessThanB			= - 1;					/* Constants for TSortedList.Compare */
const short kAEqualB			= 0;
const short kAGreaterThanB		= 1;

const short kItemGreaterThanCriteria = - 1;				/* Constants for TSortedList.Search */
const short kItemEqualCriteria	= 0;
const short kItemLessThanCriteria = 1;
			/*RW means OK for other objects to Read and Write this field*/
			/*Rx means OK for other objects to Read, but NOT TO WRITE this field*/
			/*xx means other objects should NEITHER READ NOR WRITE this field*/

			/*A dynamic list of TObjects*/

class TList : public TObject {
  public:				/*It IS permissible for descendants to add
                                                         more named fields*/
	short fSize;										/*Rx*/
	   /*number of elements in the list, from 0
                                                              to the limit of memory*/
	long fFirstOffset;									/*xx*/
	   /*= SIZEOF(SELF) = no. of bytes of named
                                                              fields before first element*/
	short fDeletions;									/*Rx*/
	   /*The number of deleted elements in the
                                                              list. These have value
                                                              kDeletedElement; fSize always
                                                              reflects # of real elements.*/
	short fEachLevel;									/*Rx*/ /*Number of Each calls in progress.*/
	ObjClassID fObjClassID;								/*xx*/ /* if <> 0 then the Class ID of the
                                                              elements of the list */

								/*Creation/Destruction Methods*/

	virtual pascal void IList(void);
				/* Initialize a new list with no elements, i.e., fSize = 0.  Always
                  call it once before calling any other method.  Never call it
                  twice.*/

	virtual pascal void FreeList(void);
				/* Frees each object in the list, then frees the list.*/

				/* Utility Methods*/

	virtual pascal short GetSize(void);
				/* Returns the number of elements in the list.*/

	virtual pascal TObject *At(short index);

	  /* Return the index'th element of the list.  It is typical for the caller to coerce the
                  result into a descendant of TObject.  All lists are indexed from 1.  R
   ange check only
                  if the compile-flag qRangeCheck is TRUE.
                  The static-array equivalent of:
                    object := objList.At(index);
                  is:
                    object := objArray[index];*/

	virtual pascal short GetEqualItemNo(TObject *item);

	  /* Return the index of the item in the list, or zero if the item is not in the list. */

	virtual pascal short GetSameItemNo(TObject *item);

	  /* Return the index of the item in the list, or zero if the item is not in the list. */

	virtual pascal TObject *First(void);

	 /* Return the first element of the list. It is typical for the caller to coerce the result.
                  Returns NIL if the size is <= 0.  The static-array equivalent of:
                    object := objList.First;
                  is:
                    object := objArray[1];*/

	virtual pascal TObject *Last(void);

	 /* Return the last element of the list.  It is typical for the caller to coerce the result.
                  Returns NIL if the size is <= 0.  The static-array equivalent of:
                    object := objList.Last;
                  is:
                    object := objArray[number of elements];*/

				/*Iterator Methods*/

	virtual pascal void Each(pascal void (*DoToItem)(TObject *item, void *DoToItem_StaticLink), void
	    *DoToItem_StaticLink);

		/* Call DoToItem once for each element of the list, in order. The actual parameter is
                  typically a procedure whose argument is a descendant of TObject. If DoToItem calls
                  InsertLast, the newly added element will NOT be enumerated.  If TestIt
   em calls AtPut,
                  InsertBefore, InsertFirst, Delete, or DeleteAll,  misbehavior will ens
   ue.  If DoToItem
                  calls Delete, the item is replaced by the constant kDeletedElement; fS
   ize is decremented
                  and fDeletions is incremented. At the end of all calls to Each, TList.
   RemoveDeletions
                  will be called.  The static-array equivalent of:
                    objList.Each(Proc)
                  is:
                    FOR index := 1 TO fSize DO
                        Proc(objArray[index]);
                */

	virtual pascal TObject *FirstThat(pascal Boolean (*TestItem)(TObject *item, void *
	   TestItem_StaticLink), void *TestItem_StaticLink);

	 /* Call TestItem once for each element of the list, in order, until TestItem returns TRUE.
                  Return the element that satisfied the test.  If none satisfied the tes
   t, return NIL.
                  The actual parameter is typically a procedure whose argument is a desc
   endant of TObject.
                  It is typical for the caller to coerce the result into that descendant of TObject.
                  If TestItem calls InsertLast, the newly added element will NOT be enumerated.
                  If TestItem calls AtPut, InsertBefore, InsertFirst, Delete, or DeleteA
   ll, misbehavior will
                  ensue.  The static-array equivalent of:
                    object := objList.FirstThat(Func)
                  is:
                    object := NIL;
                    FOR index := 1 TO fSize DO IF Func(objArray[index]) THEN
                        BEGIN
                        object := objArray[index];
                        LEAVE;
                        END;
                */

	virtual pascal TObject *LastThat(pascal Boolean (*TestItem)(TObject *item, void *
	   TestItem_StaticLink), void *TestItem_StaticLink);

	 /* Call TestItem once for each element of the list, starting with the last item in the list
                  and working toward the first item, until TestItem returns TRUE.  Retur
   n the element that
                  satisfied the test.  If none satisfied the test, return NIL. The actua
   l parameter is
                  typically a procedure whose argument is a descendant of TObject. It is
    typical for the
                  caller to coerce the result into that descendant of TObject.  If TestI
   tem calls InsertLast,
                  the newly added element will NOT be enumerated.  If TestItem calls AtP
   ut, InsertBefore,
                  InsertFirst, Delete, or DeleteAll, misbehavior will ensue.
                  The static-array equivalent of:
                    object := objList.LastThat(Func)
                  is:
                    object := NIL;
                    FOR index := fSize DOWNTO 1 DO IF Func(objArray[index]) THEN
                        BEGIN
                        object := objArray[index];
                        LEAVE;
                        END;
                */

				/*Item Insertion Methods*/

	virtual pascal void AtPut(short index, TObject *newItem);

	 /* Replace the index'th item of the list.  Does not free the old item.  If you do this from
                  within an Each the results will be unpredictable.*/

	virtual pascal void InsertBefore(short index, TObject *item);

	 /* Insert a reference to an item before the indicated item.  The index of the new element
                  will be index.   If index = 1 this inserts at the start of the list.  
   If index = fSize + 1
                  this inserts at the end of the list.  Signals Failure if unable to inc
   rease the size of
                  the list.*/

	virtual pascal void InsertFirst(TObject *item);

	   /* Insert a reference to item at the front of the list.  If the compile-flag qDebug is
                  TRUE & SetEltType was called, verify item's type.  Increase fSize by 1
   .  The index of the
                  new element will be 1. Signals Failure if unable to increase the size 
   of the list.*/

	virtual pascal void InsertLast(TObject *item);

		/* Insert a reference to item at the back of the list.  If the compile-flag qDebug is
                  TRUE & SetEltType was called, verify item's type.  Increase fSize by 1
   .  The index of
                  the new element will be fSize.  Signals Failure if unable to increase 
   the size of the
                  list.*/

				/*Item Deletion Methods*/

	virtual pascal void Delete(TObject *item);

	 /* Delete the first reference to item from the list, but do not free item.  If item does
                  not occur, do nothing. If item does occur, reduce fSize by 1.*/

	virtual pascal void DeleteAll(void);

	  /* Delete every element from the list, but do not free any objects. Leave fSize at 0.*/

	virtual pascal void FreeAll(void);
				/* Frees each element in the list.  Leave fSize at 0.*/

				/*Internal Methods*/

	virtual pascal long ComputeAddress(short index);

		/* Returns the memory address of the index-th element in the list. Used internally.*/

	virtual pascal long MaxAddress(void);
				/* Returns the memory address after the last item in the list. Used internally.*/

	virtual pascal void RemoveDeletions(void);
				/* Remove the deleted elements from the list.*/

				/* Debugging Methods */

	virtual pascal void SetEltType(StringPtr toClass);

	  /* Call this once and pass the className of objects you intend to insert into the list.
                  The TList insert methods will do a coercion to he type you specify, to
    ensure that the
                  list contains only elements of the right type.*/

	virtual pascal void SetEltTypeID(ObjClassID toClassID);

	 /* Call this once and pass the ObjClassID of objects you intend to insert into the list.
                  The TList insert methods will do a coercion to the type you specify, t
   o ensure that the
                  list contains only elements of the right type.*/

	virtual pascal void Fields(pascal void (*DoToField)(StringPtr fieldName, Ptr fieldAddr, short 
	   fieldType, void *DoToField_StaticLink), void *DoToField_StaticLink);

	virtual pascal void GetInspectorName(StringPtr inspectorName);
};

class TSortedList : public TList {
  public:

	virtual pascal TObject *DoSearch(pascal short (*TestItem)(TObject *anItem, void *
	   TestItem_StaticLink), void *TestItem_StaticLink, short *index);
				/* Searches the sorted list until TestItem returns zero or there are no more
                  items to search.  If the list contains deletions then a sequential search
                  is used; otherwise a binary search is used. */

	virtual pascal short GetEqualItemNo(TObject *item);
				/* Return the index of any item that is considered to be equal to the
                  parameter 'item'.  Two items are considered equal if comparing them with
                  the Compare method returns zero. */

	virtual pascal void Insert(TObject *item);
				/* Inserts the given item into the list in sorted order, using the Compare
                  method to determine the item's location relative to other items in the list. */

	virtual pascal short Compare(TObject *item1, TObject *item2);
				/* This method compares to items in the list.  It must be overridden.
                  A negative result indicates that item1 < item2.  A result of zero indicates
                  that item1 = item2.  A positive result indicates that item1 > item2.
                  You may wish to use the constants kALessThanB, kAEqualB, kAGreaterThanB. */

	virtual pascal TObject *Search(pascal short (*TestItem)(TObject *anItem, void *
	   TestItem_StaticLink), void *TestItem_StaticLink);
				/* This method searches the list of an item that causes your supplied
                  TestItem function to return true.  This is useful for cases
                  in which you are not comparing objects in the list with another object,

                  as does Compare.  For example, suppose each object has an fTitle field

                  and the objects are inserted into the list in order of fTitle. You
                  can use the search method to look for an object whose fTitle is equal
                  to any string.
                  A negative TestItem result indicates that your search criteria < anItem,

                  a result of zero indicates the item has been found and Search returns that
                  item.  A positive TestItem result indicates that your search criteria > your
                  anItem. */

};

extern pascal TList *NewList(void);
		/* A convenience function. Create a TList, initialize it, and  return a reference to it.
          Signals Failure if it cannot allocate the object.*/
#endif

