/*[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]*/
/* UPatch.p */
/* Copyright  1985-1989 by Apple Computer, Inc.  All rights reserved. */
/*[f-]*/
/*
        T H E O R Y   O F   O P E R A T I O N
This unit implements a general trap-patching mechanism.  Two types of
patches are implemented:
Normal Patch    The trap is patched such that it jumps to the routine
            of your choice, ignoring the routine it previously
            jumped to.  On a 128K ROM machine, the patch is made
            by setting the trap address to your routine.  On a 64K
            machine, a small block of code is allocated in the
            system heap:  The trap address is set to point to the
            block, and the block contains code to jump to your
            routine.
Tail Patch      The trap is patched such that it jumps to the
            routine of your choice, then jumps to the trap's
            routine.  Tail patches come in two flavors:  with
            no parameters and with one long-word parameter.  The
            patch is made by allocating a small block of code in
            the application heap (system heap for 64K ROMs).  The
            trap address is set to the block, and the block contains
            code to jump to your routine and return to the trap's
            original address.
References to patches are kept in TrapPatch records.  For each trap
patched you must define a variable of type TrapPatch.   This unit links
the TrapPatch records to form a linked list for facilitate unpatching
all patches without specifying each individual patch.
*/
/*[f+]*/
#ifndef  __UPatch__
#define __UPatch__  0
#endif
#if  ! __UPatch__
#define __UPatch__  1

		/*  Auto-Include the requirements for this unit's interface. */
#ifndef  __TYPES__
#include "Types.h"
#endif
#ifndef  __MEMORY__
#include "Memory.h"
#endif

typedef struct TrapPatch *TrapPatchPtr; 
struct TrapPatch {
	Ptr jmpPtr;											/* Addr of sys heap block */
	short trapNum;										/* trap # being patched */
	long oldTrapAddr;									/* old trap address */
	TrapPatchPtr nextPatch;								/* linked list of patches */
};

extern pascal void InitUPatch(void);
		/* Call this once to set things up.  Initializes the linked list of
        patches to NIL. */

extern pascal short PatchTrap(TrapPatch *thePatch, short theTrapNum, Ptr theRoutine);
		/* PatchTrap patches the given trap to theRoutine.   On a 128K ROM
        machine, we patch the trap's address directly.  On a 64K ROM
        machine, we must set up a block in the system heap, patch the
        trap to jump to the block, and insert code in the block to jump
        to theRoutine. */

extern pascal short TailPatch(TrapPatch *thePatch, short theTrapNum, Ptr theRoutine);
		/* TailPatch patches a trap such that it calls theRoutine first, then
        executes the old trap routine.  theRoutine must be a procedure
        with no arguments.  This type of patch is implemented by setting
        up a block in the system heap, patching the trap to jump to the
        block, and inserting code in the block that pushes the original
        trap address on the stack and jumps to theRoutine.  Thus,
        theRoutine returns to the original trap routine. */

extern pascal short Tail1Patch(TrapPatch *thePatch, short theTrapNum, Ptr theRoutine);
		/* Tail1Patch is just like TailPatch, except that it is intended to patch
        traps with a single long-word parameter.    The parameter passed to
        the trap is also passed to theRoutine.  theRoutine must be a
        procedure with one long-word parameter. */

extern pascal void UnpatchTrap(TrapPatch *thePatch);
		/* UnpatchTrap unpatches the given trap. */

extern pascal void UnpatchAll(void);
		/* UnpatchAll unpatches all traps. */
#endif

