{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n-]}

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShowTearOffWindowCmd.IShowTearOffWindowCmd(tearOffWindow:TWindow; where: Point);

	BEGIN
	INoChangesCommand(0, NIL, NIL, NIL);
	fTearOffWindow := tearOffWindow;
	fWhere := where;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShowTearOffWindowCmd.DoIt; OVERRIDE;

	BEGIN
	IF fTearOffWindow <> NIL THEN
		BEGIN
		fTearOffWindow.Locate(fWhere.h, fWhere.v, kDontInvalidate);	{ move it to its new home }
		fTearOffWindow.fMustForceOnScreen := TRUE;		{ so the Open will call ForceOnScreen }
		fTearOffWindow.fForcedOnScreen := FALSE;
		fTearOffWindow.Open;							{ show the torn-off window }
		fTearOffWindow.Select;							{ select the torn-off window }
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S MAFields}

PROCEDURE TShowTearOffWindowCmd.Fields(PROCEDURE DoToField(fieldName: Str255;
														   fieldAddr: Ptr;
														   fieldType: INTEGER)); OVERRIDE;

	BEGIN
	DoToField('TShowTearOffWindowCmd', NIL, bClass);
	INHERITED Fields(DoToField);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AInit}

PROCEDURE TTearOffTracker.ITearOffTracker(hitPt: Point; tearOffMenu: TTearOffMenu);

	VAR
		theMouse: VPoint;

	BEGIN
	theMouse.v := hitPt.v;
	theMouse.h := hitPt.h;	
	fOrigin := gZeroPt;
	ICommand(0, NIL, NIL, NIL);
	fCanUndo := FALSE;
	fCausesChange := FALSE;
	fTearOffMenu := tearOffMenu;
	fFreeOnCompletion := FALSE;
	fExitTracking := FALSE;
	fTracksMouse := TRUE;

	fTearOffWindowOutline := MakeNewRgn;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AClose}

PROCEDURE TTearOffTracker.Free; OVERRIDE;

	BEGIN
	IF fTearOffWindowOutline <> NIL THEN
		BEGIN
		DisposeRgn(fTearOffWindowOutline);
		fTearOffWindowOutline := NIL;
		END;

	INHERITED Free;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TTearOffTracker.DoIt;

	VAR
		where: Point;
		aTearOffWindow:TWindow;
		aShowTearOffWindowCmd: TShowTearOffWindowCmd;

	BEGIN
	IF NOT fExitTracking THEN		{ ensure that we didn't exit the tracking loop }
		BEGIN
		aTearOffWindow := fTearOffMenu.fTearOffWindow;

		where.v := fTearOffWindowOutline^^.rgnBBox.top + aTearOffWindow.fContRgnInset.v;
		where.h := fTearOffWindowOutline^^.rgnBBox.left + aTearOffWindow.fContRgnInset.h;

		IF aTearOffWindow <> NIL THEN
			BEGIN
			{ NOTE: we can't just move and show the tear off window here because the menu that
			we want to tearx off is currently displayed on the screen and the menu manager has
			cached the bits underneath the currently displayed menu and we might be moving our
			floating window into that territory thus invalidating the menu manager's cache (which
			the menu manager will blit back in place momentarily). To work-around this "feature"
			we'll simply post a command which will do the work later. Isn't MacApp wonderful! }
			New(aShowTearOffWindowCmd);
			FailNIL(aShowTearOffWindowCmd);
			aShowTearOffWindowCmd.IShowTearOffWindowCmd(aTearOffWindow, where);
			gApplication.PostCommand(aShowTearOffWindowCmd);
			END;

		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TTearOffTracker.TrackFeedback(anchorPoint, nextPoint: VPoint; turnItOn,
									   mouseDidMove: BOOLEAN);

	VAR
		mousePt: Point;

	BEGIN
	IF mouseDidMove & NOT EmptyRgn(fTearOffWindowOutline) THEN
		BEGIN
		PenPat(gray);
		FrameRgn(fTearOffWindowOutline); 							{ draw/erase }
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

FUNCTION TTearOffTracker.TrackMouse(aTrackPhase: TrackPhase; VAR anchorPoint, previousPoint,
								   nextPoint: VPoint; mouseDidMove: BOOLEAN): TCommand;

	VAR
		aTearOffWindow: TWindow;
		verticalOffset: INTEGER;
		horizontalOffset: INTEGER;
		rgnsWereBuilt: Boolean;

	BEGIN
	TrackMouse := SELF;

	fExitTracking := NOT PtInRgn(VPtToPt(nextPoint), fTearOffMenu.fTearOffTrackingRgn);

	IF (aTrackPhase = trackPress) THEN
		BEGIN
		aTearOffWindow := fTearOffMenu.fTearOffWindow;
		IF (aTearOffWindow <> NIL) THEN
			BEGIN
			rgnsWereBuilt := aTearOffWindow.BuildWindowRgns(kBuild);
			CopyRgn(WindowPeek(aTearOffWindow.fWMgrWindow)^.strucRgn, fTearOffWindowOutline);
			IF aTearOffWindow.BuildWindowRgns(rgnsWereBuilt) THEN; 	{ discard result }
			
			{ move the outline to the topleft of the window mgr graf port }
			{$Push} {$H-}
			WITH fTearOffWindowOutline^^.rgnBBox DO
				OffsetRgn(fTearOffWindowOutline, -left, -top);
			{$Pop}

			{ move the outline so that it's centered at the mouse }
			WITH fTearOffWindowOutline^^.rgnBBox DO
				BEGIN
				verticalOffset := (bottom - top) div 2;
				horizontalOffset := (right - left) div 2;
				END;
			WITH previousPoint DO
				OffsetRgn(fTearOffWindowOutline, h - horizontalOffset, v - verticalOffset);
			END;	{aTearOffWindow <> NIL}
		END
	ELSE
		WITH previousPoint DO
			OffsetRgn(fTearOffWindowOutline, nextPoint.h - h, nextPoint.v - v);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

FUNCTION TTearOffTracker.IsDoneTracking: BOOLEAN; OVERRIDE;

	BEGIN
	IsDoneTracking := NOT StillDown | fExitTracking;
	END;

{--------------------------------------------------------------------------------------------------}
{$S MAFields}

PROCEDURE TTearOffTracker.Fields(PROCEDURE DoToField(fieldName: Str255;
														   fieldAddr: Ptr;
														   fieldType: INTEGER)); OVERRIDE;

	BEGIN
	DoToField('TTearOffTracker', NIL, bClass);
	INHERITED Fields(DoToField);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AInit}

PROCEDURE TTearOffMenu.ITearOffMenu(rsrcID: INTEGER; menuWidth, menuHeight: INTEGER;
									tearOffWindow: TWindow);

	BEGIN
	fTearOffTrackingRgn := MakeNewRgn;
	fTearOffTracker := NIL;
	fTearOffWindow := tearOffWindow;

	IMenu(rsrcID, menuWidth, menuHeight);
	
	InstallTearOffTracker;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AInit}

PROCEDURE TTearOffMenu.InstallTearOffTracker;
{ For the sake of efficiency when tracking a tear-off, we'll create the tear-off tracker and
 keep it with us, so we don't need to allocate a new tracker every time we tear off a menu. If you
 want a different tracker in your subclass of TTearOffMenu, then override this method. }

	VAR
		tearOffTracker: TTearOffTracker;

	BEGIN
	New(tearOffTracker);
	FailNIL(tearOffTracker);
	tearOffTracker.ITearOffTracker(gZeroPt, SELF);
	fTearOffTracker := tearOffTracker;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AClose}

PROCEDURE TTearOffMenu.Free; OVERRIDE;

	BEGIN
	IF fTearOffTracker <> NIL THEN
		BEGIN
		fTearOffTracker.Free;
		fTearOffTracker := NIL;
		END;

	IF fTearOffTrackingRgn <> NIL THEN
		BEGIN
		DisposeRgn(fTearOffTrackingRgn);
		fTearOffTrackingRgn := NIL;
		END;

	INHERITED Free;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TTearOffMenu.HandleChooseMessage(message: INTEGER;
									theMenu: MenuHandle;
									VAR menuRect: Rect;
									hitPt: Point;
									VAR whichItem: INTEGER); OVERRIDE;

	VAR
		newItem:			INTEGER;
		hitRect: Rect;
		hitVPt: VPoint;
		hysteresis: Point;
		info: EventInfo;
		anEventRecord: EventRecord;
		theCommand: TCommand;
		saveTrackingInMenu: BOOLEAN;

	BEGIN
	{ so that trackers know that they're tracking in menus }
	saveTrackingInMenu := gTrackingInMenu;
	gTrackingInMenu := TRUE;

	newItem := kNoMenuItem; 							{ default return }

	IF IsViewEnabled THEN								{ menu enabled }
		BEGIN
		{ see if point is within hit area }
		GetQDExtent(hitRect);							{ hitRect in local coord's }
		AddPt(fBorder.topLeft, hitRect.topLeft);
		AddPt(fBorder.botRight, hitRect.botRight);
		IF PtInRect(hitPt, hitRect) THEN				{ in menu view (not border) }
			BEGIN
			{ NOTE: Either your subclass of TTearOffMenu should override DoMouseCommand or one of 
			TTearOffMenu's view's subview's should override DoMouseCommand so that it
			creates and posts a TCommand. TTearOffMenu's override of PostCommand will
			ensure that the tracker is tracked immediately. The tracker, having been posted and
			tracked, will then get executed next time PerformCommand is called. So, when we're
			done with HandleMouseDown below, we simply tell the menu manager that no menu item
			was selected, ie newItem = kNoMenuItem. }
			
			PtToVPt(hitPt, hitVPt);						{ hitPt in local coord's }
			CreateMouseDownEventInfo(hitPt, anEventRecord, info);
			hysteresis := gStdHysteresis;
			IF HandleMouseDown(hitVPt, info, hysteresis, theCommand) & (theCommand <> NIL) THEN
				PostCommand(theCommand);
			END
		ELSE
			BEGIN							{ determine if we should track a tear off image }
			LocalToGlobal(hitPt);
			IF PtInRgn(hitPt, GetTearOffTrackingRgn(menuRect)) THEN
				BEGIN
				fTearOffTracker.fInitialPt.h := hitPt.h;
				fTearOffTracker.fInitialPt.v := hitPt.v;
				PostCommand(fTearOffTracker);
				END;
			END;
		END
	ELSE
		UpdateHighlight(whichItem, newItem);			{ Update highlighting }

	gTrackingInMenu := saveTrackingInMenu;

	{ Tell MenuManager about new item }
	whichItem := newItem;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TTearOffMenu.GetTearOffTrackingRgn(menuRect: Rect): RgnHandle;
{ This function returns fTearOffTrackingRgn }

	VAR
		rgnRect: Rect;

	BEGIN
	IF EmptyRgn(fTearOffTrackingRgn) THEN
		BEGIN
		{ calculate the tracking region for the tearoff image }
		{$Push} {$H-}
		WITH menuRect DO
			SetRect(rgnRect, left - kTearOffMargin, top, right + kTearOffMargin, bottom + kTearOffMargin);
		{$Pop}
		{$IFC qDebug}
		UseTempRgn('TTearOffMenu.HandleDefproc');
		{$ENDC}
		RectRgn(gTempRgn, rgnRect);
		DiffRgn(GetGrayRgn, gTempRgn, fTearOffTrackingRgn);
		{$IFC qDebug}
		DoneWithTempRgn;
		{$ENDC}
		InsetRgn(fTearOffTrackingRgn, 4, 4);
		END;
	GetTearOffTrackingRgn := fTearOffTrackingRgn;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TTearOffMenu.Fields(PROCEDURE DoToField(fieldName: Str255;
														   fieldAddr: Ptr;
														   fieldType: INTEGER)); OVERRIDE;

	BEGIN
	DoToField('TTearOffMenu', NIL, bClass);
	INHERITED Fields(DoToField);
	END;
