{	This file has been processed by The THINK Pascal Source Converter, v1.03.	}

{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n-]}
{ UFloatWindow.p }
{ Copyright  1989-1990 by Apple Computer Inc. All rights reserved. }

{[f-]}
(*}
{				T H E O R Y   O F   O P E R A T I O N}
{}
{UFloatWindow a number of services to support floating windows, including the TFloatWindow base class}
{which provides floating window capabilities to MacApp windows. Floating windows are distinguished}
{from other windows in the application heap by the value of the windowKind field of the windowPeek}
{record. The windowKind values 128-999 are reserved by MacApp. For now, windowKind values of 128}
{are used to indicate that a window is a floating window.}
{}
{To use this unit, call InitUFloatWindow from your main program (i.e. M<your>Application.p)}
{after calling InitUMacApp. This routine ensures that the class TFloatWindow isn't dead-stripped}
{by the linker, sets gUFloatWindowInitialized to TRUE, and installs patches to the Window Manager}
{which provide floating window services. N.B.: The MacApp team wishes to thank the HyperCard team for}
{the use of their window manager patches to support floating window operation.}
{}
{We don't need to ensure that the window manager patches are removed when the application terminates}
{because we use the UPatch mechanism to patch the traps and on the way out, CleanUpMacApp calls}
{UnpatchAll which unpatches all traps patched with UPatch. This is important in non-MultiFinder worlds}
{(MultiFinder automatically removes the application patches in MF worlds when the application either}
{quits or is switched out in a context switch).}
{}
{To create a floating window:}
{ For template views, it's business as usual: specify the name of the window class, "TFloatWindow",}
{in the window's view description resource and call NewTemplateWindow as usual.}
{ For non-template views, call this unit's NewFloatPaletteWindow, instead of NewPaletteWindow; call}
{this unit's NewFloatSimpleWindow, instead of NewSimpleWindow; and call this unit's NewFloatTWindow,}
{instead of NewTWindow.}
{}
{*)
{[f+]}

unit UFloatWindow;

interface

	uses
		SysEqu, Traps, ULoMem, UMacAppUtilities, UPatch, UObject, UViewCoords, UMemory, UFailure, UMenuSetup, UList,
{  MacApp}
		UMacApp,
{  Building blocks }
		UMenu,
{  Implementation uses }
		Palettes;

	const
		CurDeKind = $A22;

	type

		TFloatWindow = object(TWindow)

				procedure TFloatWindow.IFloatWindow (itsDocument: TDocument; itsWMgrWindow: WindowPtr; canResize, canClose, disposeOnFree: BOOLEAN);

				procedure TFloatWindow.IRes (itsDocument: TDocument; itsSuperView: TView; var itsParams: Ptr);
				OVERRIDE;

				function TFloatWindow.GetBehindWindowPtr: WindowPtr;
				OVERRIDE;

				procedure TFloatWindow.Activate (entering: BOOLEAN);
				OVERRIDE;
			end;

		{ === Unit initialization === }
	procedure InitUFloatWindow;

		{ === Miscellaneous public utility routines === }
	procedure MAClipAbove (window: WindowPtr);

	function FrontDocumentWindow: WindowPtr;

	procedure SendSystemWindowsToBack;

		{ === public global variables === }
	var
		gUFloatWindowInitialized: BOOLEAN;

implementation


{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n-]}
{ UFloatWindow.inc1.p }
{ Copyright  1987-1990 by Apple Computer Inc.	All rights reserved. }

	const
		kWindoidWDEF = 16 * 3;						{ !!! our default windoid WDEF }

	type
		WindowPtrPtr = ^WindowPtr;					{ used to access low memory globals }

	var
	{ window mgr patch records }
		pBringToFrontPatch: TrapPatch;
		pDragWindowPatch: TrapPatch;
		pSelectWindowPatch: TrapPatch;
		pMoveWindowPatch: TrapPatch;
		pZoomWindowPatch: TrapPatch;
		pShowWindowPatch: TrapPatch;
		pShowHidePatch: TrapPatch;
		pHiliteWindowPatch: TrapPatch;

	{ GNE patch record to manage window mgr low memory globals }
		pGetNextEventPatch: TrapPatch;

	{ private globals needed by window mgr patches }
		pHasLayerMgr: BOOLEAN;						{ flag tells us if the Layer Mgr is in }
		pDuringSelectWindow: BOOLEAN;						{ flag to correctly handle re-entrancy}
{														 problems in SelectWindow}

{--------------------------------------------------------------------------------------------------}

{ The Layer Mgr included as of System 7.0 and AUX 2.0 presents problems with the Window Mgr patches}
{we've installed. Specifically, the problems are that when switching in an application that has}
{been switched out under MultiFinder, the application's patches are re-installed prior to the}
{application's layer being pulled to front. So, the patches are re-installed and then MF calls}
{SelectWindow on the application's layer. However, our patches get control and they need to know}
{if the windowPtr that SelectWindow has been handed is a real window or a layer. The only way to}
{know is to make a call to the Layer Mgr. However, the Layer Mgr interfaces aren't public, so we}
{have to make our own interface to the one Layer Mgr call that we need: IsLayer. This problem _may_}
{be fixed if MultiFinder is modified so that it pulls the application's layer to the front}
{*before* re-installing the application's patches.}

	function MAIsLayer (window: WindowPtr): BOOLEAN;
	inline
		$7002, $A829;

{--------------------------------------------------------------------------------------------------}
{$S AInit}

{ FORWARD declarations for patches that are applied in InitUFloatWindow }

	procedure MABringToFront (window: WindowPtr);
	FORWARD;

	procedure MADragWindow (window: WindowPtr; startPt: Point; bounds: Rect);
	FORWARD;

	procedure MASelectWindow (window: WindowPtr);
	FORWARD;

	procedure MAMoveWindow (window: WindowPtr; hGlobal: INTEGER; vGlobal: INTEGER; front: BOOLEAN);
	FORWARD;

	procedure MAZoomWindow (theWindow: WindowPtr; partCode: INTEGER; front: BOOLEAN);
	FORWARD;

	procedure MAShowWindow (theWindow: WindowPtr);
	FORWARD;

	procedure MAShowHide (theWindow: WindowPtr; showFlag: BOOLEAN);
	FORWARD;

	procedure MAHiliteWindow (theWindow: WindowPtr; fHilite: BOOLEAN);
	FORWARD;

	function MAGetNextEvent (eventMask: INTEGER; var theEvent: EventRecord): BOOLEAN;
	FORWARD;

	procedure InitUFloatWindow;

	begin
		if qTemplateViews & gDeadStripSuppression then
			if Member(TObject(nil), TFloatWindow) then
				; 	{ So the linker doesn't dead strip these }

	{ install window mgr patches }
		FailOSErr(PatchTrap(pBringToFrontPatch, _BringToFront, @MABringToFront));
		FailOSErr(PatchTrap(pDragWindowPatch, _DragWindow, @MADragWindow));
		FailOSErr(PatchTrap(pSelectWindowPatch, _SelectWindow, @MASelectWindow));
		FailOSErr(PatchTrap(pMoveWindowPatch, _MoveWindow, @MAMoveWindow));
		FailOSErr(PatchTrap(pZoomWindowPatch, _ZoomWindow, @MAZoomWindow));
		FailOSErr(PatchTrap(pHiliteWindowPatch, _HiliteWindow, @MAHiliteWindow));
		FailOSErr(PatchTrap(pShowWindowPatch, _ShowWindow, @MAShowWindow));
		FailOSErr(PatchTrap(pShowHidePatch, _ShowHide, @MAShowHide));

	{ tail patch to GNE which redirects activate/deactivate events }
		FailOSErr(PatchTrap(pGetNextEventPatch, _GetNextEvent, @MAGetNextEvent));

		pHasLayerMgr := TrapExists($A829);
		pDuringSelectWindow := FALSE;
		gUFloatWindowInitialized := TRUE;
	end;

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{$D-}
{$R-}
{$V-}
{$S MAWindowRes}

	procedure FixedSendBehind (window, behind: WindowPtr; doActivatePalette: Boolean);
		var
			windPeek: WindowPeek;
			savePort: GrafPtr;
			clobbered: RgnHandle;
			rgnTopLeft: Point;
			activeWindow: WindowPtr;
	begin
	{ set up }
		windPeek := WindowPeek(window);
		GetPort(savePort);
		SetPort(window);

	{ create a rgn so that we can figure out what rgn needs to be recalculated }
		clobbered := NewRgn;						{ don't call MakeNewRgn from patch }
		CopyRgn(window^.visRgn, clobbered);

	{ convert the rgn into globals }
		rgnTopLeft := clobbered^^.rgnBBox.topLeft;
		LocalToGlobal(rgnTopLeft);
		with clobbered^^.rgnBBox, rgnTopLeft do
			OffsetRgn(clobbered, h - left, v - top);

	{ put that window where it belongs and activate its palette if requested }
		SendBehind(window, behind);
		if gConfiguration.hasColorQD & doActivatePalette then
			ActivatePalette(window);

	{ figure out what needs to be painted, paint it, and recalculate vis rgns }
		DiffRgn(windPeek^.strucRgn, clobbered, clobbered);
		PaintOne(windPeek, clobbered);
		CalcVisBehind(windPeek, clobbered);		{windPeek^.strucRgn}

	{ clean up }
		DisposeRgn(clobbered);
		SetPort(savePort);
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{$D-}
{$R-}
{$V-}
{$S MAWindowRes}

	procedure CallBringToFront (window: WindowPtr; oldTrap: LongInt);
	inline
		$205F, $4E90;

	procedure MABringToFront (window: WindowPtr);

{ Patch for BringToFront that preserves window layering:}
{	 for floaters, leave the routine & call the real BringToFront;}
{	 for dialogs, leave the routine & call the real BringToFront;}
{	 if no floaters, leave the routine & call the real BringToFront;}
{	 otherwise, call SendBehind }

		var
			oldA5: LongInt;
			lastFloatingWindowPtr: WindowPtr;
			savePort: GrafPtr;
			windPeek: WindowPeek;
			clobbered: RgnHandle;
			rgnTopLeft: Point;
			activeWindow: WindowPtr;

	begin
		oldA5 := SetCurrentA5;
		windPeek := WindowPeek(window);

	{  Floaters: call the real BringToFront;}
		if IsFloatWindow(window) then
			begin
				if not gInBackground & (window <> FrontWindow) then
					CallBringToFront(window, pBringToFrontPatch.oldTrapAddr);
			end

	{  Layers or System Windows: call the real BringToFront;}
		else if (pHasLayerMgr & MAIsLayer(window)) | (windPeek^.windowKind < 0) then
			CallBringToFront(window, pBringToFrontPatch.oldTrapAddr)

	{  Dialogs: call the real BringToFront and then de-hilite and deactivate the active window}
		else if IsDialog(window) then
			begin
		{ bring the dialog to the front }
				CallBringToFront(window, pBringToFrontPatch.oldTrapAddr);

		{ de-hilite the active window and deactivate it }
				activeWindow := MAGetActiveWindow;
				if (activeWindow <> nil) & (activeWindow <> window) then
					begin
						HiliteWindow(activeWindow, FALSE);
						WindowPtrPtr(CurDeactive)^ := activeWindow; { set curDeactive }
						if activeWindow <> nil then
							IntegerPtr(CurDeKind)^ := WindowPeek(activeWindow)^.windowKind;
					end;
			end

	{  Document windows }
		else
			begin
		{  Document Windows: if no floaters, call the real BringToFront}
				lastFloatingWindowPtr := GetLastFloatingWindowPtr;
				if (lastFloatingWindowPtr = nil) then
					CallBringToFront(window, pBringToFrontPatch.oldTrapAddr)

		{  Document Windows: if there are floaters, call SendBehind}
				else if (window <> MAFrontWindow) then
					begin
						SendSystemWindowsToBack;  { send system windows to back }
						FixedSendBehind(window, lastFloatingWindowPtr, TRUE);
					end;
			end;
		if SetA5(oldA5) <> 0 then
			;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{$D-}
{$R-}
{$V-}
{$S MAWindowRes}

	procedure MADragWindow (window: WindowPtr; startPt: Point; bounds: Rect);

{ Complete replacement of DragWindow that preserves window layering. }

		const
			kCommandKey = 55;

		var
			oldA5: LongInt;
			wMgrPort: GrafPtr;
			savePort: GrafPtr;
			dragRgn: RgnHandle;
			result: LongInt;
			newLoc: Point;
			portBounds: Rect;
			tempRgn: RgnHandle;
			clipWindow: WindowPtr;
			commandFlag: BOOLEAN;
			theKeys: KeyMap;

	begin
		oldA5 := SetCurrentA5;

	{ set up some parameters }
		GetKeys(theKeys);
		commandFlag := theKeys[kCommandKey];
		if StillDown then
			begin
				GetPort(savePort);
				GetWMgrPort(wMgrPort);
				SetPort(wMgrPort);

				tempRgn := NewRgn;									{ what if NewRgn returns NIL??? }
				GetClip(tempRgn);

	{ drag the gray outline }
				if commandFlag then
					clipWindow := window							{ we won't be activating this window }
				else if IsFloatWindow(window) | (WindowPeek(window)^.windowKind < 0) then
					clipWindow := FrontWindow					{ floaters & DAs get dragged at front }
				else
					begin
						SendSystemWindowsToBack;  { send system windows behind selected window }
						clipWindow := FrontDocumentWindow;
					end;
				MAClipAbove(clipWindow);
				dragRgn := NewRgn;									{ what if NewRgn returns NIL??? }
				CopyRgn(WindowPeek(window)^.strucRgn, dragRgn);
				result := DragGrayRgn(dragRgn, startPt, bounds, bounds, noConstraint, nil);

				SetClip(tempRgn);
				DisposeRgn(tempRgn);
				DisposeRgn(dragRgn);

	{ determine what to do with the window }
				with newLoc do
					begin
						v := HiWrd(result);
						h := LoWrd(result);
						if v = $8000 then
							begin
				{ in certain situations, we sent the system windows to back.}
				{Since we sent the system windows to back, we need to activate the front}
				{window if it is now the front document window. }
								if (not commandFlag) & (not IsFloatWindow(window)) & (WindowPeek(window)^.windowKind >= 0) & (MAFrontWindow = FrontDocumentWindow) then
									SelectWindow(window);
							end
						else
							begin
								if BAND(CGrafPtr(window)^.portVersion, $C000) <> 0 then
									portBounds := CGrafPtr(window)^.portPixMap^^.bounds
								else
									portBounds := window^.portBits.bounds;
								v := v + window^.portRect.top - portBounds.top;
								h := h + window^.portRect.left - portBounds.left;
								MoveWindow(window, h, v, FALSE);
								if not commandFlag then
									SelectWindow(window);
							end;
					end;

	{ restore various values }
				SetPort(savePort);
			end;

		if SetA5(oldA5) <> 0 then
			;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
{$IFC qTrace}
{$D+}
{$ENDC}
{$R-}
{$V-}
{$S MAWindowRes}

	var
		pFrontWindow: WindowPtr;			{ stores the reference to the window that the MAFrontWindow patch should return }
		pFrontWindowPatch: TrapPatch;

	function FrontWindowPatch: WindowPtr;		{ patch to FrontWindow that is active while calling ActivatePalette }

	begin
		FrontWindowPatch := pFrontWindow;
	end;

	procedure MAActivatePalette (window: WindowPtr);
{ ensures that the call to ActivatePalette on "window" actually takes effect }

		var
			needsFrontWindowPatch: BOOLEAN;

	begin
		needsFrontWindowPatch := FrontWindow <> window;
		if needsFrontWindowPatch then		{ patch FrontWindow so that it returns the "window" }
			FailOSErr(PatchTrap(pFrontWindowPatch, _FrontWindow, @FrontWindowPatch));
		ActivatePalette(window);
		if needsFrontWindowPatch then
			UnpatchTrap(pFrontWindowPatch);
	end;

{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{$D-}
{$R-}
{$V-}
{$S MAWindowRes}

	procedure CallSelectWindow (window: WindowPtr; oldTrap: LongInt);
	inline
		$205F, $4E90;

	procedure MASelectWindow (window: WindowPtr);
{ Patch for SelectWindow that preserves window layering. }

		var
			oldA5: LongInt;
			savePort: GrafPtr;
			activeWindow: WindowPtr;

	begin
		oldA5 := SetCurrentA5;
		if not pDuringSelectWindow then
			begin
				if pHasLayerMgr & MAIsLayer(window) then
					CallSelectWindow(window, pSelectWindowPatch.oldTrapAddr)
				else
					begin

						GetPort(savePort);

						if IsFloatWindow(window) then
							begin
								BringToFront(window);

				{ for color environments, set the floater's palette to be the active}
{				palette iff the active window has no palette }
								if gConfiguration.hasColorQD then
									begin
										activeWindow := MAGetActiveWindow;
										if (GetPalette(activeWindow) = nil) then
											ActivatePalette(window);
									end
							end
						else
							begin
				{ unhilite and deactivate the active document window }
								activeWindow := MAGetActiveWindow;
								if (activeWindow <> nil) & (activeWindow <> window) then
									begin
										HiliteWindow(activeWindow, FALSE);
										WindowPtrPtr(CurDeactive)^ := activeWindow; { set curDeactive }
										if activeWindow <> nil then
											IntegerPtr(CurDeKind)^ := WindowPeek(activeWindow)^.windowKind;
									end;

				{ move the window to the correct position, hilite, and activate it }
								pDuringSelectWindow := TRUE;
								BringToFront(window);
								pDuringSelectWindow := FALSE;
								HiliteWindow(window, TRUE);
								if (WindowPeek(window)^.visible) then
									WindowPtrPtr(CurActivate)^ := window;	{ set curActivate }

								if gConfiguration.hasColorQD then
									MAActivatePalette(window);
							end;
						SetPort(savePort);
					end;
			end;
		if SetA5(oldA5) <> 0 then
			;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{$D-}
{$R-}
{$V-}
{$S MAWindowRes}

	procedure CallMoveWindow (window: WindowPtr; hGlobal, vGlobal: INTEGER; front: BOOLEAN; oldTrap: LongInt);
	inline
		$205F, $4E90;

	procedure MAMoveWindow (window: WindowPtr; hGlobal: INTEGER; vGlobal: INTEGER; front: BOOLEAN);
{ Tail Patch for MoveWindow that preserves window layering.}
{	We do this b/c MoveWindow doesn't call SelectWindow trap, instead jumps directly there}

		var
			oldA5: LongInt;

	begin
		oldA5 := SetCurrentA5;

	{ force FALSE for "front" parameter }
		CallMoveWindow(window, hGlobal, vGlobal, FALSE, pMoveWindowPatch.oldTrapAddr);

		if front then
			SelectWindow(window);
		if SetA5(oldA5) <> 0 then
			;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{$D-}
{$R-}
{$V-}
{$S MAWindowRes}

	procedure CallZoomWindow (theWindow: WindowPtr; partCode: INTEGER; front: BOOLEAN; oldTrap: LongInt);
	inline
		$205F, $4E90;

	procedure MAZoomWindow (theWindow: WindowPtr; partCode: INTEGER; front: BOOLEAN);

		var
			oldA5: LongInt;

	begin
		oldA5 := SetCurrentA5;

	{ force FALSE for "front" parameter }
		CallZoomWindow(theWindow, partCode, FALSE, pZoomWindowPatch.oldTrapAddr);

		if front then
			SelectWindow(theWindow);
		if SetA5(oldA5) <> 0 then
			;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{$D-}
{$R-}
{$V-}
{$S MAWindowRes}

	procedure CallHiliteWindow (theWindow: WindowPtr; fHilite: BOOLEAN; oldTrap: LongInt);
	inline
		$205F, $4E90;

	procedure MAHiliteWindow (theWindow: WindowPtr; fHilite: BOOLEAN);

{ patch to HiliteWindow which ensures that the correct window gets hilited/dehilited }
{ called from NewDialog/CloseDialog }

		var
			oldA5: LongInt;
			toHilite: WindowPtr;

	begin
		oldA5 := SetCurrentA5;
		if (WindowPeek(theWindow)^.visible) then
			begin
				if not IsFloatWindow(theWindow) then
					CallHiliteWindow(theWindow, fHilite, pHiliteWindowPatch.oldTrapAddr)
				else
					begin
						if fHilite then
							begin
								toHilite := MAFrontWindow;
								WindowPtrPtr(CurActivate)^ := toHilite;		{ set curActivate }
							end
						else
							begin
								toHilite := MAGetActiveWindow;
								WindowPtrPtr(CurDeactive)^ := toHilite;		{ set CurDeactive }
								if toHilite <> nil then
									IntegerPtr(CurDeKind)^ := WindowPeek(theWindow)^.windowKind;
							end;
						if toHilite <> nil then
							CallHiliteWindow(toHilite, fHilite, pHiliteWindowPatch.oldTrapAddr);
					end;
			end;

		if SetA5(oldA5) <> 0 then
			;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{$D-}
{$R-}
{$V-}
{$S MAWindowRes}

	procedure CallShowWindow (theWindow: WindowPtr; oldTrap: LongInt);
	inline
		$205F, $4E90;

	procedure MAShowWindow (theWindow: WindowPtr);

{ tail patch to ShowWindow which ensures that SelectWindow gets called for a shown window }

		var
			oldA5: LongInt;

	begin
		oldA5 := SetCurrentA5;
		if not WindowPeek(theWindow)^.visible then
			begin
				CallShowWindow(theWindow, pShowWindowPatch.oldTrapAddr);
				if (MAFrontWindow = theWindow) then
					SelectWindow(theWindow);
			end;
		if SetA5(oldA5) <> 0 then
			;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{$D-}
{$R-}
{$V-}
{$S MAWindowRes}

	procedure CallShowHide (theWindow: WindowPtr; showFlag: BOOLEAN; oldTrap: LongInt);
	inline
		$205F, $4E90;

	procedure MAShowHide (theWindow: WindowPtr; showFlag: BOOLEAN);

{ This patch manages the global pActiveWindow, and activates the next document window in the case}
{where we're hiding the currently active document window and there are floaters present.}
{}
{  Note that this patch to ShowHide actually gets called from 5 other traps, since:}
{}
{		HideWindow calls ShowHide}
{		CloseWindow calls ShowHide}
{		DisposWindow calls CloseWindow}
{		CloseDialog calls CloseWindow}
{		DisposDialog calls CloseDialog}
{}
{  So, ShowHide is the routine through which all other Window Mgr routines eventually bottleneck to}
{actually hide the window.}

		var
			oldA5: LongInt;
			needActivate: BOOLEAN;
			toActivate: WindowPtr;

	begin
		oldA5 := SetCurrentA5;

	{ if we're hiding a front document window that's not the actual front window (i.e. it's in back}
{	of a floater) then we need to select the next document window}
		needActivate := (not showFlag) & (theWindow = MAFrontWindow) & (FrontWindow <> theWindow);

		CallShowHide(theWindow, showFlag, pShowHidePatch.oldTrapAddr);

	{ activate the next window if necessary }
		if needActivate then
			begin
				toActivate := MAFrontWindow;
				if toActivate <> nil then
					begin											{ instead, call SelectWindow ??? }
						WindowPtrPtr(CurActivate)^ := toActivate;		{ set curActivate }
						HiliteWindow(toActivate, TRUE);
					end;
			end;

		if SetA5(oldA5) <> 0 then
			;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{$D-}
{$R-}
{$V-}
{$S MAWindowRes}

	function CallGetNextEvent (eventMask: INTEGER; var theEvent: EventRecord; oldTrap: LongInt): BOOLEAN;
	inline
		$205F, $4E90;

	function MAGetNextEvent (eventMask: INTEGER; var theEvent: EventRecord): BOOLEAN;

{ tail patch to GetNextEvent that:}
{	 redirects activate events from floating windows to front doc window}
{	 redirects deactivate events from floating windows to front doc window}
		const
			osEvt = app4Evt;
			osEvtMessageMask = $FF000000;
			suspendResumeMessage = $01;

		var
			oldA5: LongInt;
			myToActivate: WindowPtr;
			myToDeactivate: WindowPtr;

	begin
		oldA5 := SetCurrentA5;

		MAGetNextEvent := CallGetNextEvent(eventMask, theEvent, pGetNextEventPatch.oldTrapAddr);

		with theEvent do
			begin
				case what of
					activateEvt: 
						begin
							if Odd(modifiers) then					{  activate event }
								begin
				{ redirect all activate calls from float windows to the front, non-floater window }
									if IsFloatWindow(WindowPtr(message)) then
										begin
											myToActivate := MAFrontWindow; { the front, non-floater window }
											if myToActivate = nil then
												what := nullEvent					{ then zap the activate event! }
											else
												begin
													HiliteWindow(myToActivate, TRUE); { restore hiliting }
													message := ORD(myToActivate);	{ redirect activate event }
												end
										end
								end 								{ IF Odd(modifiers) THEN }
							else									{  deactivate event }
				{ redirect all deactivate calls from float windows to the active window }
								if IsFloatWindow(WindowPtr(message)) then { if deactivate evt is for floater }
									what := nullEvent;					{ then zap the deactivate event! }
						end;									{ IF what = activateEvt THEN }
					osEvt: 
						if (BSR(BAND(message, osEvtMessageMask), 24) = suspendResumeMessage) then
							begin
								myToActivate := MAFrontWindow; { the front, non-floater window }
								if myToActivate <> nil then
									HiliteWindow(myToActivate, Odd(message)); { switching in/out }
							end;
				end;											{ CASE what OF }
			end;											{ WITH theEvent DO }

		if SetA5(oldA5) <> 0 then
			;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$S MAOpen}

	procedure TFloatWindow.IFloatWindow (itsDocument: TDocument; itsWMgrWindow: WindowPtr; canResize, canClose, disposeOnFree: BOOLEAN);

		var
			saveStdWTopleft: Point;

	begin
		saveStdWTopleft := gStdWSizeRect.topLeft;
		gStdWSizeRect.topLeft := gZeroPt;
		IWindow(itsDocument, itsWMgrWindow, canResize, canClose, disposeOnFree);
		gStdWSizeRect.topLeft := saveStdWTopleft;

		fHideOnSuspend := TRUE;
		fDoFirstClick := TRUE;
		fFloats := TRUE;
		WindowPeek(itsWMgrWindow)^.windowKind := kFloatWindowKind; { mark it as a floating window }
	end;

{--------------------------------------------------------------------------------------------------}
{$S MAOpen}

	procedure TFloatWindow.IRes (itsDocument: TDocument; itsSuperView: TView; var itsParams: Ptr);

		var
			saveStdWTopleft: Point;

	begin
		saveStdWTopleft := gStdWSizeRect.topLeft;
		gStdWSizeRect.topLeft := gZeroPt;
		inherited IRes(itsDocument, itsSuperView, itsParams);
		gStdWSizeRect.topLeft := saveStdWTopleft;

		fHideOnSuspend := TRUE;
		fDoFirstClick := TRUE;
		fFloats := TRUE;
		WindowPeek(fWMgrWindow)^.windowKind := kFloatWindowKind; { mark it as a floating window }
	end;

{--------------------------------------------------------------------------------------------------}
{$S MAOpen}

	function TFloatWindow.GetBehindWindowPtr: WindowPtr;

	begin
		GetBehindWindowPtr := WindowPtr(-1);				{ bring the float window up in front of all}
{														 other windows}
	end;

{--------------------------------------------------------------------------------------------------}
{$S MAActivate}

	procedure TFloatWindow.Activate (entering: BOOLEAN);
		OVERRIDE;

		var
			different: BOOLEAN;

	begin
		InvalidateFocus;									{ Must refocus to make sure thePort is set }
		different := entering <> fIsActive;

		if different then
			begin
				Update; 										{ Logically the order is wrong, but it gives}
{														 the correct visual effect. ??? Revisit?}
				inherited Activate(entering);

				if entering then
					fIsActive := entering;
			end;

		if fIsResizable & Focus & IsVisible then
			DrawResizeIcon;
	end;

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{$D-}
{$R-}
{$V-}
 					{ called from patches }
{$S MAWindowRes}

	procedure MAClipAbove (window: WindowPtr);
{ fixes ROM anomaly in which ClipAbove clips to the current clip rgn.}
{	we assume the current port is the window mgr port}

	begin
		SetClip(GetGrayRgn);
		ClipAbove(WindowPeek(window));
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{$D-}
{$R-}
{$V-}
 					{ called from patches }
{$S MAWindowRes}

	function FrontDocumentWindow: WindowPtr;

{ returns the first visible document window in the window list }

		var
			aWindowPtr: WindowPtr;
			wPeek: WindowPeek;

	begin
		aWindowPtr := GetWindowList;
		while (aWindowPtr <> nil) do
			begin
				wPeek := WindowPeek(aWindowPtr);
				if (wPeek^.visible) & IsDocumentWindow(aWindowPtr) then
					LEAVE;
				aWindowPtr := WindowPtr(wPeek^.nextWindow);
			end;
		FrontDocumentWindow := aWindowPtr;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{$D-}
{$R-}
{$V-}
{$S MAWindowRes}

	procedure DeactivateSystemWindow (window: WindowPtr);
	{ dehilite & deactivate hilited system windows }

		var
			cacheActivate: WindowPtr;
			theEvent: EventRecord;

	begin
		if (window <> nil) & (WindowPeek(window)^.windowKind < 0) & (WindowPeek(window)^.hilited) then
			begin
		{ cache the CurActivate so that EventAvail gives us the Deactivate event }
				cacheActivate := WindowPtrPtr(CurActivate)^;
				WindowPtrPtr(CurActivate)^ := nil;

		{ de-hilite the system window }
				HiliteWindow(window, FALSE);

		{ process the deactivate event }
				WindowPtrPtr(CurDeactive)^ := window; { set curDeactive }
				IntegerPtr(CurDeKind)^ := WindowPeek(window)^.windowKind;
				if EventAvail(activMask, theEvent) then	{ gimme that deactivate! }
					with theEvent do
						if (what = activateEvt) & not odd(modifiers) & (WindowPtr(message) = window) & SystemEvent(theEvent) then
							WindowPtrPtr(CurDeactive)^ := nil; { zap it, it got handled }

		{ restore CurActivate }
				WindowPtrPtr(CurActivate)^ := cacheActivate;
			end;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{$D-}
{$R-}
{$V-}
{$S MAWindowRes}

	procedure SendSystemWindowsToBack;
	{ send system windows behind all windows }

		var
			nextWindow: WindowPeek;
			thisWindow: WindowPeek;
			lastWindow: WindowPeek;

	begin
	{ find out which non-system window is the last window }
		thisWindow := WindowPeek(FrontWindow);
		lastWindow := thisWindow;
		while (thisWindow <> nil) do
			begin
				if (thisWindow^.windowKind >= 0) & (thisWindow^.windowKind < 32700) then
					lastWindow := thisWindow;
				thisWindow := thisWindow^.nextWindow;
			end;

	{ iterate until no more windows or we've reached the last window }
		thisWindow := WindowPeek(FrontWindow);
		while (thisWindow <> nil) & (thisWindow <> lastWindow) do
			begin
				nextWindow := thisWindow^.nextWindow;	{ what is it *NOW* before we munge the window list }
				if thisWindow^.windowKind < 0 then	{ if it's a System window }
					begin
						FixedSendBehind(WindowPtr(thisWindow), nil, FALSE);			{ send thisWindow to back }
						DeactivateSystemWindow(WindowPtr(thisWindow));
					end;
				thisWindow := nextWindow;
			end;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}

end.