{	This file has been processed by The THINK Pascal Source Converter, v1.03.	}

{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n-]}
{ UTearOffMenu.p }
{ Copyright  1989-1990 by Apple Computer Inc. All rights reserved. }

{[f-]}
(*}
{*)
{[f+]}

unit UTearOffMenu;

interface

	uses
		SysEqu, Traps, ULoMem, UMacAppUtilities, UPatch, UObject, UViewCoords, UMemory, UFailure, UMenuSetup, UList,
{  MacApp}
		UMacApp,
{  Building blocks }
		UMenu;
{  Implementation uses }

	const
		kTearOffMargin = 10;	{ the slop around the displayed menu within which we don't tear-off;}
{									see Apple Human Interface Guidelines p. 91;}
{									note: some programs (eg. HyperCard) use a 15 pixel slop value}
		kTearOffOffset = 3;		{ pixel offset from pointer to gray rgn being dragged around }

	type

		TShowTearOffWindowCmd = object(TNoChangesCommand)
			{ moves, shows and selects a TTearOffMenu }

				fTearOffWindow: TWindow;

				fWhere: Point;

				procedure TShowTearOffWindowCmd.IShowTearOffWindowCmd (tearOffWindow: TWindow; where: Point);

				procedure TShowTearOffWindowCmd.DoIt;
				OVERRIDE;

				procedure TShowTearOffWindowCmd.Fields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: INTEGER));
				OVERRIDE;
			end;

		TTearOffTracker = object(TCommand)
			{ abstract superclass providing basic tear-off menu tracking: tracks the grey outline of}
{			the menu around the screen}

				fTearOffMenu: TTearOffMenu;

				fTearOffWindowOutline: RgnHandle;

				fExitTracking: BOOLEAN;

				fOrigin: Point;

				procedure TTearOffTracker.ITearOffTracker (hitPt: Point; tearOffMenu: TTearOffMenu);

				procedure TTearOffTracker.Free;
				OVERRIDE;

				procedure TTearOffTracker.DoIt;
				OVERRIDE;

				procedure TTearOffTracker.TrackFeedback (anchorPoint, nextPoint: VPoint; turnItOn, mouseDidMove: BOOLEAN);
				OVERRIDE;

				function TTearOffTracker.TrackMouse (aTrackPhase: TrackPhase; var anchorPoint, previousPoint, nextPoint: VPoint; mouseDidMove: BOOLEAN): TCommand;
				OVERRIDE;

				function TTearOffTracker.IsDoneTracking: BOOLEAN;
				OVERRIDE;

				procedure TTearOffTracker.Fields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: INTEGER));
				OVERRIDE;

			end;

		TTearOffMenu = object(TMenu)
			{ abstract superclass providing basic tear-off menu behavior }

				fTearOffTrackingRgn: RgnHandle;

				fTearOffTracker: TTearOffTracker;

				fTearOffWindow: TWindow;

				procedure TTearOffMenu.ITearOffMenu (rsrcID: INTEGER; menuWidth, menuHeight: INTEGER; tearOffWindow: TWindow);

				procedure TTearOffMenu.InstallTearOffTracker;
				{ For the sake of efficiency when tracking a tear-off, we'll create the tear-off tracker and}
{				 keep it with us, so we don't need to allocate a new tracker every time we tear off a menu. If you}
{				 want a different tracker in your subclass of TTearOffMenu, then override this method.}

				procedure TTearOffMenu.Free;
				OVERRIDE;

				procedure TTearOffMenu.HandleChooseMessage (message: INTEGER; theMenu: MenuHandle; var menuRect: Rect; hitPt: Point; var whichItem: INTEGER);
				OVERRIDE;
				{ initiates tracking if mouse is in fTearOffTrackingRgn }

				function TTearOffMenu.GetTearOffTrackingRgn (menuRect: Rect): RgnHandle;
				{ accessor for fTearOffTrackingRgn: a rgn that represents the desktop less the menu rect (with some buffer area}
{				around the menu rect). This is useful for testing whether tracking should occur because the mouse}
{				just might be in the menu bar (which isn't in the desktop & isn't in the menu rect) and so tracking}
{				should not occur while the mouse is in the menu bar (MenuSelect does that tracking just fine).}

				procedure TTearOffMenu.Fields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: INTEGER));
				OVERRIDE;
			end;

implementation

{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n-]}

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

	procedure TShowTearOffWindowCmd.IShowTearOffWindowCmd (tearOffWindow: TWindow; where: Point);

	begin
		INoChangesCommand(0, nil, nil, nil);
		fTearOffWindow := tearOffWindow;
		fWhere := where;
	end;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

	procedure TShowTearOffWindowCmd.DoIt;
		OVERRIDE;

	begin
		if fTearOffWindow <> nil then
			begin
				fTearOffWindow.Locate(fWhere.h, fWhere.v, kDontInvalidate);	{ move it to its new home }
				fTearOffWindow.fMustForceOnScreen := TRUE;		{ so the Open will call ForceOnScreen }
				fTearOffWindow.fForcedOnScreen := FALSE;
				fTearOffWindow.Open;							{ show the torn-off window }
				fTearOffWindow.Select;							{ select the torn-off window }
			end;
	end;

{--------------------------------------------------------------------------------------------------}
{$S MAFields}

	procedure TShowTearOffWindowCmd.Fields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: INTEGER));
		OVERRIDE;

	begin
		DoToField('TShowTearOffWindowCmd', nil, bClass);
		inherited Fields(DoToField);
	end;

{--------------------------------------------------------------------------------------------------}
{$S AInit}

	procedure TTearOffTracker.ITearOffTracker (hitPt: Point; tearOffMenu: TTearOffMenu);

		var
			theMouse: VPoint;

	begin
		theMouse.v := hitPt.v;
		theMouse.h := hitPt.h;
		fOrigin := gZeroPt;
		ICommand(0, nil, nil, nil);
		fCanUndo := FALSE;
		fCausesChange := FALSE;
		fTearOffMenu := tearOffMenu;
		fFreeOnCompletion := FALSE;
		fExitTracking := FALSE;
		fTracksMouse := TRUE;

		fTearOffWindowOutline := MakeNewRgn;
	end;

{--------------------------------------------------------------------------------------------------}
{$S AClose}

	procedure TTearOffTracker.Free;
		OVERRIDE;

	begin
		if fTearOffWindowOutline <> nil then
			begin
				DisposeRgn(fTearOffWindowOutline);
				fTearOffWindowOutline := nil;
			end;

		inherited Free;
	end;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

	procedure TTearOffTracker.DoIt;

		var
			where: Point;
			aTearOffWindow: TWindow;
			aShowTearOffWindowCmd: TShowTearOffWindowCmd;

	begin
		if not fExitTracking then		{ ensure that we didn't exit the tracking loop }
			begin
				aTearOffWindow := fTearOffMenu.fTearOffWindow;

				where.v := fTearOffWindowOutline^^.rgnBBox.top + aTearOffWindow.fContRgnInset.v;
				where.h := fTearOffWindowOutline^^.rgnBBox.left + aTearOffWindow.fContRgnInset.h;

				if aTearOffWindow <> nil then
					begin
			{ NOTE: we can't just move and show the tear off window here because the menu that}
{			we want to tearx off is currently displayed on the screen and the menu manager has}
{			cached the bits underneath the currently displayed menu and we might be moving our}
{			floating window into that territory thus invalidating the menu manager's cache (which}
{			the menu manager will blit back in place momentarily). To work-around this "feature"}
{			we'll simply post a command which will do the work later. Isn't MacApp wonderful!}
						New(aShowTearOffWindowCmd);
						FailNIL(aShowTearOffWindowCmd);
						aShowTearOffWindowCmd.IShowTearOffWindowCmd(aTearOffWindow, where);
						gApplication.PostCommand(aShowTearOffWindowCmd);
					end;

			end;
	end;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

	procedure TTearOffTracker.TrackFeedback (anchorPoint, nextPoint: VPoint; turnItOn, mouseDidMove: BOOLEAN);

		var
			mousePt: Point;

	begin
		if mouseDidMove & not EmptyRgn(fTearOffWindowOutline) then
			begin
				PenPat(gray);
				FrameRgn(fTearOffWindowOutline); 							{ draw/erase }
			end;
	end;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

	function TTearOffTracker.TrackMouse (aTrackPhase: TrackPhase; var anchorPoint, previousPoint, nextPoint: VPoint; mouseDidMove: BOOLEAN): TCommand;

		var
			aTearOffWindow: TWindow;
			verticalOffset: INTEGER;
			horizontalOffset: INTEGER;
			rgnsWereBuilt: Boolean;

	begin
		TrackMouse := SELF;

		fExitTracking := not PtInRgn(VPtToPt(nextPoint), fTearOffMenu.fTearOffTrackingRgn);

		if (aTrackPhase = trackPress) then
			begin
				aTearOffWindow := fTearOffMenu.fTearOffWindow;
				if (aTearOffWindow <> nil) then
					begin
						rgnsWereBuilt := aTearOffWindow.BuildWindowRgns(kBuild);
						CopyRgn(WindowPeek(aTearOffWindow.fWMgrWindow)^.strucRgn, fTearOffWindowOutline);
						if aTearOffWindow.BuildWindowRgns(rgnsWereBuilt) then
							; 	{ discard result }

			{ move the outline to the topleft of the window mgr graf port }
			{$Push}
						with fTearOffWindowOutline^^.rgnBBox do
							OffsetRgn(fTearOffWindowOutline, -left, -top);
			{$Pop}

			{ move the outline so that it's centered at the mouse }
						with fTearOffWindowOutline^^.rgnBBox do
							begin
								verticalOffset := (bottom - top) div 2;
								horizontalOffset := (right - left) div 2;
							end;
						with previousPoint do
							OffsetRgn(fTearOffWindowOutline, h - horizontalOffset, v - verticalOffset);
					end;	{aTearOffWindow <> NIL}
			end
		else
			with previousPoint do
				OffsetRgn(fTearOffWindowOutline, nextPoint.h - h, nextPoint.v - v);
	end;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

	function TTearOffTracker.IsDoneTracking: BOOLEAN;
		OVERRIDE;

	begin
		IsDoneTracking := not StillDown | fExitTracking;
	end;

{--------------------------------------------------------------------------------------------------}
{$S MAFields}

	procedure TTearOffTracker.Fields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: INTEGER));
		OVERRIDE;

	begin
		DoToField('TTearOffTracker', nil, bClass);
		inherited Fields(DoToField);
	end;

{--------------------------------------------------------------------------------------------------}
{$S AInit}

	procedure TTearOffMenu.ITearOffMenu (rsrcID: INTEGER; menuWidth, menuHeight: INTEGER; tearOffWindow: TWindow);

	begin
		fTearOffTrackingRgn := MakeNewRgn;
		fTearOffTracker := nil;
		fTearOffWindow := tearOffWindow;

		IMenu(rsrcID, menuWidth, menuHeight);

		InstallTearOffTracker;
	end;

{--------------------------------------------------------------------------------------------------}
{$S AInit}

	procedure TTearOffMenu.InstallTearOffTracker;
{ For the sake of efficiency when tracking a tear-off, we'll create the tear-off tracker and}
{ keep it with us, so we don't need to allocate a new tracker every time we tear off a menu. If you}
{ want a different tracker in your subclass of TTearOffMenu, then override this method.}

		var
			tearOffTracker: TTearOffTracker;

	begin
		New(tearOffTracker);
		FailNIL(tearOffTracker);
		tearOffTracker.ITearOffTracker(gZeroPt, SELF);
		fTearOffTracker := tearOffTracker;
	end;

{--------------------------------------------------------------------------------------------------}
{$S AClose}

	procedure TTearOffMenu.Free;
		OVERRIDE;

	begin
		if fTearOffTracker <> nil then
			begin
				fTearOffTracker.Free;
				fTearOffTracker := nil;
			end;

		if fTearOffTrackingRgn <> nil then
			begin
				DisposeRgn(fTearOffTrackingRgn);
				fTearOffTrackingRgn := nil;
			end;

		inherited Free;
	end;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

	procedure TTearOffMenu.HandleChooseMessage (message: INTEGER; theMenu: MenuHandle; var menuRect: Rect; hitPt: Point; var whichItem: INTEGER);
		OVERRIDE;

		var
			newItem: INTEGER;
			hitRect: Rect;
			hitVPt: VPoint;
			hysteresis: Point;
			info: EventInfo;
			anEventRecord: EventRecord;
			theCommand: TCommand;
			saveTrackingInMenu: BOOLEAN;

	begin
	{ so that trackers know that they're tracking in menus }
		saveTrackingInMenu := gTrackingInMenu;
		gTrackingInMenu := TRUE;

		newItem := kNoMenuItem; 							{ default return }

		if IsViewEnabled then								{ menu enabled }
			begin
		{ see if point is within hit area }
				GetQDExtent(hitRect);							{ hitRect in local coord's }
				AddPt(fBorder.topLeft, hitRect.topLeft);
				AddPt(fBorder.botRight, hitRect.botRight);
				if PtInRect(hitPt, hitRect) then				{ in menu view (not border) }
					begin
			{ NOTE: Either your subclass of TTearOffMenu should override DoMouseCommand or one of }
{			TTearOffMenu's view's subview's should override DoMouseCommand so that it}
{			creates and posts a TCommand. TTearOffMenu's override of PostCommand will}
{			ensure that the tracker is tracked immediately. The tracker, having been posted and}
{			tracked, will then get executed next time PerformCommand is called. So, when we're}
{			done with HandleMouseDown below, we simply tell the menu manager that no menu item}
{			was selected, ie newItem = kNoMenuItem.}

						PtToVPt(hitPt, hitVPt);						{ hitPt in local coord's }
						CreateMouseDownEventInfo(hitPt, anEventRecord, info);
						hysteresis := gStdHysteresis;
						if HandleMouseDown(hitVPt, info, hysteresis, theCommand) & (theCommand <> nil) then
							PostCommand(theCommand);
					end
				else
					begin							{ determine if we should track a tear off image }
						LocalToGlobal(hitPt);
						if PtInRgn(hitPt, GetTearOffTrackingRgn(menuRect)) then
							begin
								fTearOffTracker.fInitialPt.h := hitPt.h;
								fTearOffTracker.fInitialPt.v := hitPt.v;
								PostCommand(fTearOffTracker);
							end;
					end;
			end
		else
			UpdateHighlight(whichItem, newItem);			{ Update highlighting }

		gTrackingInMenu := saveTrackingInMenu;

	{ Tell MenuManager about new item }
		whichItem := newItem;
	end;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

	function TTearOffMenu.GetTearOffTrackingRgn (menuRect: Rect): RgnHandle;
{ This function returns fTearOffTrackingRgn }

		var
			rgnRect: Rect;

	begin
		if EmptyRgn(fTearOffTrackingRgn) then
			begin
		{ calculate the tracking region for the tearoff image }
		{$Push}
				with menuRect do
					SetRect(rgnRect, left - kTearOffMargin, top, right + kTearOffMargin, bottom + kTearOffMargin);
		{$Pop}
		{$IFC qDebug}
				UseTempRgn('TTearOffMenu.HandleDefproc');
		{$ENDC}
				RectRgn(gTempRgn, rgnRect);
				DiffRgn(GetGrayRgn, gTempRgn, fTearOffTrackingRgn);
		{$IFC qDebug}
				DoneWithTempRgn;
		{$ENDC}
				InsetRgn(fTearOffTrackingRgn, 4, 4);
			end;
		GetTearOffTrackingRgn := fTearOffTrackingRgn;
	end;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

	procedure TTearOffMenu.Fields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: INTEGER));
		OVERRIDE;

	begin
		DoToField('TTearOffMenu', nil, bClass);
		inherited Fields(DoToField);
	end;


end.