/********************************************************************************/
/*																				*/
/*	File:			Calc.r		 												*/
/*																				*/
/*	Description:	Resource File for the Calc Sample Program.					*/
/*																				*/
/*																				*/
/*	Copyright  1985 - 1990 by Apple Computer, Inc.  All rights reserved.  		*/
/********************************************************************************/

/*  Auto-Include the requirements for this source */

#ifndef __TYPES.R__
#include "Types.r"
#endif

#ifndef __SYSTYPES.R__
#include "SysTypes.r"
#endif

#ifndef __MacAppTypes__
#include "MacAppTypes.r"
#endif

#ifndef __ViewTypes__
#include "ViewTypes.r"
#endif


#if qDebug
include "Debug.rsrc";
#endif

include "MacApp.rsrc";
include "Printing.rsrc";

include $$Shell("ObjApp")"Calc" 'CODE';

/* Include the separately compiled resources */
include "CalcUtilities.rsrc";
include "GrabberTracker.rsrc";

#define cPrintSelection		180
#define cRecalculate		401
#define cAutoCalc			403
#define cManualCalc			404
#define cGeneral			501
#define cNoDecimal			503
#define cDecimal			504
#define cScientific			505
#define cSystemJustify		506
#define cForceLeftJustify	507
#define cRightJustify		508
#define cCenter				509
#define cSelection			1000
#define cSizeColumn			1001
#define cCutText			1103
#define cCopyText			1104
#define cClearText			1106
#define cCutCells			1203
#define cCopyCells			1204
#define cClearCells			1206
#define cStandardCut		1303
#define cStandardCopy		1304
#define cStandardClear		1306

/* static variables to initialize view templates */
#define	kMaxRows			120				/* Max number of rows */
#define	kMaxColumns			64				/* max number of columns */
#define	kCellWidth			80				/* default width of each cell */
#define	kCellHeight			17				/* default height of each cell */
#define	kRowInset			4				/* pixels separating cell contents from top/bottom edge */
#define	kColumnInset		4				/* pixels separating cell contents from left/right edge */
#define	kRowTitleWidth		32				/* width of row titles */
#define kRowTitleHeight		kCellHeight		/* height of the row titles */
#define	kColumnTitleWidth	kCellWidth		/* width of column titles */
#define	kColumnTitleHeight	20				/* height of column titles */
#define kFontSize			10
#define kMaxChars			255 			/* number of characters in the text edit field */

	
#define kCalcWindowType		1001 			/* ID of the main window or view resource */

#define kSignature			'SS12'			/* Application signature */
#define kFileType			'SF12'			/* Data file type */

#define phSplash			1001			/* ID of splash screen dialog */

/*--------------------------------------------------------------------------------
 MultiFinder/Switcher info
--------------------------------------------------------------------------------*/
resource 'SIZE' (-1) {
	saveScreen,
	acceptSuspendResumeEvents,
	enableOptionSwitch,
	canBackground,
	MultiFinderAware,
	backgroundAndForeground,
	dontGetFrontClicks,
	ignoreChildDiedEvents,
	is32BitCompatible,
	reserved,
	reserved,
	reserved,
	reserved,
	reserved,
	reserved,
	reserved,
#if qDebug
	(720-32) * 1024,
	(620-32) * 1024
#else
	(600-32) * 1024,
	(500-32) * 1024
#endif
};

/*--------------------------------------------------------------------------------
 Memory usage info for MacApp
--------------------------------------------------------------------------------*/
resource 'seg!' (256,
#if qNames
"Calc",
#endif
	purgeable) {
	{	"GTerminate";
		"GNonRes";
		"GOpen";
		"GWriteFile";
		"GFile";
		"GClose";
		"GClipboard";
		"GDoCommand";
		"GSelCommand";
		"ARes"
	}
};


/*--------------------------------------------------------------------------------
 About Box
--------------------------------------------------------------------------------*/
resource 'DITL' (phAboutApp,
#if qNames
"phAboutApp",
#endif
	purgeable) {
	 {	/* array DITLarray: 3 elements */
		/* [1] */
		{160, 182, 180, 262},
		Button {
			enabled,
			"OK"
		};
		/* [2] */
		{10, 75, 150, 316},
		StaticText {
			disabled,
			"This sample program implements a spreadsheet with MacApp."
			"\n\nThis program was written "
			"with MacApp  1985-1990 Apple Computer, Inc."
		};
		/* [3] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			1
		}
	}
};

include "Defaults.rsrc"  'ALRT' (phAboutApp);	// Grab the default about box

/*--------------------------------------------------------------------------------
 Menus!
--------------------------------------------------------------------------------*/
include "Defaults.rsrc" 'cmnu' (mApple);		// Grab the default Apple/File menus

resource 'cmnu' (2) {
	2,
	textMenuProc,
	0x7FFFEEFB,
	enabled,
	"File",
	 {	/* array: 14 elements */
/* [1] */	"New",				noIcon, "N",	noMark, plain, cNew;
/* [2] */	"Open",			noIcon, "O",	noMark, plain, cOpen;
/* [3] */	"-",				noIcon, noKey,	noMark, plain, nocommand;
/* [4] */	"Close",			noIcon, "W",	noMark, plain, cClose;
/* [5] */	"Save",				noIcon, "S",	noMark, plain, cSave;
/* [6] */	"Save As",			noIcon, noKey,	noMark, plain, cSaveAs;
/* [7] */	"Save a Copy In",	noIcon, noKey,	noMark, plain, cSaveCopy;
/* [8] */	"Revert",			noIcon, noKey,	noMark, plain, cRevert;
/* [9] */	"-",				noIcon, noKey,	noMark, plain, nocommand;
/* [10] */	"Page Setup",		noIcon, noKey,	noMark, plain, cPageSetup;
/* [11] */	"Print",			noIcon, "P",	noMark, plain, cPrint;
/* [12] */	"Print Selection",	noIcon, noKey,	noMark, plain, cPrintSelection;
/* [13] */	"-",				noIcon, noKey,	noMark, plain, nocommand;
/* [14] */	"Quit",				noIcon, "Q",	noMark, plain, cQuit
	}
};

include "Defaults.rsrc" 'cmnu' (mEdit);		// Grab the default Edit menu

resource 'cmnu' (4) {
	4,
	textMenuProc,
	0x7FFFFFFF,
	enabled,
	"Format",
	 {	/* array: 8 elements */
/* [1] */	"General",					noIcon, noKey, noMark, plain, cGeneral;
/* [2] */	"No Decimal",				noIcon, noKey, noMark, plain, cNoDecimal;
/* [3] */	"Decimal",					noIcon, noKey, noMark, plain, cDecimal;
/* [4] */	"Scientific",				noIcon, noKey, noMark, plain, cScientific;
/* [5] */	"-",						noIcon, noKey, noMark, plain, nocommand;
/* [6] */	"System Justification",		noIcon, noKey, noMark, plain, cSystemJustify;
/* [7] */	"Force Left Justification",	noIcon, noKey, noMark, plain, cForceLeftJustify;
/* [8] */	"Center Justification",		noIcon, noKey, noMark, plain, cCenter;
/* [9] */	"Right Justification",		noIcon, noKey, noMark, plain, cRightJustify
	  }
	};

resource 'cmnu' (5) {
	5,
	textMenuProc,
	0x7FFFFFFF,
	enabled,
	"Calculation",
	 {	/* array: 4 elements */
/* [1] */	"Recalculate",		noIcon, "R",	noMark,	plain, cRecalculate;
/* [2] */	"-",				noIcon, noKey,	noMark,	plain, nocommand;
/* [3] */	"Automatic Calc",	noIcon, noKey,	check,	plain, cAutoCalc;
/* [4] */	"Manual Calc",		noIcon, noKey,	noMark,	plain, cManualCalc
	}
};

resource 'cmnu' (mBuzzwords) {
	mBuzzwords,
	textMenuProc,
	allEnabled,
	enabled,
	"Buzzwords",
	 {	/* array: 13 elements */
		"Page Setup Change",	noIcon,	noKey,	noMark, plain, cChangePrinterStyle;
		"Typing",				noIcon,	noKey,	noMark,	plain, cTyping;
		"Selection",			noIcon,	noKey,	noMark, plain, cSelection;
		"Column Resize",		noIcon,	noKey,	noMark, plain, cSizeColumn;
		"Cut Text",				noIcon,	"X",	noMark, plain, cCutText;
		"Copy Text",			noIcon,	"C",	noMark, plain, cCopyText;
		"Clear Text",			noIcon,	noKey,	noMark, plain, cClearText;
		"Cut Cells",			noIcon,	"X",	noMark, plain, cCutCells;
		"Copy Cells",			noIcon,	"C",	noMark, plain, cCopyCells;
		"Clear Cells",			noIcon,	noKey,	noMark, plain, cClearCells;
		"Cut",					noIcon,	"X",	noMark, plain, cStandardCut;
		"Copy",					noIcon,	"C",	noMark, plain, cStandardCopy;
		"Clear",				noIcon,	 noKey,	noMark, plain, cStandardClear
	}
};

resource 'MBAR' (kMBarDisplayed,
#if qNames
"Calc",
#endif
	purgeable) {
	{mApple; mFile; mEdit; 4; 5}
};


/*--------------------------------------------------------------------------------
 For building the main view from templates
--------------------------------------------------------------------------------*/
resource 'view' (kCalcWindowType,
#if qNames
"Spreadsheet",
#endif
	purgeable) {
	{
	/* Main (and only) Window */
		root, 'WIND', { 40, 10 }, { 288, 447 }, sizeVariable, sizeVariable, shown, enabled,
		Window		{ "TCalcWindow", zoomDocProc, goAwayBox, resizable,
			modeless, ignoreFirstClick,freeOnClosing, disposeOnFree, closesDocument,
			openWithDocument, dontAdaptToScreen, stagger, forceOnScreen,
			dontCenter, noID, "Spreadsheet <<<>>>" };
			
	/* Coordinate View */
		'WIND', 'CORD',
		{ 0, 8 }, { 32, 48 }, sizeFixed, sizeFixed, shown, enabled,
		View {
			"TCoordView" },
			
	/* Cells View */
		'WIND', 'SCL1',
		{ 52, 32 }, { 221, 400 }, sizeRelSuperView, sizeRelSuperView, shown, enabled,
		Scroller {
			"TPrimaryScroller",
			VertScrollBar, HorzScrollBar,
			0, 0, 0, 0,
			VertConstrain, HorzConstrain,
			{-16, -31, 0, 0 }},

		'SCL1', 'CELL',
		{ 0, 0 }, { 0, 0 }, sizeVariable, sizeVariable, shown, enabled,
		TextGridView {
			"TCellsView",
			kMaxRows, kMaxColumns, kCellHeight, kCellWidth, kRowInset, kColumnInset,
			AdornRows, AdornCols,
			multipleSelection,
			plain,
			kFontSize, { 0x0, 0x0, 0x0 }, "a" },
	/* Columns View */
		'WIND', 'SCL2',
		{ 35, 32 }, { kCellHeight, 400 }, sizeFixed, sizeRelSuperView, shown, enabled,
		Scroller {
			"TSecondaryScroller",
			noVertScrollBar, noHorzScrollBar,
			0, 0, 0, 0,
			noVertConstrain, noHorzConstrain,
			noInset },
		'SCL2', 'COLS',
		{ 0, 0 }, { kCellHeight, 400 }, sizeVariable, sizeVariable, shown, enabled,
		TextGridView {
			"TColumnsView",
			1, kMaxColumns, kColumnTitleHeight, kCellWidth, 0, kColumnInset,
			dontAdornRows, AdornCols,
			multipleSelection,
			bold,
			kFontSize, { 0x0, 0x0, 0x0 }, "a" },
			
	/* Rows View */
		'WIND', 'SCL3',
		{ 52, 0 }, { 221, 32 }, sizeRelSuperView, sizeFixed, shown, enabled,
		Scroller {
			"TSecondaryScroller",
			noVertScrollBar, noHorzScrollBar,
			0, 0, 0, 0,
			noVertConstrain, noHorzConstrain,
			noInset },
		'SCL3', 'ROWS',
		{ 0, 0 }, { 237, 32 }, sizeVariable, sizeVariable, shown, enabled,
		TextGridView {
			"TRowsView",
			kMaxRows, 1, kCellHeight, kRowTitleWidth, kRowInset, 0,
			AdornRows, dontAdornCols,
			multipleSelection,
			bold,
			kFontSize, { 0x0, 0x0, 0x0 }, "a" },
			
	/* Cell Entry View */
		'WIND', 'ENTV',
		{ 8, 64 }, { 20, 364 }, sizeFixed, sizeFixed, shown, enabled,
		TEView {
			"TEntryView",
			withoutStyle, crOnly, acceptChanges, dontFreeText, cTyping,
			kMaxChars, {2, 2, 2, 2}, justSystem, plain,
			kFontSize, { 0x0, 0x0, 0x0 }, "a"
		}
	}
};


/*--------------------------------------------------------------------------------
 Splash screen
--------------------------------------------------------------------------------*/
resource 'DITL' (phSplash,
#if qNames
"phSplash",
#endif
	purgeable, preload) {
	{	/* array DITLarray: 1 elements */
		/* [1] */
		{1, 1, 453-261+1, 490-101+1},
		Picture {
			disabled,
			phSplash
		}
	}
};

resource 'DLOG' (phSplash,
#if qNames
"phSplash",
#endif
	purgeable, preload) {
	{76, 80, 453-261+76+2, 490-101+80+2},
	altDBoxProc,
	visible,
	noGoAway,
	0,
	phSplash,
	""
};

#Include "Pict.r"

resource 'PICT' (phSplash, purgeable, preload) {
	{0, 0, 453-261, 490-101},
	VersionOne {
		{	/* array OpCodes: 4 elements */
			/* [1] */
			shortComment {
				130
			},
			/* [2] */
			clipRgn {
				{0, 0, 720, 576},
				$""
			},
			/* [3] */
			packBitsRect {
				50,
				{0, 0, 453-261, 496-96},
				{0, 0, 453-261, 490-101},
				{0, 0, 453-261, 490-101},
				srcCopy,
				$"02 CF 00 02 CF 00 02 CF 00 02 CF 00 02 CF 00 02 CF 00 02 CF"
				$"00 02 CF 00 02 CF 00 02 CF 00 1B 17 00 03 EF BE FB EF BE FB"
				$"EF BE FB EF BE FB EF BE FB EF BE FB EF BE FB E0 E7 00 1B 17"
				$"00 03 EF BE FB EF BE FB EF BE FB EF BE FB EF BE FB EF BE FB"
				$"EF BE FB E0 E7 00 1B 17 00 03 EF BE FB EF BE FB EF BE FB EF"
				$"BE FB EF BE FB EF BE FB EF BE FB E0 E7 00 1B 17 00 03 EF BE"
				$"FB EF BE FB EF BE FB EF BE FB EF BE FB EF BE FB EF BE FB E0"
				$"E7 00 1B 17 00 03 EF BE FB EF BE FB EF BE FB EF BE FB EF BE"
				$"FB EF BE FB EF BE FB E0 E7 00 10 E5 00 05 3F E0 30 00 00 1E"
				$"F8 00 01 3F E0 FC 00 21 02 00 03 E0 EE 00 01 03 E0 FE 00 0B"
				$"01 FF FE 70 00 00 1E 00 00 01 FF FF FE 00 03 01 FF FE 40 FD"
				$"00 21 02 00 03 E0 EE 00 01 03 E0 FE 00 06 07 F0 3F F0 00 00"
				$"3F FE 00 01 3F F8 FE 00 03 07 F0 3F C0 FD 00 21 02 00 03 E0"
				$"EE 00 01 03 E0 FE 00 06 1F C0 0F F0 00 00 3F FE 00 01 1F F0"
				$"FE 00 03 1F C0 0F C0 FD 00 21 02 00 03 E0 EE 00 01 03 E0 FE"
				$"00 06 3F 00 03 F0 00 00 3F FE 00 01 0F E0 FE 00 03 3F 00 03"
				$"C0 FD 00 21 02 00 03 E0 EE 00 01 03 E0 FE 00 0B 7E 00 01 F0"
				$"00 00 7F 80 00 00 0F E0 FE 00 03 7E 00 01 C0 FD 00 18 E6 00"
				$"0B FC 00 00 F8 00 00 7F 80 00 00 0F E0 FE 00 03 FC 00 00 C0"
				$"FD 00 21 02 00 03 E0 EE 00 17 03 E0 00 00 01 F8 00 00 78 00"
				$"00 FF C0 00 00 0F E0 00 00 01 F8 00 00 40 FD 00 1E 02 00 03"
				$"E0 EE 00 14 03 E0 00 00 03 F8 00 00 38 00 00 FF C0 00 00 0F"
				$"E0 00 00 03 F8 FA 00 1E 02 00 03 E0 EE 00 14 03 E0 00 00 03"
				$"F0 00 00 38 00 00 FF E0 00 00 0F E0 00 00 03 F0 FA 00 1E 02"
				$"00 03 E0 EE 00 14 03 E0 00 00 07 F0 00 00 18 00 01 EF E0 00"
				$"00 0F E0 00 00 07 F0 FA 00 1D 02 00 03 E0 EE 00 05 03 E0 00"
				$"00 07 E0 FD 00 0A 01 EF E0 00 00 0F E0 00 00 07 E0 FA 00 15"
				$"E7 00 01 0F E0 FD 00 0A 03 CF F0 00 00 0F E0 00 00 0F E0 FA"
				$"00 1D 02 00 03 E0 EE 00 05 03 E0 00 00 0F E0 FD 00 0A 03 C7"
				$"F0 00 00 0F E0 00 00 0F E0 FA 00 1D 02 00 03 E0 EE 00 05 03"
				$"E0 00 00 0F E0 FD 00 0A 07 87 F8 00 00 0F E0 00 00 0F E0 FA"
				$"00 1D 02 00 03 E0 EE 00 05 03 E0 00 00 1F C0 FD 00 0A 07 83"
				$"F8 00 00 0F E0 00 00 1F C0 FA 00 1D 02 00 03 E0 EE 00 05 03"
				$"E0 00 00 1F C0 FD 00 0A 0F 03 FC 00 00 0F E0 00 00 1F C0 FA"
				$"00 1D 02 00 03 E0 EE 00 05 03 E0 00 00 1F C0 FD 00 0A 0F 01"
				$"FC 00 00 0F E0 00 00 1F C0 FA 00 15 E7 00 01 1F C0 FD 00 0A"
				$"0E 01 FC 00 00 0F E0 00 00 1F C0 FA 00 1D 02 00 03 E0 EE 00"
				$"05 03 E0 00 00 1F C0 FD 00 0A 1E 00 FE 00 00 0F E0 00 00 1F"
				$"C0 FA 00 1D 02 00 03 E0 EE 00 05 03 E0 00 00 1F C0 FD 00 0A"
				$"1E 00 FE 00 00 0F E0 00 00 1F C0 FA 00 1D 02 00 03 E0 EE 00"
				$"05 03 E0 00 00 1F C0 FD 00 0A 3C 00 7F 00 00 0F E0 00 00 1F"
				$"C0 FA 00 1D 02 00 03 E0 EE 00 05 03 E0 00 00 1F C0 FD 00 0A"
				$"3C 00 7F 00 00 0F E0 00 00 1F C0 FA 00 1D 02 00 03 E0 EE 00"
				$"05 03 E0 00 00 1F C0 FD 00 0A 7F FF FF 80 00 0F E0 00 00 1F"
				$"C0 FA 00 15 E7 00 01 0F E0 FD 00 0A 7F FF FF 80 00 0F E0 00"
				$"00 0F E0 FA 00 2D 1B 00 03 E0 3E FB EF BE FB EF BE FB EF BE"
				$"FB EF BE FB EF BE FB EF BE 03 E0 00 00 0F E0 FD 00 0A F8 00"
				$"3F 80 00 0F E0 00 00 0F E0 FA 00 2D 1B 00 03 E0 3E FB EF BE"
				$"FB EF BE FB EF BE FB EF BE FB EF BE FB EF BE 03 E0 00 00 0F"
				$"E0 FD 00 0A F0 00 3F C0 00 0F E0 00 00 0F E0 FA 00 2D 1B 00"
				$"03 E0 3E FB EF BE FB EF BE FB EF BE FB EF BE FB EF BE FB EF"
				$"BE 03 E0 00 00 07 F0 FD 00 0A F0 00 1F C0 00 0F E0 00 06 07"
				$"F0 FA 00 2E 2A 00 03 E0 3E FB EF BE FB EF BE FB EF BE FB EF"
				$"BE FB EF BE FB EF BE 03 E0 00 00 07 F0 00 00 0C 01 E0 00 1F"
				$"E0 00 0F E0 00 06 07 F0 FA 00 2E 2A 00 03 E0 3E FB EF BE FB"
				$"EF BE FB EF BE FB EF BE FB EF BE FB EF BE 03 E0 00 00 03 F8"
				$"00 00 1C 01 E0 00 0F E0 00 0F E0 00 0E 03 F8 FA 00 16 E7 00"
				$"10 01 F8 00 00 38 03 C0 00 0F F0 00 0F E0 00 0E 01 F8 FA 00"
				$"31 2D 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0"
				$"00 03 E0 00 03 E0 00 00 01 FC 00 00 78 03 C0 00 0F F0 00 0F"
				$"E0 00 1C 01 FC 00 00 40 FD 00 31 17 00 03 E0 00 03 E0 00 03"
				$"E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 FE 00 12 FE"
				$"00 00 F0 07 80 00 07 F8 00 0F E0 00 3C 00 FE 00 00 C0 FD 00"
				$"31 17 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0"
				$"00 03 E0 00 03 E0 FE 00 12 7F 80 01 E0 07 80 00 07 F8 00 0F"
				$"E0 00 FC 00 7F 80 01 C0 FD 00 31 17 00 03 E0 00 03 E0 00 03"
				$"E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 FE 00 12 3F"
				$"C0 07 C0 0F 80 00 07 FC 00 1F FF FF F8 00 3F C0 07 C0 FD 00"
				$"31 17 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0"
				$"00 03 E0 00 03 E0 FE 00 12 0F F8 3F 80 1F C0 00 0F FE 00 3F"
				$"FF FF F8 00 0F F8 3F 80 FD 00 17 E6 00 09 03 FF FE 00 7F F8"
				$"00 7F FF 81 FE FF 04 F8 00 03 FF FE FC 00 25 17 00 03 E0 00"
				$"03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0"
				$"FD 00 01 7F F0 F4 00 01 7F F0 FC 00 1B 17 00 03 E0 00 03 E0"
				$"00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 E7 00"
				$"1B 17 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0"
				$"00 03 E0 00 03 E0 E7 00 1B 17 00 03 E0 00 03 E0 00 03 E0 00"
				$"03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 E7 00 1B 17 00 03"
				$"E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00"
				$"03 E0 E7 00 02 CF 00 21 17 00 03 E0 00 03 E0 00 03 E0 00 03"
				$"E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 F4 00 02 06 00 03 F7"
				$"00 25 17 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03"
				$"E0 00 03 E0 00 03 E0 F4 00 02 1E 00 0F FD 00 00 01 FC 00 25"
				$"17 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00"
				$"03 E0 00 03 E0 F4 00 02 06 00 03 FD 00 00 03 FC 00 25 17 00"
				$"03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0"
				$"00 03 E0 F4 00 02 06 00 03 FD 00 00 03 FC 00 25 17 00 03 E0"
				$"00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03"
				$"E0 F4 00 02 06 00 03 FD 00 00 03 FC 00 18 E7 00 12 03 E0 00"
				$"7C 31 E0 67 0F 80 7C 03 F6 07 C3 1C 03 E0 1F 03 FC 00 31 2D"
				$"00 03 E0 3E FB EF BE FB EF BE FB EF BE FB EF BE FB EF BE FB"
				$"EF BE 03 E0 00 00 0E 70 01 8C F7 F9 EF 39 E1 CE 0E 3E 18 C3"
				$"7E 0E 78 73 CF E0 FD 00 30 2C 00 03 E0 3E FB EF BE FB EF BE"
				$"FB EF BE FB EF BE FB EF BE FB EF BE 03 E0 00 00 08 38 03 04"
				$"38 38 71 30 61 07 0C 0E 30 43 87 0C 18 60 C3 FC 00 30 17 00"
				$"03 E0 3E FB EF BE FB EF BE FB EF BE FB EF BE FB EF BE FB EF"
				$"BE 03 E0 FE 00 11 18 03 00 30 1C 60 60 70 03 18 06 30 03 03"
				$"18 1C C0 E3 FC 00 30 17 00 03 E0 3E FB EF BE FB EF BE FB EF"
				$"BE FB EF BE FB EF BE FB EF BE 03 E0 FE 00 11 38 03 C0 30 0C"
				$"60 60 30 07 18 06 3C 03 03 18 0C C0 63 FC 00 30 2C 00 03 E0"
				$"3E FB EF BE FB EF BE FB EF BE FB EF BE FB EF BE FB EF BE 03"
				$"E0 00 00 01 F8 01 F8 30 0C 60 7F F0 3F 18 06 1F 83 03 1F FC"
				$"FF E3 FC 00 18 E7 00 12 0F 18 00 7C 30 0C 60 60 01 E3 18 06"
				$"07 C3 03 18 00 C0 03 FC 00 30 2C 00 03 E0 00 03 E0 00 03 E0"
				$"00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 00 1C 18 00"
				$"0E 30 0C 60 60 03 83 18 06 00 E3 03 18 00 C0 03 FC 00 30 2C"
				$"00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03"
				$"E0 00 03 E0 00 00 18 18 00 06 30 0C 60 70 03 03 1C 06 00 63"
				$"03 1C 00 E0 03 FC 00 30 2C 00 03 E0 00 03 E0 00 03 E0 00 03"
				$"E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 00 18 38 02 06 38"
				$"18 60 38 33 07 0E 0E 20 63 03 0E 0C 70 63 FC 00 31 2D 00 03"
				$"E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00"
				$"03 E0 00 00 1C D8 03 0C 3E 38 60 3E E3 9B 0F F6 30 C3 03 0F"
				$"B8 7D C3 E0 FD 00 31 2D 00 03 E0 00 03 E0 00 03 E0 00 03 E0"
				$"00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 00 0F 9C 03 F8 37 E0"
				$"F0 0F 81 F3 83 C7 3F 87 83 83 E0 1F 01 C0 FD 00 06 E3 00 00"
				$"30 EE 00 1B 0B 00 03 E0 00 03 E0 00 03 E0 00 03 E0 FA 00 04"
				$"03 E0 00 03 E0 FB 00 00 30 EE 00 1B 0B 00 03 E0 00 03 E0 00"
				$"03 E0 00 03 E0 FA 00 04 03 E0 00 03 E0 FB 00 00 30 EE 00 1B"
				$"0B 00 03 E0 00 03 E0 00 03 E0 00 03 E0 FA 00 04 03 E0 00 03"
				$"E0 FB 00 00 30 EE 00 1B 0B 00 03 E0 00 03 E0 00 03 E0 00 03"
				$"E0 FA 00 04 03 E0 00 03 E0 FB 00 00 30 EE 00 1B 0B 00 03 E0"
				$"00 03 E0 00 03 E0 00 03 E0 FA 00 04 03 E0 00 03 E0 FB 00 00"
				$"30 EE 00 06 E3 00 00 FC EE 00 1B 12 00 03 E0 00 03 E0 00 03"
				$"E0 00 03 E0 00 00 0F BE FB EF BE FE 00 01 03 E0 E7 00 1F 12"
				$"00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 00 0F BE FB EF BE FE"
				$"00 01 03 E0 F7 00 00 60 F2 00 26 12 00 03 E0 00 03 E0 00 03"
				$"E0 00 03 E0 00 00 0F BE FB EF BE FE 00 01 03 E0 F8 00 01 01"
				$"E0 FC 00 02 03 04 04 FA 00 25 12 00 03 E0 00 03 E0 00 03 E0"
				$"00 03 E0 00 00 0F BE FB EF BE FE 00 01 03 E0 F7 00 00 60 FC"
				$"00 02 03 0C 0C FA 00 24 12 00 03 E0 00 03 E0 00 03 E0 00 03"
				$"E0 00 00 0F BE FB EF BE FE 00 01 03 E0 F7 00 00 60 FB 00 01"
				$"0C 0C FA 00 0B DF 00 00 60 FB 00 01 0C 0C FA 00 31 0E 00 03"
				$"E0 3E FB EF BE FB EF BE FB EF 80 03 E0 FD 00 1B F8 00 3E 03"
				$"E0 00 00 03 E0 7C 0C 70 70 31 E0 60 7C 00 F0 60 E6 73 0C 0C"
				$"03 E0 63 80 FE 00 31 0E 00 03 E0 3E FB EF BE FB EF BE FB EF"
				$"80 03 E0 FD 00 1B F8 00 3E 03 E0 00 00 0C 61 CE 3D F9 F8 F7"
				$"F8 61 CF 00 60 60 5E FF 3F BF 8E 79 EF C0 FE 00 31 0E 00 03"
				$"E0 3E FB EF BE FB EF BE FB EF 80 03 E0 FD 00 15 F8 00 3E 03"
				$"E0 00 00 18 21 07 0E 1E 1C 38 38 61 83 00 60 E0 C7 13 FE 0C"
				$"02 18 70 E0 FE 00 31 0E 00 03 E0 3E FB EF BE FB EF BE FB EF"
				$"80 03 E0 FD 00 09 F8 00 3E 03 E0 00 00 18 00 03 FE 0C 0E 30"
				$"1C 63 03 80 30 B0 86 03 0C 0C 18 1C 60 60 FE 00 31 0E 00 03"
				$"E0 3E FB EF BE FB EF BE FB EF 80 03 E0 FD 00 09 F8 00 3E 03"
				$"E0 00 00 1E 00 07 FE 0C 0E 30 0C 63 01 80 30 B0 86 03 0C 0C"
				$"18 0C 60 60 FE 00 1A E7 00 02 0F C0 3F FE 0C 0E 30 0C 63 FF"
				$"80 31 B1 86 03 0C 0C 1F FC 60 60 FE 00 30 0D 00 03 E0 00 03"
				$"E0 00 03 E0 00 03 E0 00 F8 FC 00 09 03 E0 00 03 E0 00 00 03"
				$"E1 E3 FE 0C 0E 30 0C 63 00 00 19 19 06 03 0C 0C 18 00 60 60"
				$"FE 00 30 0D 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 F8 FC 00"
				$"04 03 E0 00 03 E0 FE 00 01 73 83 FE 0C 0E 30 0C 63 00 00 19"
				$"19 06 03 0C 0C 18 00 60 60 FE 00 30 0D 00 03 E0 00 03 E0 00"
				$"03 E0 00 03 E0 00 F8 FC 00 04 03 E0 00 03 E0 FE 00 01 33 03"
				$"FE 0C 0E 30 0C 63 80 00 1B 1B 06 03 0C 0C 1C 00 60 60 FE 00"
				$"30 0D 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 F8 FC 00 09 03"
				$"E0 00 03 E0 00 00 10 33 07 FE 0C 0E 38 18 61 C1 80 0E 0E 06"
				$"03 0C 0C 0E 0C 60 60 FE 00 30 0D 00 03 E0 00 03 E0 00 03 E0"
				$"00 03 E0 00 F8 FC 00 09 03 E0 00 03 E0 00 00 18 63 9B FE 0C"
				$"0E 3E 38 61 F7 00 0E 0E 06 03 0F 8F 8F B8 60 60 FE 00 1A E7"
				$"00 14 1F C1 F3 9E 1E 0E 37 E0 F0 7C 00 04 04 0F 07 87 07 03"
				$"E0 F0 70 FE 00 1E 0F 00 03 E0 00 03 E0 00 03 E0 00 03 E0 3E"
				$"00 0F BE FD 00 03 0F 80 03 E0 F9 00 00 30 F0 00 1E 0F 00 03"
				$"E0 00 03 E0 00 03 E0 00 03 E0 3E 00 0F BE FD 00 03 0F 80 03"
				$"E0 F9 00 00 30 F0 00 1E 0F 00 03 E0 00 03 E0 00 03 E0 00 03"
				$"E0 3E 00 0F BE FD 00 03 0F 80 03 E0 F9 00 00 30 F0 00 1E 0F"
				$"00 03 E0 00 03 E0 00 03 E0 00 03 E0 3E 00 0F BE FD 00 03 0F"
				$"80 03 E0 F9 00 00 30 F0 00 1E 0F 00 03 E0 00 03 E0 00 03 E0"
				$"00 03 E0 3E 00 0F BE FD 00 03 0F 80 03 E0 F9 00 00 30 F0 00"
				$"06 E1 00 00 30 F0 00 1C 08 00 03 E0 00 03 E0 00 03 E0 FE 00"
				$"04 3E FB E0 00 F8 FD 00 00 3E F7 00 00 FC F0 00 18 08 00 03"
				$"E0 00 03 E0 00 03 E0 FE 00 04 3E FB E0 00 F8 FD 00 00 3E E5"
				$"00 29 08 00 03 E0 00 03 E0 00 03 E0 FE 00 04 3E FB E0 00 F8"
				$"FD 00 00 3E F9 00 05 06 00 00 78 00 1C FD 00 00 20 FC 00 00"
				$"7C FE 00 2C 08 00 03 E0 00 03 E0 00 03 E0 FE 00 04 3E FB E0"
				$"00 F8 FD 00 00 3E FB 00 07 01 82 1E 00 00 1C 00 38 FD 00 00"
				$"70 FD 00 01 01 83 FE 00 2D 08 00 03 E0 00 03 E0 00 03 E0 FE"
				$"00 04 3E FB E0 00 F8 FD 00 00 3E FB 00 07 01 86 06 00 00 1C"
				$"00 38 FD 00 00 70 FD 00 02 02 00 80 FF 00 16 E4 00 06 06 06"
				$"00 00 1E 00 78 FD 00 00 F8 FD 00 02 05 F8 40 FF 00 30 18 00"
				$"03 E0 3E FB EF BE FB EF 80 FB EF 80 03 EF BE FB EF BE F8 00"
				$"00 FB EF BE FD 00 06 06 06 00 00 16 00 58 FD 00 00 98 FD 00"
				$"02 08 CC 20 FF 00 33 2F 00 03 E0 3E FB EF BE FB EF 80 FB EF"
				$"80 03 EF BE FB EF BE F8 00 00 FB EF BE 00 3C 18 39 86 06 38"
				$"00 17 00 D8 1F 00 FC 01 9C 03 1E 06 3C 08 CC 20 FF 00 33 2F"
				$"00 03 E0 3E FB EF BE FB EF 80 FB EF 80 03 EF BE FB EF BE F8"
				$"00 00 FB EF BE 00 18 18 17 9F C6 FC 00 13 00 98 73 83 8E 01"
				$"0C 0F 7F 9E FF 10 CC 10 FF 00 33 2F 00 03 E0 3E FB EF BE FB"
				$"EF 80 FB EF 80 03 EF BE FB EF BE F8 00 00 FB EF BE 00 18 38"
				$"31 86 07 0E 00 13 81 98 41 C3 06 03 0E 03 83 87 07 10 F8 10"
				$"FF 00 33 2F 00 03 E0 3E FB EF BE FB EF 80 FB EF 80 03 EF BE"
				$"FB EF BE F8 00 00 FB EF BE 00 0C 2C 21 86 06 06 00 11 81 18"
				$"00 C6 02 02 06 03 01 C6 03 90 D8 10 FF 00 1B E7 00 15 0C 2C"
				$"21 86 06 06 00 11 C3 18 01 C6 00 06 07 03 00 C6 01 90 CC 10"
				$"FF 00 30 0F 00 03 E0 00 03 E0 00 03 E0 00 00 0F 80 00 0F BE"
				$"FB 00 19 03 EF BE 00 0C 6C 61 86 06 06 00 10 C2 18 0F C6 00"
				$"07 FF 03 00 C6 01 90 CC 10 FF 00 30 0F 00 03 E0 00 03 E0 00"
				$"03 E0 00 00 0F 80 00 0F BE FB 00 19 03 EF BE 00 06 46 41 86"
				$"06 06 00 10 E6 18 78 C6 00 0C 03 83 00 C6 01 88 C6 20 FF 00"
				$"30 0F 00 03 E0 00 03 E0 00 03 E0 00 00 0F 80 00 0F BE FB 00"
				$"19 03 EF BE 00 06 46 41 86 06 06 00 10 64 18 E0 C6 00 08 01"
				$"83 00 C6 01 89 E7 20 FF 00 30 0F 00 03 E0 00 03 E0 00 03 E0"
				$"00 00 0F 80 00 0F BE FB 00 19 03 EF BE 00 06 C6 C1 86 06 06"
				$"00 10 7C 18 C0 C7 00 18 01 C3 00 C6 01 84 00 40 FF 00 30 0F"
				$"00 03 E0 00 03 E0 00 03 E0 00 00 0F 80 00 0F BE FB 00 19 03"
				$"EF BE 00 03 83 81 86 06 06 00 10 38 18 C1 C3 83 10 00 C3 81"
				$"87 03 02 00 80 FF 00 1A E7 00 14 03 83 81 87 C6 06 00 10 38"
				$"18 E6 C3 EE 30 00 E3 E3 87 C7 01 83 FE 00 2C 08 00 03 E0 00"
				$"03 E0 00 03 E0 FE 00 00 3E F8 00 18 03 EF BE 00 01 01 03 C3"
				$"8F 07 00 7C 10 7C 7C E0 F8 78 01 F3 7E 06 FC 00 7C FE 00 1C"
				$"08 00 03 E0 00 03 E0 00 03 E0 FE 00 00 3E F8 00 02 03 EF BE"
				$"F1 00 02 03 00 06 FB 00 1C 08 00 03 E0 00 03 E0 00 03 E0 FE"
				$"00 00 3E F8 00 02 03 EF BE F1 00 02 03 00 06 FB 00 1C 08 00"
				$"03 E0 00 03 E0 00 03 E0 FE 00 00 3E F8 00 02 03 EF BE F1 00"
				$"02 03 00 06 FB 00 1C 08 00 03 E0 00 03 E0 00 03 E0 FE 00 00"
				$"3E F8 00 02 03 EF BE F1 00 02 03 00 06 FB 00 08 D8 00 02 03"
				$"00 06 FB 00 1D 0D 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 F8"
				$"F9 00 02 03 EF BE F1 00 02 03 00 06 FB 00 1E 0D 00 03 E0 00"
				$"03 E0 00 03 E0 00 03 E0 00 F8 F9 00 02 03 EF BE F1 00 03 0F"
				$"C0 1F 80 FC 00 17 0D 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00"
				$"F8 F9 00 02 03 EF BE E8 00 17 0D 00 03 E0 00 03 E0 00 03 E0"
				$"00 03 E0 00 F8 F9 00 02 03 EF BE E8 00 17 0D 00 03 E0 00 03"
				$"E0 00 03 E0 00 03 E0 00 F8 F9 00 02 03 EF BE E8 00 02 CF 00"
				$"19 0F 00 03 E0 3E FB EF BE FB EF BE FB EF 80 03 EF 80 FB 00"
				$"02 03 EF BE E8 00 19 0F 00 03 E0 3E FB EF BE FB EF BE FB EF"
				$"80 03 EF 80 FB 00 02 03 EF BE E8 00 19 0F 00 03 E0 3E FB EF"
				$"BE FB EF BE FB EF 80 03 EF 80 FB 00 02 03 EF BE E8 00 19 0F"
				$"00 03 E0 3E FB EF BE FB EF BE FB EF 80 03 EF 80 FB 00 02 03"
				$"EF BE E8 00 19 0F 00 03 E0 3E FB EF BE FB EF BE FB EF 80 03"
				$"EF 80 FB 00 02 03 EF BE E8 00 02 CF 00 1C 0B 00 03 E0 00 03"
				$"E0 00 03 E0 00 03 E0 FE 00 09 3E FB EF BE FB EF 80 03 EF BE"
				$"E8 00 1C 0B 00 03 E0 00 03 E0 00 03 E0 00 03 E0 FE 00 09 3E"
				$"FB EF BE FB EF 80 03 EF BE E8 00 1C 0B 00 03 E0 00 03 E0 00"
				$"03 E0 00 03 E0 FE 00 09 3E FB EF BE FB EF 80 03 EF BE E8 00"
				$"1C 0B 00 03 E0 00 03 E0 00 03 E0 00 03 E0 FE 00 09 3E FB EF"
				$"BE FB EF 80 03 EF BE E8 00 1C 0B 00 03 E0 00 03 E0 00 03 E0"
				$"00 03 E0 FE 00 09 3E FB EF BE FB EF 80 03 EF BE E8 00 02 CF"
				$"00 19 0E 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 FB 00"
				$"03 3E FB EF BE E8 00 19 0E 00 03 E0 00 03 E0 00 03 E0 00 03"
				$"E0 00 03 E0 FB 00 03 3E FB EF BE E8 00 19 0E 00 03 E0 00 03"
				$"E0 00 03 E0 00 03 E0 00 03 E0 FB 00 03 3E FB EF BE E8 00 19"
				$"0E 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00 03 E0 FB 00 03 3E"
				$"FB EF BE E8 00 19 0E 00 03 E0 00 03 E0 00 03 E0 00 03 E0 00"
				$"03 E0 FB 00 03 3E FB EF BE E8 00 02 CF 00 0C 02 00 03 E0 EE"
				$"00 02 03 EF BE E8 00 0C 02 00 03 E0 EE 00 02 03 EF BE E8 00"
				$"0C 02 00 03 E0 EE 00 02 03 EF BE E8 00 0C 02 00 03 E0 EE 00"
				$"02 03 EF BE E8 00 0C 02 00 03 E0 EE 00 02 03 EF BE E8 00 02"
				$"CF 00 19 15 00 03 EF BE FB EF BE FB EF BE FB EF BE FB EF BE"
				$"FB EF BE FB EF BE E5 00 19 15 00 03 EF BE FB EF BE FB EF BE"
				$"FB EF BE FB EF BE FB EF BE FB EF BE E5 00 19 15 00 03 EF BE"
				$"FB EF BE FB EF BE FB EF BE FB EF BE FB EF BE FB EF BE E5 00"
				$"19 15 00 03 EF BE FB EF BE FB EF BE FB EF BE FB EF BE FB EF"
				$"BE FB EF BE E5 00 19 15 00 03 EF BE FB EF BE FB EF BE FB EF"
				$"BE FB EF BE FB EF BE FB EF BE E5 00 02 CF 00 02 CF 00 02 CF"
				$"00 02 CF 00 02 CF 00 02 CF 00 2C 02 00 00 74 FE 00 0A 10 02"
				$"00 03 C0 04 00 00 20 00 74 FB 00 10 01 C0 00 00 04 18 38 F8"
				$"07 0C 00 01 02 20 01 00 20 FA 00 00 20 FF 00 2E 02 00 01 8C"
				$"FD 00 09 06 04 0C 30 04 00 00 60 01 8C FD 00 13 80 00 00 80"
				$"00 00 0C 24 44 80 08 92 00 01 06 60 00 00 60 40 FB 00 00 60"
				$"FF 00 2E 02 00 01 04 FD 00 09 02 04 08 10 0A 00 00 20 01 04"
				$"FD 00 13 80 00 00 80 00 00 04 42 44 80 08 A1 00 02 82 20 00"
				$"00 20 40 FB 00 00 20 FF 00 33 2F 00 02 00 61 E7 7D 90 EA CE"
				$"11 C8 0A 0F 1E 23 82 00 63 66 3C CD CE 6C 00 9B 1C 00 04 42"
				$"28 E0 05 21 00 02 82 20 D9 0E AC E7 0D 9C 73 9B 77 71 E0 FF"
				$"00 33 2F 00 02 00 93 12 26 B1 13 24 12 48 11 18 B1 24 42 00"
				$"91 99 62 44 91 34 00 8C A6 00 04 42 10 10 02 21 00 04 42 20"
				$"6B 11 32 49 06 A2 94 4D 22 8A 20 FF 00 33 2F 00 02 01 09 12"
				$"24 11 12 24 12 08 11 08 91 24 42 01 09 11 22 44 91 20 00 88"
				$"A0 00 04 26 28 08 05 13 00 04 42 20 41 11 22 48 04 22 84 48"
				$"22 8A 20 FF 00 33 2F 00 02 01 09 11 44 11 12 24 11 C8 1F 08"
				$"91 27 C2 01 09 11 22 44 9F 20 00 88 A0 00 04 1A 44 0B C8 8D"
				$"00 07 C2 20 41 11 22 46 04 3E 67 C8 14 FA 20 FF 00 33 2F 00"
				$"01 05 09 11 44 10 E2 24 08 10 20 88 91 24 01 05 09 11 22 44"
				$"90 20 00 88 A0 00 04 04 44 08 08 82 00 08 22 20 41 0E 22 41"
				$"04 20 14 08 14 82 20 FF 00 33 2F 00 01 88 91 10 84 11 02 25"
				$"0C 30 20 88 91 24 41 88 91 11 22 4C B1 20 00 88 A2 00 04 0C"
				$"44 90 08 86 00 08 22 20 41 10 22 59 04 22 94 48 08 8A 20 FF"
				$"00 33 2F 00 00 70 61 E0 8E 39 E7 72 03 C0 71 CF 1E 73 80 70"
				$"63 BB BC 36 4E 70 81 DD DC 40 0E 30 38 60 07 18 20 1C 77 70"
				$"E3 9E 77 2E 0E 1C E3 9C 08 71 D2 FF 00 1B FE 00 04 01 01 00"
				$"02 10 FD 00 01 08 10 FC 00 00 20 FE 00 00 80 F3 00 00 21 F6"
				$"00 1B FE 00 04 01 01 00 02 10 FD 00 01 08 10 FC 00 00 20 FE"
				$"00 00 80 F3 00 00 21 F6 00 1A FE 00 04 03 86 00 01 E0 FD 00"
				$"01 1C 38 FC 00 03 70 00 00 01 F2 00 00 1E F6 00 02 CF 00 02"
				$"CF 00 02 CF 00 02 CF 00 02 CF 00 02 CF 00 02 CF 00 02 CF 00"
			},
			/* [4] */
			shortComment {
				131
			}
		}
	}
};


#if qDebug
resource 'dbug' (kDebugParamsID,
#if qNames
"Debug",
#endif
	purgeable) {
		normal,							// Bounding rect for debug window
		4, 								// Debug window font rsrc ID (monaco)
		normal,				 			// Debug window font size
		60,								// Number of lines
		90,								// Width of lines in characters
		openInitially,					// show window at startup
		"Calc Debug Transcript"			// Window title
};
#endif

include "Defaults.rsrc"  'STR#' (kDefaultCredits);		// Grab the default credits

// Icons, Bundles and FRefsOh my! (don't forget the Signature)
resource 'ICN#' (128,
#if qNames
"Application",
#endif
	purgeable) {
	{ /* array: 2 elements */
		/* [1] */
		$"00 00 00 00 00 00 00 00 7F FF FF FC 40 00 00 04"
		$"40 00 00 04 40 00 00 04 40 00 00 04 5F FF FF F4"
		$"44 44 44 44 44 44 44 44 44 44 44 44 5F FF FF F4"
		$"44 44 44 44 44 44 40 44 44 44 3F 04 5F FF 40 B4"
		$"44 44 80 44 44 45 30 24 44 41 C8 14 5F EE 7F 8F"
		$"44 42 30 07 44 45 00 07 44 44 80 07 5F FF 60 07"
		$"44 44 1F E7 44 44 40 1F 40 00 00 07 7F FF FF FC",
		/* [2] */
		$"00 00 00 00 00 00 00 00 7F FF FF FC 7F FF FF FC"
		$"7F FF FF FC 7F FF FF FC 7F FF FF FC 7F FF FF FC"
		$"7F FF FF FC 7F FF FF FC 7F FF FF FC 7F FF FF FC"
		$"7F FF FF FC 7F FF FF FC 7F FF FF FC 7F FF FF FC"
		$"7F FF FF FC 7F FF FF FC 7F FF FF FC 7F FF FF FF"
		$"7F FF FF FF 7F FF FF FF 7F FF FF FF 7F FF FF FF"
		$"7F FF FF FF 7F FF FF FF 7F FF FF FF 7F FF FF FC"
	}
};

resource 'ICN#' (129,
#if qNames
"Document",
#endif
	purgeable) {
	{ /* array: 2 elements */
		/* [1] */
		$"0F FF FE 00 08 00 03 00 08 00 02 80 08 80 02 40"
		$"08 80 02 20 08 80 02 10 0F F0 03 F8 08 80 00 08"
		$"08 80 00 08 08 80 00 08 08 00 00 08 08 00 00 08"
		$"08 0F E0 08 08 00 00 08 08 00 00 08 08 00 00 08"
		$"08 00 10 08 08 00 54 08 08 00 38 08 08 00 FE 08"
		$"08 00 38 08 08 00 54 08 08 00 10 08 08 00 00 08"
		$"08 00 00 88 08 00 00 08 08 00 07 F8 08 00 00 08"
		$"08 00 00 88 08 00 00 08 08 00 00 08 0F FF FF F8",
		/* [2] */
		$"0F FF FE 00 0F FF FF 00 0F FF FF 80 0F FF FF C0"
		$"0F FF FF E0 0F FF FF F0 0F FF FF F8 0F FF FF F8"
		$"0F FF FF F8 0F FF FF F8 0F FF FF F8 0F FF FF F8"
		$"0F FF FF F8 0F FF FF F8 0F FF FF F8 0F FF FF F8"
		$"0F FF FF F8 0F FF FF F8 0F FF FF F8 0F FF FF F8"
		$"0F FF FF F8 0F FF FF F8 0F FF FF F8 0F FF FF F8"
		$"0F FF FF F8 0F FF FF F8 0F FF FF F8 0F FF FF F8"
		$"0F FF FF F8 0F FF FF F8 0F FF FF F8 0F FF FF F8"
	}
};

resource 'FREF' (128,
#if qNames
"Application",
#endif
	purgeable) {
	'APPL',
	0,
	""
};

resource 'FREF' (129,
#if qNames
"Document",
#endif
	purgeable) {
	kFileType,
	1,
	""
};

resource 'BNDL' (128,
#if qNames
"Calc",
#endif
	purgeable) {
	kSignature,
	0,
	{ /* array TypeArray: 2 elements */
		/* [1] */
		'ICN#',
		{ /* array IDArray: 2 elements */
			/* [1] */
			0, 128,
			/* [2] */
			1, 129
		},
		/* [2] */
		'FREF',
		{ /* array IDArray: 2 elements */
			/* [1] */
			0, 128,
			/* [2] */
			1, 129
		}
	}
};

type kSignature as 'STR ';
resource kSignature (0,
#if qNames
"Signature",
#endif
	purgeable) {
	"Calc 1.0 Apple Computer, Inc. 1988-1990"
};

// Get the default Version resources
/*--------------------------------------------------------------------------------
 The revision of this particular file
--------------------------------------------------------------------------------*/
RESOURCE 'vers' (1,
#if qNames
"File Version",
#endif
	purgeable) {
	0x01,
	0x00,
	beta,
	0x0A,
	verUs,
	"1.0b10",
	"Calc 1.0b10, Apple Computer, Inc. 1988-1990"
};
include "Defaults.rsrc"  'vers' (2);		// Overall package

/********************************************************************************/
/*	End of File: Calc.r															*/
/********************************************************************************/