// Copyright  1988, 1989 by Apple Computer, Inc.  All rights reserved.

//   MacApp
#include "UMacApp.h"

//	 Building Blocks
#include "UPrinting.h"

//	 Implementation Use
#include <StdLib.h>
#include <Picker.h>
#include <ToolUtils.h>
#include <Resources.h>


// Constants

const OSType kSignature =	'ICED'; 			// Application signature
const OSType kFileType =	'ICON'; 			// File-type for document files created by this application

// Types

	typedef short		Integer;
	typedef long		Longint;

	struct TIconEditApplication : TApplication
		{
		virtual pascal void IIconEditApplication(OSType itsMainFileType);
			/* Initializes the application and globals. */

		virtual pascal struct TDocument* DoMakeDocument(CmdNumber itsCmdNumber);
			/* Creates a document of type TIconDocument and returns a reference to it. */

		virtual pascal TView* MakeViewForAlienClipboard();
			/* Makes a view of type TIconEditView for the Clipboard if the public desk
				scrap contains data of type 'ICON'.  Otherwise it calls INHERITED
				MakeViewForAlienClipboard. */

		};


	struct TIconDocument : TDocument
		{
		struct TIconBitMap		*fIconBitMap;				// The icon's bit map.
		struct TIconEditView	*fIconDocView;				// The view created by DoMakeViews

		virtual pascal void IIconDocument();
			/* Initializes the document. */

		virtual pascal void Free();
			/* Needed to free the fIconBitMap object. */

		virtual pascal void DoInitialState();
			/* Sets the document's data to represent a "new" document. */

		virtual pascal void DoMakeViews (Boolean forPrinting);
			/* Creates the window and view objects when an icon document is opened. */

		virtual pascal struct TCommand* DoMenuCommand (CmdNumber aCmdNumber);
			/* Handles menus specific to this class. */

		virtual pascal void DoSetupMenus();
			/* Sets the state of the menus to which this class responds. */

#if qDebug
		virtual pascal void Fields (
			pascal void (*DoToField)(StringPtr fieldName, Ptr fieldAddr,
									 short fieldType, void* DoToField_StaticLink),
			void* DoToField_StaticLink);
#endif

		virtual pascal void InvertIcon();
			/* Inverts the bits of this document's icon and redraw its views.  */

		virtual pascal void ClearIcon();
			/* Clear the bits of this document's icon and redraw its views. */

		virtual pascal void SetIcon (struct TIconBitMap* newIcon);
			/* Set the new document icon and redraw its views. */
			
		virtual pascal void RedrawViews();
			/* Causes all views representing this document to be redrawn. */

		virtual pascal void DoNeedDiskSpace (Longint* dataForkBytes, Longint* rsrcForkBytes);
			/* Adds the size of the icon bit map in bytes to dataForkBytes. */

		virtual pascal void DoRead (Integer aRefNum, Boolean rsrcExists, Boolean forPrinting);
			/* Reads the icon bit map from the data fork. */

		virtual pascal void DoWrite (Integer aRefNum, Boolean makingCopy);
			/* Writes the icon bit map to the data fork. */
		};


	struct TIconEditView : TView
		{
		Integer					fMagnification;			// No. of times to magnify the icon.
		struct TIconDocument	*fIconDocument;			// Reference to the view's icon document.

		virtual pascal void IRes (TDocument* itsDocument,
								  TView* itsSuperView, Ptr* itsParams);
			/* Initializes the view from a resource template. */

		virtual pascal void CalcMinSize (VPoint* minSize);
			/* Returns the view's minimum size. */

		virtual pascal Boolean ContainsClipType(ResType aType);
			/* Called when a view of this type is installed in the Clipboard.
				Returns true if aType is kIconClipType (i.e. 'ICON'). */

		virtual pascal struct TCommand* DoKeyCommand (short ch, Integer aKeyCode, EventInfo* info);
			/* Treats the backspace the same as the Clear command. */

		virtual pascal struct TCommand* DoMenuCommand (CmdNumber aCmdNumber);
			/* Handles menus specific to this class. */

		virtual pascal struct TCommand* DoMouseCommand (Point* theMouse, EventInfo* info, Point* hysteresis);

		virtual pascal Boolean DoSetCursor (Point localPoint, RgnHandle cursorRgn);
			/* Sets the cursor to the pencil when the mouse is over the icon. */

		virtual pascal void DoSetupMenus();
			/* Sets the state of the menus to which this class responds. */

		virtual pascal void Draw (Rect *area);
			/* Draws this view. */

		virtual pascal void DrawBit (Point theBit, Boolean turnBitOn);
			/* Draws the given bit to the given state. */

#if qDebug
		virtual pascal void Fields (
			pascal void (*DoToField)(StringPtr fieldName, Ptr fieldAddr,
									 short fieldType, void* DoToField_StaticLink),
			void* DoToField_StaticLink);
#endif

		virtual pascal Boolean PointToBit (Point thePoint, Point* iconBit);
			/* Converts the given mouse point to an icon bit. */
	
		virtual pascal void SetMagnification (Integer magnification);
			/* Sets the view's magnification. */

		virtual pascal void WriteToDeskScrap();
			/* Writes the view's data to the desk scrap. */

		};


	struct TInvertCommand : TCommand
		{
		struct TIconDocument	*fIconDocument;		// The document affected by this command.

		virtual pascal void IInvertCommand (TIconDocument* itsIconDocument);
			/* Initializes this command, associates it with the given document. */

		virtual pascal void DoIt();
			/* Implements the command's action by calling the document's Invert method. */

		virtual pascal void RedoIt();
			/* Redoes the command's action by calling the document's Invert method. */

		virtual pascal void UndoIt();
			/* Undoes the command's action by calling the document's Invert method. */
		
		};


	struct TIconDrawCommand : TCommand
		{
		struct TIconDocument	*fIconDocument;			// The document affected by this command.
		struct TIconEditView	*fIconEditView;			// The view in which this command draws.
		struct TIconBitMap		*fIconBitMap;			// The icon in which drawing takes place.
		struct TIconBitMap		*fOriginalIcon;			// A copy of the original icon.
		Boolean					fTurnBitsOn;			// Whether to draw bits black or white.

		virtual pascal void IIconDrawCommand (TIconEditView* itsIconEditView);
			/* Initializes the command and associates it with a view. */

		virtual pascal void Free();
			/* Frees this object by disposing of fIconBitMap or fOriginalIcon,
				depending on whether the command is in the "done" or "undone" state. */

		virtual pascal void DoIt();
			/* Sets the fIconDocument's icon bitmap to that of fIconBitMap by calling
				the document's SetIcon method. Validates the edit view to avoid
				flashing. */

		virtual pascal void RedoIt();
			/* Sets the fIconDocument's icon bitmap to that of fIconBitMap by calling
				the document's SetIcon method. */

		virtual pascal void TrackConstrain(VPoint *anchorPoint, VPoint *previousPoint, VPoint *nextPoint
	   );
			/* Constrains the mouse to be within the icon in the edit view by
				factoring out the margins. */

		virtual pascal void TrackFeedback (VPoint *anchorPoint, VPoint *nextPoint, Boolean turnItOn, 
	   Boolean mouseDidMove);
			/* Overridden to avoid standard feedback. */

		virtual pascal struct TCommand* TrackMouse(TrackPhase aTrackPhase,
									VPoint* anchorPoint, VPoint* previousPoint, VPoint* nextPoint,
									Boolean mouseDidMove);
			/* Tracks the mouse. */

		virtual pascal void UndoIt();
			/* Sets the fIconDocument's icon bitmap to that of fOriginalIcon by calling
				the document's SetIcon method. */

		};


	struct TIconEditCommand : TCommand
		{
		struct TIconDocument	*fIconDocument;			// The document affected by this command.
		struct TIconEditView	*fIconEditView;			// The view in which this command draws.
		struct TIconBitMap		*fIconBitMap;			// The icon in which drawing takes place.
		struct TIconBitMap		*fOriginalIcon;			// A copy of the original icon.

		virtual pascal void IIconEditCommand (CmdNumber itsCmdNumber, TIconEditView* itsIconEditView);
			/* Initializes the command object, associates it with a view and document,
				makes a copy of the document's icon for undo purposes. */

		virtual pascal void Free();
			/* Free fIconBitMap or fOriginalIcon depending on whether the command
				is left in the done or undo state. */

		virtual pascal void DoIt();
			/* Carries out the Cut, Copy or Clear command. */

		virtual pascal void RedoIt();
			/* Redoes the Cut, Copy or Clear command. */

		virtual pascal void UndoIt();
			/* Undoes the Cut, Copy or Clear command. */

		};


	struct TIconPasteCommand : TCommand
		{
		struct TIconDocument	*fIconDocument;			// The document affected by this command.
		struct TIconEditView	*fIconEditView;			// The view in which this command draws.
		struct TIconBitMap		*fClipIcon;				// A copy of the icon being pasted.
		struct TIconBitMap		*fOriginalIcon;			// A reference of the original icon.

		virtual pascal void IIconPasteCommand (TIconEditView* itsIconEditView);
			/* Initializes the command object, associates it with a view and document,
				saves a reference to the document's icon for undo purposes. */

		virtual pascal void Free();
			/* Frees fIconBitMap or fOriginalIcon depending on whether the command
				is left in the done or undo state. */

		virtual pascal void DoIt();
			/* Carries out the Paste command by setting the document's icon to a
				copy of the icon in the Clipboard. */

		virtual pascal void RedoIt();
			/* Redoes the Paste command. */

		virtual pascal void UndoIt();
			/* Undoes the Paste command. */

		};


	struct TIconBitMap : TObject
		{
		Handle				fDataHandle;			// Handle to the icon's data.

		virtual pascal void IIconBitMap();
			/* Initializes the IconBitMap object and allocates space for its data. */

		virtual pascal void Free();
			/* Free the icon's bit map. */
			
		virtual pascal void SetIconBitMap(Handle theBitMap);
			/* Set the contents of the icon bit map to the new bit map. */
			
		virtual pascal void ReadFrom (Integer aRefNum);
			/* Reads the icon bit map from the data fork. */

		virtual pascal void WriteTo (Integer aRefNum);
			/* Writes the icon bit map to the data fork. */
			
		virtual pascal void LoadFromScrap();
			/* Load the icon's bit map from the clipboard. */
			
		virtual pascal void WriteToScrap();
			/* Write the icon's bit map to the clipboard. */
			
		virtual pascal struct TIconBitMap* Copy();
			/* Create a new bit map which is a copy of itself. */
			
		virtual pascal void CopyDataTo (TIconBitMap* anIcon);
			/* Copy icon data to the indicated icon object. */
			
		virtual pascal void Clear();
			/* Clear the icon map by setting its bits to zero. */
			
		virtual pascal void Invert();
			/* Invert the icon's bit map. */
			
		virtual pascal void Draw (const Rect* area);
			/* Draw the icon's bit map. */
			
		virtual pascal Boolean GetBit(Point iconBit);
			/* Return the state of the given bit. */
			
		virtual pascal void SetBit(Point iconBit, Boolean turnBitOn);
			/* Set the state of the given bit as indicated. */

#if qDebug
		virtual pascal void Fields (
			pascal void (*DoToField)(StringPtr fieldName, Ptr fieldAddr,
									 short fieldType, void* DoToField_StaticLink),
			void* DoToField_StaticLink);
#endif

		};

