/*
	Goodies and support routines for interfacing with Pascal
*/

#ifndef __PASCAL__
#define __PASCAL__

#ifndef __STRING__
	#include <String.h>
#endif

#ifndef __SETJMP__
	#include <SetJmp.h>
#endif



inline void pstrcpy(void* p, void* q)					// Pascal string assignment
{	memcpy(p, q, *(char*)(q)+1);	}

inline Boolean pstrcmp(void* p, void* q)				// Pascal string compare
{	return memcmp(p, q, *(char*)(p)+1)==0;	}

inline Point* RectTopLeft(Rect& r)						// provide access to rect.topLeft 
{	return (Point*)(&r.top); }

inline Point* RectBotRight(Rect& r)						// provide access to rect.botRight 
{	return (Point*)(&r.bottom); }

inline void FailNULL(void* p)							// so you can write FailNULL
{	FailNIL(p); }


// Provide a way to implement the Fields method.  See Tech Note 265
// P.S. We'll probably fix this in the first post 2.0 release of MacApp by
// changing the way Fields works.

typedef pascal void (*FieldProcPtr)(StringPtr, Ptr, short, void*);

pascal void CallDoToField(StringPtr fieldName, void* fieldAddr, short fieldType,
											void* staticLink, FieldProcPtr DoToField) = {
											
	0x205F,		// 		MOVE.L	(SP)+,A0	; get call address
	0x4A97,		// 		TST.L	(SP)		; see if static link is 0
	0x6602,		//		BNE.S	@0			; branch if not
	0x588F,		//		ADDQ.L	#4,SP		; if so, then pop it off the stack
	0x4E90		// @0	JSR		(A0)		; call the procedure
};

/*
provide a more convenient failure handler syntax (courtesy of Jack Palevich):

	TRY
		statement
	EXCEPT
		handler
	ENDTRY
	
There are some restrictions, however.

	You can't exit from the TRY part of the construct with a goto or return.  The reason 
	is that Success is called within the ENDTRY macro and if you fail to call it before 
	exiting, then you will get an error from MacApp.
*/

static OSErr gFailError;
static long gFailMessage;

pascal void StandardHandler(OSErr err, long message, void* Handler_StaticLink)
{
	gFailError = err;
	gFailMessage = message;
	longjmp((jmp_buf)Handler_StaticLink, 1);
}


#define TRY \
{ \
	jmp_buf errorBuf; \
	if (!setjmp(errorBuf)) { \
		FailInfo fi; \
		CatchFailures(&fi, StandardHandler, errorBuf);
	
		
#define EXCEPT \
		Success(&fi); \
		} \
	else {
	
	
#define ENDTRY \
		Failure(gFailError, gFailMessage); \
		} \
	}
	
	
#endif
