{ UObject.p }
{ Copyright  1989  Symantec Corporation.  All rights reserved. }
{ Copyright  1984-1989  Apple Computer, Inc.  All rights reserved. }

unit UObject;

interface


	uses
		Traps, SysEqu, ULoMem, UMacAppUtilities, UFailure, UDebug, UPatch, UMemory;

	const
		kFailNone = 1;
		kFailAbstract = 2;
		kFailCoercion = 3;
		kFailMethNotFound = 4;

		kInvalidObj = '*Not an object*';	{ return value from LookupObjName if not an}
{														 object}
		kNilClass = 0;					{ Value for superclass of the root object }

	type

		ObjClassID = kNilClass..16766; 	{ Object Class identifier. The first two}
{														 bytes of each object contain the class of}
{														 the object as an ObjClassID. }
		ObjClassIDPtr = ^ObjClassID;			{ Preferred }
		ObjClassIDHandle = ^ObjClassIDPtr;		{ Preferred }
		PObjClassID = ObjClassIDPtr;		{ Left in for compatibility (2.0) }
		HObjClassID = ObjClassIDHandle; 	{ Left in for compatibility (2.0) }

{--------------------------------------------------------------------------------------------------}
			{ Definition of the system's root object }


		TObject = object
				function TObject.Clone: TObject;
				{ Makes a duplicate copy of SELF. The default calls SELF.ShallowClone, which makes a}
{				literal copy of instance variables but does not attempt to clone owned objects. A}
{				subclass which owns other objects should override this to clone the owned objects}
{				and data structures as well. }

				procedure TObject.Fields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: INTEGER));
				{ Calls DoToField once for each instance variable in SELF. fieldName is a string}
{				representing the name of the variable. fieldAddr is the address of that variable.}
{				fieldType is one of the types defined in UMacAppUtilities. It is legal to use a}
{				local variable to hold data and pass the address of that variable to DoToField as}
{				if it were part of the object's instance variables. By convention, the first call}
{				to DoToField should have the class name as the fieldName, NIL as the fieldAddr, and}
{				bClass as the fieldType. Each subclass should override this method, particularly}
{				with qDebug on, dump its own information, then call INHERITED. }

				procedure TObject.DynamicFields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: integer));
				{ Used by the Inspector and the Debugger to display the contents of this class's}
{				dynamic area. Defaults to nothing.  Override if your object uses the dynamic area}
{				See TList for an example. }

				procedure TObject.ForAllSubClassesDo (procedure DoToSubClass (theClass: ObjClassID));
				{ Calls DoToSubClass for each subclass of SELF. The order of traversal is arbitrary;}
{				see EachSubClass global procedure. }

				procedure TObject.ForAllSuperClassesDo (procedure DoToSubClass (theClass: ObjClassID));
				{ Calls DoToSubClass for each superclass of SELF. The order of traversal is the order}
{				of ancestry starting from the class of SELF; see EachSuperClass global procedure. }

				procedure TObject.Free;
				{ Called to dispose of an object. Gives object a chance to cleanup after itself.}
{				Default simply calls SELF.ShallowFree, which makes no attempt to free instance}
{				variables. Should be overridden by any class which allocates space or owns other}
{				objects in its instance variables.. Be sure to call INHERITED! }

				function TObject.GetClass: ObjClassID;
				{ Returns the class ID of SELF. }

				procedure TObject.GetClassName (var clName: MAName);
				{ Returns the class name of SELF. }

				function TObject.GetClassSize: Size;
				{ Returns the basic instantiation size. i.e. the size in bytes of a newly created}
{				object of this class }

				function TObject.GetDynamicPtr: Ptr;
				{ Our objects have a dynamic area available. In 2.0 it is at the end of the object}
{				handle but it may not always be so.  Returns a direct heap pointer to the start}
{				of the dynamic area.  Use with caution as this pointer can be invalidated if the}
{				heap compacts. }

				function TObject.GetDynamicSize: Size;
				{ Our objects can have a dynamic area that can be allocated with a SetDynamicSize}
{				call.  Returns the size of the dynamic area in bytes. Preferred. }

				function TObject.GetInstanceSize: Size;
				{ Our objects have a dynamic area after all the fixed fields. Returns the current}
{				size in bytes of this instance accounting for the dynamic area.}
{				Left in for compatibility (2.0)  }

				function GetSuperClass: ObjClassID;
				{ Returns immediate superclass of class. Returns kNilClass for TObject. If we ever}
{				get MI this will have to be an enumerator. }

				procedure TObject.GetInspectorName (var inspectorName: Str255);
				{ Returns additional information useful to the inspector. For instance (ha ha!) a}
{				TDocument could give its title. }

				procedure TObject.Initialize;
				{ Call this to put a newly created object into a known state. Subclasses should}
{				override it and call INHERITED. Typically used to put the object into a state where}
{				it can be safely FREED. Defaults to nothing. }

				procedure TObject.Inspect;
				{ Called from the debugger. Displays the fields of the object in the writeln window}
{				by calling the object's Fields method, passing InspectField as the DoToField}
{				routine. You can override this if you want to display any additional information in}
{				the writeln window when an object is inspected with the MacApp debugger. NOTE: this}
{				is not currently called from the "Inspector".}

				procedure TObject.IObject;
				{ The ancestral initializer. Should be called in the I<ClassName> chain. i.e. IView}
{				-> IEvtHandler -> IOBJECT }

				function TObject.IsMemberClass (testClass: ObjClassID): BOOLEAN;
				{ True if I am a MEMBER of the testClass. i.e. the same class OR a subclass. }

				function TObject.IsSameClass (testClass: ObjClassID): BOOLEAN;
				{ True if I belong to the immediate class. }

				function TObject.Lock (lockIt: BOOLEAN): BOOLEAN;
				{ Locks down or unlocks an object AND its dynamic area if any. Returns old state.}
{				Since objects can float around in memory some occasions may arise where they need}
{				to be locked down (The Inspect method is an example: since addresses of instance}
{				variables are being passed, the object had better not move during the Inspect!) }
{				Typically the old state is saved in a local and re-applied when ascending the}
{				call chain. Don't forget to re-apply the old state from failure handlers.}
{				_PLEASE_ use this sparingly and don't just go locking stuff down willy-nilly just}
{				because you might be afraid of handles. }

				procedure TObject.SetDynamicSize (newSize: Size);
				{ Set this instance's Dynamic Area size in bytes.  The dynamic area starts out}
{				unallocated. Signals Failure on memory error.  The size applies only to the size}
{				of the Dynamic Area. Preferred. }

				procedure TObject.SetInstanceSize (newSize: Size);
				{ Set this instance's size in bytes. Thus changing the size of the dynamic area.}
{				Fails on attempts to set a size smaller than instantiation size and on memory}
{				error.   Left in for compatibility (2.0) }

				function TObject.ShallowClone: TObject;
				{ Lowest level method for copying an object; should not be overridden except in very}
{				unusual cases. Simply calls HandToHand to copy the object data. }

				procedure TObject.ShallowFree;
				{ Lowest level method for freeing an object; should not be overridden except in very}
{				unusual cases. Simply calls Dispose (the Pascal builtin) to free the object. }

			end;

		TObjectPtr = ^TObject; 			{ Preferred }
		TObjectHandle = ^TObjectPtr;			{ Preferred }
		PTObject = TObjectPtr;			{ Left in for compatibility (2.0) }
		HTObject = TObjectHandle;		{ Left in for compatibility (2.0) }

{--------------------------------------------------------------------------------------------------}
			{ Utility routines to support objects }

	function AddNewObjectsToInspector (add: BOOLEAN): BOOLEAN;
		{ Set to true (default) to automatically add all newly created objects to the inspector.}
{		Function returns old setting. }

	function AllocateObjectsFromPerm (allocateFromPerm: BOOLEAN): BOOLEAN;
		{ Set to TRUE to make object allocation calls use Permanent memory (the default).}
{		Set to false to make object allocation calls use temporary memory (if they cannot be}
{		allowed to fail).  Returns old state }

	function DisciplineMethodCalls (discipline: BOOLEAN): BOOLEAN;
		{ Set to true to discipline all method calls that go through the dispatcher. Function returns}
{		old setting. }

	procedure EachClassDo (procedure DoToClass (theClass: ObjClassID));
		{ Calls DoToClass for each class in the application }

	procedure EachSubClassDo (testClass: ObjClassID; procedure DoToClass (theClass: ObjClassID));
		{ Calls DoToClass for subclass of the test class. Uses EachClassDo, so the order of iteration}
{		is not based on the class hierarchy but on the sequence of class IDs in the class table,}
{		Sorry }

	procedure EachSuperClassDo (testClass: ObjClassID; procedure DoToClass (theClass: ObjClassID));
		{ Calls DoToClass for superclass of the test class. Order of iteration is in order of}
{		ancestry. }

	procedure FailNonObject (obj: univ TObject);
		{ called by dispatcher to discipline an object. Invokes failure if parameter is not IsObject}
{		}

	procedure FreeIfObject (obj: TObject);
		{ IF obj <> NIL THEN obj.Free; useful for freeing an object that might sometimes be NIL. }

	procedure FreeObject (obj: TObject);
		{ Synonym for FreeIfObject. Left in for compatibility (2.0) }

	function GetClassSizeFromID (classID: ObjClassID): Size;
		{ Given an object id, return the class's instantiation size. }

	procedure GetClassNameFromID (classID: ObjClassID; var clName: MAName);
		{ Given an object id, return the object's name. ??? add error checking ??? }

	function GetClassIDFromName (clName: MAName): ObjClassID;
		{ Given an object name, return its id. }

	function GetClassID (obj: TObject): ObjClassID;
		{ Given an object, return the object's ClassID. }

	function GetSuperClassID (objID: ObjClassID): ObjClassID;
		{ returns immediate superclass of class. Returns kNilClass for TObject. If we ever get MI}
{		this will have to be an enumerator. }

	procedure IDUobject;
		{ Writeln UObject compile date }

	procedure InitUObject;
		{ Essential one time initialization for this unit }

	procedure InspectObject (obj: TObject);
		{ Called by debugger to inspect an object. NOTE: not currently called by the "inspector" }

	function IsObject (obj: univ TObject): BOOLEAN;
		{ Debugging check. Returns true if obj references a real object, false if obj is NIL or a}
{		non-object reference. This is not an absloutely sure-fire test of objectness, but does}
{		validate the following: (1) is it a handle? (2) is the handle non-purgeable? (3) is the}
{		class ID valid? (4) is the handle size at least as large as the class size? }

	function IsClassIDMemberClass (testClass: ObjClassID; superClass: ObjClassID): BOOLEAN;
		{ tests the testClass for MEMBERship in the superclass }

	function IsMemberClassID (obj: TObject; objID: ObjClassID): BOOLEAN;
	   { Returns true if obj references the class or a subclass of the class represented by objID. }

	procedure OBJFail (error: INTEGER);
		{ Entered due (presumably) to some object failing a test for objectness. There are two codes}
{		which result in a special message: kFailCoercion, which means that an object could not be}
{		coerced to a class due to not being a subclass of that class; and kFailMethNotFound, which}
{		means a method was called for a class, but that method is not defined for the class. Other}
{		codes are allowed but are not given special treatment. In qDebug mode a ProgramBreak}
{		preceeds the Failure. }

	function NewObjectByClassId (classID: ObjClassID): TObject;
		{ Creates an object of the given class id }

	function NewObjectByClassName (className: MAName): TObject;
		{ Creates an object of the given class name }

	function VerboseIsObject (obj: univ TObject): BOOLEAN;
		{ Debugging check. Indicate whether the parameter appears to really be an object and emit}
{		diagnostics if it is not }

	procedure WrLblField (fieldName: Str255; fieldAddr: Ptr; fieldType: INTEGER);
		{ Writes the given field in the writeln window in the form: WRITE(fieldName, '=',}
{		valueAsString).}

implementation


{--------------------------------------------------------------------------------------------------}

	type

		HClassIdTable = ^PClassIdTable;
		PClassIdTable = ^ClassIdTable;
		ClassIdTable = array[1..16000] of ObjClassId; { Actually variable size }

{--------------------------------------------------------------------------------------------------}

	var
		pNoOfOrderedClasses: INTEGER;
		pOrderedClassIds: HClassIdTable;
		pClassTable: Handle; 							{handle to class table}
		pTObjectClassID: ObjClassId; 					{ClassID of the Root class}

		pAddNewObjectsToInspector: BOOLEAN;
		{$Push}
		{$Z+}
		pDisciplineMethodCalls: BOOLEAN;				{ Discipline method calls }
		{$Pop}
		pInspectLinePos: INTEGER;						{ Used to do line breaks when inspecting fields. }

		pAllocateObjectsFromPerm: BOOLEAN;	{ Used to track whether to allocate objects from permanent memory or not . }

{--------------------------------------------------------------------------------------------------}

	procedure OrderClassIdsByName;
	FORWARD;

	function IsClassIDMemberClass (testClass: ObjClassId; superClass: ObjClassId): BOOLEAN;
	EXTERNAL;

	procedure AddObjectToInspector (theObject: TObject);
	EXTERNAL;
{ Defined in UInspector.p }

	procedure RemoveObjectFromInspector (theObject: TObject);
	EXTERNAL;
{ Defined in UInspector.p }

{--------------------------------------------------------------------------------------------------}

{$S MAObjectRes}
	function AddNewObjectsToInspector (add: BOOLEAN): BOOLEAN;
{$Push}
 {$IFC qTrace}
 {$N++}
 {$ENDC}
{ Set to true (default) to automatically add all newly created objects to the inspector.}
{Function returns old setting. }

	begin
		AddNewObjectsToInspector := pAddNewObjectsToInspector;
		pAddNewObjectsToInspector := add;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$S MAObjectRes}

	function AllocateObjectsFromPerm (allocateFromPerm: BOOLEAN): BOOLEAN;

	begin
		AllocateObjectsFromPerm := pAllocateObjectsFromPerm;
		pAllocateObjectsFromPerm := allocateFromPerm;
	end;

{--------------------------------------------------------------------------------------------------}

{$S MAObjectRes}
	function DisciplineMethodCalls (discipline: BOOLEAN): BOOLEAN;
{ Set to true to discipline all method calls that go through the dispatcher. Function returns}
{old setting.  }

	begin
		DisciplineMethodCalls := pDisciplineMethodCalls;
		pDisciplineMethodCalls := discipline;
	end;

{--------------------------------------------------------------------------------------------------}

{$S MAObjectRes}
	procedure EachClassDo (procedure DoToClass (theClass: ObjClassId));
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{ Calls DoToClass for each class in the application }

		var
			tableSize: INTEGER;
			tableOffset: INTEGER;

	begin
	{offset to first jump table entry in %_MethTables segment}
		tableOffset := PInteger(pClassTable^)^ + 34;
	{add # of jump table entries in %_MethTables segment}
		tableSize := tableOffset + PInteger(Ord(pClassTable^) + 2)^ * 8;
	{ loop on all Classes }
		while tableOffset <> tableSize do
			begin
			{ make sure this is a real Class Info Proc and not the %_InitObj routine }
				if PInteger(PLongint(GetA5 + 2 + tableOffset)^)^ <> $4E75 then
					DoToClass(ObjClassId(tableOffset));
			{ go on to next class ID (jump table entry) }
				tableOffset := tableOffset + 8;
			end;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}

{$S MAObjectRes}
	procedure EachSubClassDo (testClass: ObjClassId; procedure DoToClass (theClass: ObjClassId));
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{ Calls DoToClass for subclass of the test class }

		procedure DoToCandidateClass (theClass: ObjClassId);

		begin
			if (theClass <> testClass) & IsClassIDMemberClass(theClass, testClass) then
				DoToClass(theClass);
		end;

	begin
		EachClassDo(DoToCandidateClass);
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}

{$S MAObjectRes}
	procedure EachSuperClassDo (testClass: ObjClassId; procedure DoToClass (theClass: ObjClassId));
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{ Calls DoToClass for superclass of the test class }

		var
			theSuperClass: ObjClassId;

	begin
		theSuperClass := GetSuperClassID(testClass);
		while theSuperClass <> kNilClass do
			begin
				DoToClass(theSuperClass);
				theSuperClass := GetSuperClassID(theSuperClass);
			end;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}

{$S MAObjectRes}
	procedure FailNonObject (obj: univ TObject);
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{ called by dispatcher to discipline an object.  Invokes failure if parameter is not IsObject }

	begin
		if not IsObject(obj) then
			begin
		{$IFC qDebug}
				if VerboseIsObject(obj) then
					;					{ show why }
				WrLblHexLongint('Object that failed discipline', ord(obj));
				WriteLn;
				ProgramBreak('');
		{$ENDC}
				Failure(minErr, 0); 							{ ??? need to assign a message }
			end;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}

{$S MAObjectRes}
	procedure FreeIfObject (obj: TObject);
{$Push}
 {$IFC qTrace}
 {$N++}
 {$ENDC}
{ IF obj <> NIL THEN obj.Free; useful for freeing an object that might}
{sometimes be NIL. }

	begin
		if obj <> nil then
			begin
		{$IFC qDebug}
				FailNonObject(obj);
		{$ENDC}
				obj.Free;
			end;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}

{$S MAObjectRes}
	procedure FreeObject (obj: TObject);
{$Push}
 {$IFC qTrace}
 {$N++}
 {$ENDC}
{ Synonym for FreeIfObject.  Left in for backward compatibility }

	begin
		FreeIfObject(obj);
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}

{$S MAObjectRes}
	function GetClassID (obj: TObject): ObjClassId;
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{ Given an object, return the object's ClassID. }

	begin
	{$Ifc qDebug}
		FailNonObject(obj);
	{$Endc}
		GetClassID := ObjClassId(HInteger(obj)^^);
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}

{$S MAObjectRes}
	function GetClassIDFromName (clName: MAName): ObjClassId;
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{$R-}
{$V-}
{ Given an object name, return its id. }

		var
			high, low, index: INTEGER;
			nameFromTable: MAName;
			compareResult: INTEGER;
			id: INTEGER;

	begin
		UprMAName(clName);
		if pNoOfOrderedClasses > 0 then
			begin
				low := 1;
				high := pNoOfOrderedClasses;
				repeat
					index := BSR(low + high, 1);				{ (low + high) DIV 2 }
					id := pOrderedClassIds^^[index];
					GetClassNameFromID(id, nameFromTable);
					UprMAName(nameFromTable);
					compareResult := CompareStrings(clName, nameFromTable);
					if compareResult = 0 then
						begin
							GetClassIDFromName := id;
							EXIT(GetClassIDFromName);
						end;
					if compareResult < 0 then
						high := index - 1
					else
						low := index + 1;
				until low > high;
			end;

	{$IFC qDebug}
		ProgramBreak(Concat('Can''t find class name ', clName));
	{$ENDC qDebug}
		GetClassIDFromName := kNilClass;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}

{$S MAObjectRes}
	procedure GetClassNameFromID (classID: ObjClassId; var clName: MAName);
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{$R-}
{$V-}
{ Given an object id, return the object's name.  ??? add error checking ??? }

		var
			classPtr: Ptr;

	begin
		if (classID = kNilClass) | ODD(classID) then
			clName := kInvalidObj
		else
			begin
				classPtr := Ptr(PLongint(classID + GetA5 + 2)^);
				clName := StringPtr(Ord(classPtr) + 8 + PInteger(Ord(classPtr) + 6)^ * 4)^;
			end;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}

{$S MAObjectRes}
	function GetClassSizeFromID (classID: ObjClassID): Size;
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{ Given an object id, return the class's instantiation size. }

	begin
		GetClassSizeFromId := LongInt(PInteger(PLongint(classID + GetA5 + 2)^ + 4)^);
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}

{$S MAObjectRes}
	function GetSuperClassID (objID: ObjClassId): ObjClassId;
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{$R-}
{$V-}
{ returns immediate superclass of class. Returns kNilClass for TObject }

	begin
		if objID <> kNilClass then
			GetSuperClassID := PInteger(PLongint(objID + GetA5 + 2)^ + 2)^
		else
			GetSuperClassID := kNilClass;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}

{$S MADebug}
	procedure IDUobject;
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{ Writeln UObject compile date }

	begin
		WriteLn('UObject of ', COMPDATE, ' @ ', COMPTIME);
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}


	procedure %_InitObj;
	external;
{ This routine is only present to force the Segment Loader to load the %_MethTables.}
{ It had better be the only non-Class Info Proc in that segment!}


{--------------------------------------------------------------------------------------------------}

{$S MAInit}
	procedure InitUObject;
{ Essential one time initialization for this unit }

	begin
	{$IFC qInspector}
		pAddNewObjectsToInspector := TRUE;
	{$EndC}
	{$IFC qDebug}
		pDisciplineMethodCalls := TRUE;
	{$ENDC qDebug}
	{ Force the %_MethTables segment to be loaded and locked }
		%_InitObj;
	{ Obtain the handle to the Class Info Procs }
		pClassTable := GetNamedResource('CODE', '%_MethTables');
	{$IFC qDebug}
		if pClassTable = nil then
			ProgramBreak('Can''t find %_MethTables segment');
	{$ENDC}

		OrderClassIdsByName;
		pTObjectClassID := GetClassIDFromName('TObject');
	end;

{--------------------------------------------------------------------------------------------------}

{$S MADebug}
	procedure InspectField (fieldName: Str255; fieldAddr: Ptr; fieldType: INTEGER);
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}

		var
			s: Str255;
			x: INTEGER;

	begin
		if fieldType <> bClass then
			begin
				FieldToString(fieldAddr, fieldType, s);
				x := LENGTH(fieldName) + 1 + LENGTH(s);
		{??? maybe a better solution would be to let the transcript do it's own word breaks}
		{$IFC qDebug}
				if pInspectLinePos + x + 2 >= DebugTranscriptWidth then
					begin
						WriteLn;
						pInspectLinePos := 0;
					end
				else if pInspectLinePos <> 0 then				{ If not at the start of a line }
					begin
						Write('  ');
						pInspectLinePos := pInspectLinePos + 2;
					end;
		{$ENDC}
				Write(fieldName, '=', s);
				pInspectLinePos := pInspectLinePos + x;
			end;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}

{$S MADebug}
	procedure InspectObject (obj: TObject);
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{$R-}
{$V-}

{ Called by debugger to inspect an object. }

		var
			oldState: BOOLEAN;

	begin
		if VerboseIsObject(obj) then
			begin
				oldState := obj.Lock(TRUE);
				obj.Inspect;
				WriteLn;
				oldState := obj.Lock(oldState);
			end
		else
			begin
				WritePtr(obj);
				WriteLn(' is not a TObject!');
			end;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}

{$S MAObjectRes}
	function IsObject (obj: univ TObject): BOOLEAN;
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{$R-}
{$V-}
{ Returns true if obj references a real object, false if obj is NIL or a non-object reference. }

	begin

		if (obj <> nil) & IsHandle(obj) & (Ptr(StripLong(Handle(obj)^)) <> nil) & IsClassIDMemberClass(HObjClassID(obj)^^, pTObjectClassID) & (GetHandleSize(Handle(obj)) >= GetClassSizeFromId(HObjClassID(obj)^^)) then
	{ Test for handle not purged since we don't allow purgeable objects (??? yet?, ever?) }
	{ Test objecthood }
			IsObject := TRUE
		else
			IsObject := FALSE;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}

{$S MAObjectRes}
	function IsMemberClassID (obj: TObject; objID: ObjClassId): BOOLEAN;
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{$R-}
{$V-}
{ Returns true if obj references the class or a subclass of the class represented by objID. }

	begin
	{$IFC qDebug}
		FailNonObject(obj);
	{$ENDC}
		if IsObject(obj) then
			IsMemberClassID := IsClassIDMemberClass(HObjClassID(obj)^^, objID)
		else
			IsMemberClassID := FALSE;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}

{$S MAObjectRes}
	function MakeNewInstance (classID: ObjClassId): TObject;
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{ makes objects for "new" calls.  Internal use only. }

		{$IFC qDebug}
		const
			initVal = $F1F1;	{ guaranteed to be odd at all byte boundaries }
		{$ENDC}

		var
		{$IFC qDebug}
			i: INTEGER;
			p: PInteger;
		{$ENDC}
			itsSize: INTEGER;
			obj: TObject;

	begin
		if classID <> 0 then
			begin
				itsSize := GetClassSizeFromId(classID);
		{$IFC qMacApp}
				Handle(obj) := NewPermHandle(itsSize);
		{$ELSEC}
				Handle(obj) := NewHandle(itsSize);
		{$ENDC}

				if obj <> nil then
					begin

			{$IFC qDebug}
			{Initialize the object to $F1F1F1F1...}
						p := PInteger(Handle(obj)^);
						for i := 1 to itsSize div 2 do
							begin
								p^ := initVal;
								p := PInteger(ord(p) + 2);
							end;
			{$ENDC}

			{ Install class ID into object }
						HObjClassID(obj)^^ := classID;

			{$IFC qInspector}
						if pAddNewObjectsToInspector then
							AddObjectToInspector(obj);
			{$ENDC}
					end;
				MakeNewInstance := obj;
			end
		else
			MakeNewInstance := nil;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}

{$S MAObjectRes}
	function NewObjectByClassId (classID: ObjClassId): TObject;
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{$R-}
{$V-}
{ Creates an object of the given class id }

		{$IFC qDebug}
		var
			i: INTEGER;					{must be first local and 2 bytes in size}
			s: MAName;
			className: MAName;
		{$ENDC}

	begin
	{$IFC qDebug}
		if gAskAboutAlloc & CanReadLn then
			begin

				GetCallersMethodName(s);
				GetClassNameFromID(classID, className);
				WriteLn('Within ', s, ', trying to make a ''', className, '''.');

				if ReadYesNo('     Return NIL (Y or N) [N]? ') then
					begin
						NewObjectByClassId := nil;
						EXIT(NewObjectByClassId);
					end;
			end;
	{$ENDC qDebug}

		NewObjectByClassId := MakeNewInstance(classID);
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}

{$S MAObjectRes}
	function NewObjectByClassName (className: MAName): TObject;
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{$R-}
{$V-}
{ Creates an object of the given class name }

		var
			classID: ObjClassId;
		{$IFC qDebug}
			i: INTEGER;					{must be first local and 2 bytes in size}
			p: PInteger;
			s: MAName;
		{$ENDC}

	begin
	{$IFC qDebug}
		if gAskAboutAlloc & CanReadLn then
			begin

				GetCallersMethodName(s);
				WriteLn('Within ', s, ', trying to make a ''', className, '''.');

				if ReadYesNo('     Return NIL (Y or N) [N]? ') then
					begin
						NewObjectByClassName := nil;
						EXIT(NewObjectByClassName);
					end;
			end;
	{$ENDC qDebug}

		classID := GetClassIDFromName(className);
		NewObjectByClassName := MakeNewInstance(classID);
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}

{$S MAObjectRes}
	procedure OBJFail (error: INTEGER);
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}

	begin
	{$IFC qDebug}

		case error of
			kFailCoercion: 
				ProgramBreak('Object type coercion error.');
			kFailMethNotFound: 
				ProgramBreak('Method not found');
			otherwise
				begin
					WriteLn('Failure code: ', error);
					ProgramBreak('Object runtime failure. See UObject.p.');
				end;
		end;
	{$ENDC}
		Failure(minErr, 0); 								{ ??? need to assign a message }
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}

{$S MAObjectRes}
	procedure OrderClassIdsByName;
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{$R-}
{$V-}

		var
			aClassName: MAName;
			nameFromTable: MAName;
			high, low, index: INTEGER;
			id: ObjClassId;
			tableSize, tableCount: INTEGER;

	begin
		pNoOfOrderedClasses := 0;

{ pClassTable is a handle to the %_MethTables segment, which contains Class Info Procs laid}
{ out sequentially, plus the %_InitObj routine which is just an RTS.  We can go through all}
{ jump table entries for the segment and if the routine is *not* an RTS, we know it is a}
{ Class Info Proc - the name is at the end of the method list.}

		tableSize := PInteger(Ord(pClassTable^) + 2)^; 		{# of jump table entries in %_MethTables segment}
		id := PInteger(pClassTable^)^ + 34;					{offset to first jump table entry in %_MethTables segment}

		pOrderedClassIds := HClassIdTable(NewHandle(tableSize * Sizeof(ObjClassId)));
		{FailNIL(pOrderedClassIds);}

		for tableCount := 1 to tableSize do
			begin
			{ make sure this is a real Class Info Proc and not the %_InitObj routine }
				if PInteger(PLongint(GetA5 + 2 + id)^)^ <> $4E75 then
					begin
						GetClassNameFromID(ObjClassId(id), aClassName);
						UprMAName(aClassName);

						if pNoOfOrderedClasses = 0 then
							index := 1
						else
							begin
								low := 1;
								high := pNoOfOrderedClasses;
								repeat
									index := BSR(low + high, 1);		{ (low + high) DIV 2 }
									GetClassNameFromID(pOrderedClassIds^^[index], nameFromTable);
									UprMAName(nameFromTable);
									if aClassName < nameFromTable then
										high := index - 1
									else
										begin
											low := index + 1;
											index := index + 1;
										end;
								until low > high;
							end;

						if index <= pNoOfOrderedClasses then
							BlockMove(@pOrderedClassIds^^[index], @pOrderedClassIds^^[index + 1], (pNoOfOrderedClasses - index + 1) * sizeof(ObjClassId));
						pOrderedClassIds^^[index] := id;
						pNoOfOrderedClasses := pNoOfOrderedClasses + 1;
					end;

			{ go on to next class ID (jump table entry offset + 2) }
				id := id + 8;
			end;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$S MADebug}

	function VerboseIsObject (obj: univ TObject): BOOLEAN;

		var
			className: MAName;
			classSize: Size;
			instSize: Size;

	begin
		VerboseIsObject := FALSE;
		if VerboseIsHandle(obj) then
	{ Test for handle not purged since we don't allow purgeable objects (??? yet?, ever?) }
			if (Ptr(StripLong(Handle(obj)^)) = nil) then
				WriteLn('  That handle appears to be purged.')
			else if not IsClassIDMemberClass(GetClassID(obj), GetClassIDFromName('TObject')) then
				WriteLn('  That handle is not a subclass of TObject.')
			else if (GetHandleSize(Handle(obj)) < GetClassSizeFromId(GetClassID(obj))) then
				begin
					GetClassNameFromID(GetClassID(obj), className);
					classSize := GetClassSizeFromId(GetClassID(obj));
					instSize := GetHandleSize(Handle(obj));
					WriteLn('  That handle at: ', instSize : 1, ' bytes is smaller than a ', className, ' is supposed to be at: ', classSize : 1, ' bytes.');
				end
			else
				VerboseIsObject := TRUE;
	end;
{--------------------------------------------------------------------------------------------------}

{$S MADebug}
	procedure WrLblField (fieldName: Str255; fieldAddr: Ptr; fieldType: INTEGER);
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{ Writes the given field in the writeln window in the form:}
{WRITE(fieldName, '=', valueAsString).}

	begin
		pInspectLinePos := 0;
		InspectField(fieldName, fieldAddr, fieldType);
		pInspectLinePos := 0;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$Z+}
 {$IFC qTrace}
 {$N+}
 {$ENDC}

{$S Main}
	function %_OBCHK (obj: TObject; jumpTablePtr: Ptr): TObject;

{ LOW LEVEL routine called at run time verify object coercions.  It returns its obj}
{parameter if the parameter is nil or passes the membership test.  Otherwise it calls}
{ObjFail.  }

	begin
	{$IFC qDebug}
		if (obj <> nil) then
			FailNonObject(obj);
	{$ENDC}
		%_OBCHK := obj;
		if (obj <> nil) & (not IsClassIDMemberClass(HObjClassID(obj)^^, Ord(jumpTablePtr) - GetA5)) then
			OBJFail(kFailCoercion);
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$Z+}
 {$IFC qTrace}
 {$N++}
 {$ENDC}

{$S Main}
	procedure %_OBDISP (var obj: TObject);
{ LOW LEVEL routine called by DISPOSE(<object>); }

	begin
	{$Ifc qDebug}
		FailNonObject(obj);
	{$Endc}

	{$IFC qInspector}
		RemoveObjectFromInspector(obj);
	{$ENDC}

		DisposIfHandle(Handle(obj));
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}

{$S Main}
	procedure %_ObjError;
{$Push}
 {$Z+}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{ LOW LEVEL Error routine that ROM method dispatch routine jumps to if method not found}
{  Address of this routine is stuffed at lomem location MAErrProc at startup }

	begin
		OBJFail(kFailMethNotFound); 						{ Method Not Found }
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$Z+}
 {$IFC qTrace}
 {$N++}
 {$ENDC}

{$S Main}
	procedure %_OBNEW (var obj: TObject; jumpTablePtr: Ptr; itsSize: INTEGER);
{ LOW LEVEL routine called by NEW(<object>); }

		var
		{$IFC qDebug}
			i: INTEGER;					{must be first local and 2 bytes in size}
			p: PInteger;
			n: MAName;
			s: MAName;
		{$ENDC}
			classID: ObjClassId;

	begin
		classID := Ord(jumpTablePtr) - GetA5;
	{$IFC qDebug}
		if gAskAboutAlloc & CanReadLn then
			begin

				GetCallersMethodName(s);
				GetClassNameFromID(classID, n);
				WriteLn('Within ', s, ', trying to make a ''', n, '''.');

				if ReadYesNo('     Return NIL (Y or N) [N]? ') then
					begin
						obj := nil;
						EXIT(%_OBNEW);
					end;
			end;
	{$ENDC qDebug}

		obj := MakeNewInstance(classID);
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$Z+}
 {$IFC qTrace}
 {$N+}
 {$ENDC}

{$S Main}
	function %_OPTINOBJ (obj: TObject; jumpTablePtr: Ptr): BOOLEAN;

{ LOW LEVEL called to perform MEMBER function }

	begin
	{$IFC qDebug}
		if (obj <> nil) then
			FailNonObject(obj);
	{$ENDC}
		%_OPTINOBJ := (obj <> nil) & IsClassIDMemberClass(HObjClassID(obj)^^, Ord(jumpTablePtr) - GetA5);
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$P}
{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n-]}
{ UObject.TObject.p }
{ Copyright  1984-1990 by Apple Computer, Inc.  All rights reserved. }
{$IFC UNDEFINED qMacApp}
{$SETC qMacApp := FALSE}
{$ENDC}

{$Push}
 {$IFC NOT qDebugTheDebugger}
{$W+}
{$R-}
{$Init-}
{$OV-}
{$ENDC}

{--------------------------------------------------------------------------------------------------}
{$S MAObjectRes}

	function TObject.Clone;

	begin
		Clone := ShallowClone;
	end;

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$D+}
 {$ENDC}
{$S MAFields}

	procedure TObject.DynamicFields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: integer));

	begin
	{ ??? Would a nicer default be a formatted memory dump? }
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$D+}
 {$ENDC}
{$S MAFields}

	procedure TObject.Fields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: integer));

		const
			bObjClassID = bInteger;

		var
			myID: ObjClassID;
			mySize: Size;

	begin
		myID := GetClass;
		mySize := GetInstanceSize;
		DoToField('TObject', nil, bClass);
		DoToField('ObjClassID', @myID, bObjClassID);
		DoToField('Size in bytes', @mySize, bLongint);

		DynamicFields(DoToField);							{ Get the dynamic part inspected }

	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$S MAObjectRes}

	procedure TObject.ForAllSubClassesDo (procedure DoToSubClass (theClass: ObjClassID));

	begin
		EachSubClassDo(GetClass, DoToSubClass);
	end;

{--------------------------------------------------------------------------------------------------}
{$S MAObjectRes}

	procedure TObject.ForAllSuperClassesDo (procedure DoToSubClass (theClass: ObjClassID));

	begin
		EachSuperClassDo(GetClass, DoToSubClass);
	end;

{--------------------------------------------------------------------------------------------------}
{$S MAObjectRes}

	procedure TObject.Free;

	begin
		ShallowFree;
	end;

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$D+}
 {$ENDC}
{$S MAObjectRes}

	procedure TObject.GetClassName (var clName: MAName);

	begin
		GetClassNameFromID(GetClass, clName);
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$D+}
 {$ENDC}
{$S MAObjectRes}

	function TObject.GetClass: ObjClassID;

	begin
		GetClass := GetClassID(self);
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$D+}
 {$ENDC}
{$S MAObjectRes}

	function TObject.GetClassSize: Size;

	begin
		GetClassSize := GetClassSizeFromID(GetClass);
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$D+}
 {$ENDC}
{$S MAObjectRes}

	function TObject.GetDynamicPtr: Ptr;
		var
			classSize: Size;

	begin
		classSize := GetClassSize;

		if (GetHandleSize(Handle(self)) - classSize) > 0 then
			GetDynamicPtr := Ptr(striplong(Handle(self)^) + classSize)
		else
			GetDynamicPtr := nil;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$D+}
 {$ENDC}
{$S MAInspector}

	procedure TObject.GetInspectorName (var inspectorName: Str255);

	begin
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$D+}
 {$ENDC}
{$S MAObjectRes}

	function TObject.GetDynamicSize: Size;

	begin
		GetDynamicSize := GetHandleSize(Handle(self)) - GetClassSize;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$D+}
 {$ENDC}
{$S MAObjectRes}

	function TObject.GetInstanceSize: Size;

	begin
		GetInstanceSize := GetHandleSize(Handle(self));
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$D+}
 {$ENDC}
{$S MAObjectRes}

	function TObject.GetSuperClass: ObjClassID;

	begin
		GetSuperClass := GetSuperClassID(GetClass);
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$S MAObjectRes}

	procedure TObject.Initialize;

	begin
	end;

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$D+}
 {$ENDC}
{$S MADebug}

	procedure TObject.Inspect;

		var
			clName: MAName;

	begin
		GetClassName(clName);
		Write(clName, '     ');
		WritePtr(self);
		WriteLn;
		pInspectLinePos := 0;
		Fields(InspectField);
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$S MAObjectRes}

	procedure TObject.IObject;

	begin
		Initialize;
	end;

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$D+}
 {$ENDC}
{$S MAObjectRes}

	function TObject.IsSameClass (testClass: ObjClassID): BOOLEAN;

	begin
		IsSameClass := (GetClass = testClass);
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$D+}
 {$ENDC}
{$S MAObjectRes}

	function TObject.IsMemberClass (testClass: ObjClassID): BOOLEAN;

	begin
		IsMemberClass := IsClassIDMemberClass(GetClass, testClass);
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$D+}
 {$ENDC}
{$W+}
{$R-}
{$OV-}
{$S MAObjectRes}

	function TObject.Lock (lockIt: BOOLEAN): BOOLEAN;

		var
			wasLocked: BOOLEAN;

	begin
		wasLocked := IsHandleLocked(self);
		Lock := wasLocked;
		if wasLocked <> lockIt then
			if lockIt then
				HLock(Handle(self))
			else
				HUnLock(Handle(self))
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$D+}
 {$ENDC}
{$S MAObjectRes}

	procedure TObject.SetInstanceSize (newSize: Size);

	begin
	{$IFC qDebug}
		if newSize < GetClassSize then
			ProgramBreak('In TObject.SetInstanceSize trying to set instance size less than class minimum');
	{$Endc}
		if qMacApp & pAllocateObjectsFromPerm then
			SetPermHandleSize(Handle(self), newSize)
		else
			SetHandleSize(Handle(self), newSize);
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$D+}
 {$ENDC}
{$S MAObjectRes}

	procedure TObject.SetDynamicSize (newSize: Size);

	begin
		if qMacApp & pAllocateObjectsFromPerm then
			SetPermHandleSize(Handle(self), newSize + GetClassSize)
		else
			SetHandleSize(Handle(self), newSize + GetClassSize);
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$S MAObjectRes}

	function TObject.ShallowClone: TObject;

		var
			result: TObject;
			oldPerm: BOOLEAN;
			err: OSErr;

		{$IFC qDebug}
			s: MAName;
		{$ENDC}

	begin
	{$IFC qDebug}
		if gAskAboutAlloc then
			begin
				GetCallersMethodName(s);

				if s = 'TOBJECT.CLONE' then 					{ report about caller of TObject.Clone,}
{														 instead }
					GetMethodName(LongIntPtr(GetCurStackFramePtr)^ + 4, s);

				Write('Within ', s, ': trying to clone a: ''');
				GetClassName(s);
				WriteLn(s, '''.');

				if ReadYesNo('     Return NIL (Y or N) [N]? ') then
					begin
						ShallowClone := nil;
						EXIT(ShallowClone);
					end;
			end;
	{$ENDC}

		result := self;

	{$IFC qMacApp}
		oldPerm := PermAllocation(TRUE);
	{$ENDC}

		err := HandToHand(Handle(result));

	{$IFC qMacApp}
		oldPerm := PermAllocation(oldPerm);
	{$ENDC}

		if err <> noErr then
			result := nil
		else
			begin
{$IFC qInspector}
				if pAddNewObjectsToInspector then
					AddObjectToInspector(result);
{$ENDC}
			end;

		ShallowClone := result;
	end;

{--------------------------------------------------------------------------------------------------}
{$S MAObjectRes}

	procedure TObject.ShallowFree;

	begin
		Dispose(self);
	end;

{$Pop}
end. {UObject.p}