Unit UItem;

INTERFACE

USES
	UMacApp,
	ToolUtils,
	Resources,
	Packages,
	UPrinting,
	UTEView,
	UDialog,
	UGridView,
	UCRCCards,
	UTextHandler,
	UCard,
	UDesignDocument,
	UMyViews,
	UMyCommands;
	
TYPE
	TItem = OBJECT(TObject)
		fName: str255;
		fCard: TCard;
		fTextHandler: TTextHandler;
		
		PROCEDURE TItem.IItem(theCard: TCard; theName: str255);
		PROCEDURE TItem.StripIllegalChars(VAR theStr: str255);
		FUNCTION TItem.GetTheName: str255;
		PROCEDURE TItem.SetTheName(theName: str255);
		FUNCTION TItem.GetTheItem: str255;
		FUNCTION TItem.GetTheCard: TCard;
		FUNCTION TItem.GetTextHandler: TTextHandler;
		PROCEDURE TItem.SetTextHandler(theTextHandler: TTextHandler);
		FUNCTION TItem.ReturnBytes: LONGINT;
		PROCEDURE TItem.ReadTheItem(aRefNum: INTEGER; theCard: TCard);
		PROCEDURE TItem.WriteTheItem(aRefNum: INTEGER);
		PROCEDURE TItem.WriteDeclaration(theTEView: TMyTEView);
		PROCEDURE TItem.CheckForPrefix;
		PROCEDURE TItem.Fields(PROCEDURE DoToField(fieldName: Str255; fieldAddr: Ptr; 
				fieldType: INTEGER)); OVERRIDE;
	END;
	
	TFieldItem = OBJECT(TItem)
		fItemType: str255;
		fGetAccessor: TGetAccessorItem;
		fSetAccessor: TSetAccessorItem;
		fFieldsTypeConst: str255;
		fPassByAddress: BOOLEAN;
		
		PROCEDURE TFieldItem.IFieldItem(theCard: TCard; theName, theTypeStr: str255);
		FUNCTION TFieldItem.GetItemType: str255;
		PROCEDURE TFieldItem.SetItemType(theItemType: str255);
		FUNCTION TFieldItem.GetTheItem: str255; OVERRIDE;
		FUNCTION TFieldItem.GetTheFieldStr: str255;
		PROCEDURE TFieldItem.SetPassByAddress;
		FUNCTION TFieldItem.GetPassByAddress: BOOLEAN;
		FUNCTION TFieldItem.GetTheFieldsTypeConst: str255;
		PROCEDURE TFieldItem.SetTheFieldsTypeConst;
		PROCEDURE TFieldItem.GetAccessors(VAR theGetter: TGetAccessorItem;
																			VAR theSetter: TSetAccessorItem);
		PROCEDURE TFieldItem.SetGetAccessor(theGetter: TGetAccessorItem);
		PROCEDURE TFieldItem.SetSetAccessor(theSetter: TSetAccessorItem);
		PROCEDURE TFieldItem.MakeGetNIL;
		PROCEDURE TFieldItem.MakeSetNIL;
		PROCEDURE TFieldItem.CheckForPrefix; OVERRIDE;
		FUNCTION TFieldItem.GetTheDeclaration: str255;
		FUNCTION TFieldItem.GetTheImplDeclaration: str255;
		PROCEDURE TFieldItem.WriteDeclaration(theTEView: TMyTEView); OVERRIDE;
		FUNCTION TFieldItem.ReturnBytes: LONGINT; OVERRIDE;
		PROCEDURE TFieldItem.ReadTheItem(aRefNum: INTEGER; theCard: TCard); OVERRIDE;
		PROCEDURE TFieldItem.WriteTheItem(aRefNum: INTEGER); OVERRIDE;
		PROCEDURE TFieldItem.Fields(PROCEDURE DoToField(fieldName: Str255; fieldAddr: Ptr; 
				fieldType: INTEGER)); OVERRIDE;
	END;
	
	TMethodItem = OBJECT(TItem)
		fOVERRIDE: boolean;
	
		FUNCTION TMethodItem.GetOVERRIDE: boolean;
		PROCEDURE TMethodItem.SetOVERRIDE(overrideIt: boolean);
		PROCEDURE TMethodItem.IMethodItem(theCard: TCard; theName: str255; 
																			makeItOVERRIDE: boolean);
		PROCEDURE TMethodItem.CheckForPrefix; OVERRIDE;
		FUNCTION TMethodItem.ReturnBytes: LONGINT; OVERRIDE;
		FUNCTION TMethodItem.GetNumRealMethods: integer;
		PROCEDURE TMethodItem.ReadTheItem(aRefNum: INTEGER; theCard: TCard); OVERRIDE;
		PROCEDURE TMethodItem.WriteTheItem(aRefNum: INTEGER); OVERRIDE;
		PROCEDURE TMethodItem.WriteDeclaration(theTEView: TMyTEView); OVERRIDE;
		FUNCTION TMethodItem.GetTheDeclaration: str255;
		FUNCTION TMethodItem.GetTheImplDeclaration: str255;
		PROCEDURE TMethodItem.WriteInlineProcedures(theTEView: TMyTEView);
		PROCEDURE TMethodItem.WriteImplementation(theTEView: TMyTEView);
		FUNCTION TMethodItem.GetTheCodeLine: str255;
		PROCEDURE TMethodItem.StripIllegalChars(VAR theStr: str255); OVERRIDE;
		PROCEDURE TMethodItem.Fields(PROCEDURE DoToField(fieldName: Str255; fieldAddr: Ptr; 
				fieldType: INTEGER)); OVERRIDE;
	END;
	
	TAccessorItem = OBJECT(TMethodItem)
		fFunction: boolean;
		fFieldAccessed: TFieldItem;
		
		PROCEDURE TAccessorItem.IAccessorItem(theCard: TCard; 
										theName: str255; ourField: TFieldItem);
		PROCEDURE TAccessorItem.SetFieldReference(theField: TFieldItem);
		FUNCTION TAccessorItem.GetFieldAccessed: TFieldItem;
		PROCEDURE TAccessorItem.SetFunction(makeItAFunction: boolean);
		FUNCTION TAccessorItem.GetTheItem: str255; OVERRIDE;
		FUNCTION TAccessorItem.GetTheDeclaration: str255; OVERRIDE;
		FUNCTION TAccessorItem.GetTheImplDeclaration: str255; OVERRIDE;
		FUNCTION TAccessorItem.GetParameter: str255;
		PROCEDURE TAccessorItem.WriteImplementation(theTEView: TMyTEView); OVERRIDE;
		FUNCTION TAccessorItem.ReturnBytes: LONGINT; OVERRIDE;
		FUNCTION TAccessorItem.GetNumRealMethods: integer; OVERRIDE;
		PROCEDURE TAccessorItem.ReadTheItem(aRefNum: INTEGER; theCard: TCard); OVERRIDE;
		PROCEDURE TAccessorItem.WriteTheItem(aRefNum: INTEGER); OVERRIDE;
		PROCEDURE TAccessorItem.Fields(PROCEDURE DoToField(fieldName: Str255; fieldAddr: Ptr; 
				fieldType: INTEGER)); OVERRIDE;
	END;
	
	TGetAccessorItem = OBJECT(TAccessorItem)
		FUNCTION TGetAccessorItem.GetTheCodeLine: str255; OVERRIDE;
		FUNCTION TGetAccessorItem.GetTheItem: str255; OVERRIDE;
		FUNCTION TGetAccessorItem.GetTheDeclaration: str255; OVERRIDE;
		FUNCTION TGetAccessorItem.GetTheImplDeclaration: str255; OVERRIDE;
		PROCEDURE TGetAccessorItem.WriteInlineProcedures(theTEView: TMyTEView); OVERRIDE;
		PROCEDURE TGetAccessorItem.CheckForPrefix; OVERRIDE;
		PROCEDURE TGetAccessorItem.Free; OVERRIDE;
	END;
	
	TSetAccessorItem = OBJECT(TAccessorItem)
		FUNCTION TSetAccessorItem.GetTheCodeLine: str255; OVERRIDE;
		FUNCTION TSetAccessorItem.GetTheItem: str255; OVERRIDE;
		FUNCTION TSetAccessorItem.GetTheDeclaration: str255; OVERRIDE;
		FUNCTION TSetAccessorItem.GetTheImplDeclaration: str255; OVERRIDE;
		PROCEDURE TSetAccessorItem.WriteInlineProcedures(theTEView: TMyTEView); OVERRIDE;
		PROCEDURE TSetAccessorItem.CheckForPrefix; OVERRIDE;
		PROCEDURE TSetAccessorItem.Free; OVERRIDE;
	END;
	
	TFieldsMethodItem = OBJECT(TMethodItem)
		FUNCTION TFieldsMethodItem.GetTheDeclaration: str255; OVERRIDE;
		FUNCTION TFieldsMethodItem.GetTheImplDeclaration: str255; OVERRIDE;
		PROCEDURE TFieldsMethodItem.WriteImplementation(theTEView: TMyTEView); OVERRIDE;
		FUNCTION TFieldsMethodItem.GetNumRealMethods: integer; OVERRIDE;
		FUNCTION TFieldsMethodItem.ReturnBytes: LONGINT; OVERRIDE;
		PROCEDURE TFieldsMethodItem.ReadTheItem(aRefNum: INTEGER; theCard: TCard); OVERRIDE;
		PROCEDURE TFieldsMethodItem.WriteTheItem(aRefNum: INTEGER); OVERRIDE;
	END;
	
	TCollaboratorItem = OBJECT(TFieldItem)
		PROCEDURE TCollaboratorItem.ICollaboratorItem(theCard: TCard; theName: str255);
		PROCEDURE TCollaboratorItem.CheckForPrefix; OVERRIDE;
		FUNCTION TCollaboratorItem.GetTheItem: str255; OVERRIDE;
		FUNCTION TCollaboratorItem.GetTheFieldStr: str255; OVERRIDE;
		PROCEDURE TCollaboratorItem.SetTheFieldsTypeConst; OVERRIDE;
		FUNCTION TCollaboratorItem.GetTheDeclaration: str255; OVERRIDE;
		FUNCTION TCollaboratorItem.GetTheImplDeclaration: str255; OVERRIDE;
		FUNCTION TCollaboratorItem.ReturnBytes: LONGINT; OVERRIDE;
		PROCEDURE TCollaboratorItem.ReadTheItem(aRefNum: INTEGER; theCard: TCard); OVERRIDE;
		PROCEDURE TCollaboratorItem.WriteTheItem(aRefNum: INTEGER); OVERRIDE;
	END;
	
IMPLEMENTATION

{$I UItem.inc1.p}

END.