/************************************************************

Created: Thursday, October 19, 1989 at 11:10 PM
	Aliases.h
	C Interface to the Macintosh Libraries


	Copyright Apple Computer, Inc. 1989-90
	All rights reserved

************************************************************/


#ifndef __ALIASES__
#define __ALIASES__

#ifndef __TYPES__
#include <Types.h>
#endif
#ifndef __DIALOGS__
#include <Dialogs.h>
#endif
#ifndef __FILES__
#include <Files.h>
#endif

/**********************************************************************
 ***************			Constants		***************************
 *********************************************************************/

#define rAliasType		'alis'		/* Aliases are stored as resources of this type */

/* define alias resolution action rules mask */
enum {kARMmountVol = 0x00000001};                                           /* mount the volume automatically */
enum {kARMnoUI = 0x00000002};                                               /* no user interface allowed during resolution */
enum {kARMmultVols = 0x00000008};                                           /* search on multiple volumes */
enum {kARMsearch = 0x00000100};                                             /* search quickly */
enum {kARMsearchMore = 0x00000200};                                         /* search further */
enum {kARMsearchRelFirst = 0x00000400};                                     /* search target on a relative path first */

/*  define alias record information types */

enum {asiZoneName = -3};                                                    /* get zone name */
enum {asiServerName = -2};                                                  /* get server name */
enum {asiVolumeName = -1};                                                  /* get volume name */
enum {asiAliasName = 0};                                                    /* get aliased file/folder/volume name */
enum {asiParentName = 1};                                                   /* get parent folder name */

/**********************************************************************
 ***************			Data Structures 	***********************
 *********************************************************************/

/* define the alias record that will be the blackbox for the caller */
typedef struct
{
/* following two fields for application use */
	OSType 	userType;			/* appl stored type like creator type */
	unsigned short aliasSize;	/* alias record size in bytes, for appl usage */
	/* what follows is a variable length amount of data that is private */
} AliasRecord;

typedef AliasRecord *AliasPtr;		/* pointer to alias record */
typedef AliasRecord **AliasHandle;	/* handle to alias record */
typedef short AliasInfoType;		/* alias record information type */

typedef pascal Boolean (*AliasFilterProcPtr) (CInfoPBPtr cpbPtr, /*I*/
											Boolean *quitFlag,	 /*O*/
											Ptr yourDataPtr);	 /*I*/
											
#ifdef __cplusplus
extern "C" {
#endif

/**********************************************************************
 ***************			Public Interfaces	***********************
 *********************************************************************/

/****						High Level Routines 			****/

/* create a new alias between fromFile-target and return alias
   record handle */

pascal OSErr NewAlias (const FSSpec *fromFile,				/*I*/
								const FSSpec *target, 		/*I*/
								AliasHandle *alias) 		/*O*/
= {0x7002,0xA823};


/* create a minimal new alias for a target and return alias
   record handle */

pascal OSErr NewAliasMinimal (const FSSpec *target,		/*I*/
								AliasHandle *alias) 	/*O*/
= {0x7008,0xA823};


/* create a minimal new alias from a target fullpath (optional zone and server name)
	and return alias record handle */

pascal OSErr NewAliasMinimalFromFullpath (short fullpathLength, /*I*/
								const unsigned char *fullpath, 	/*I*/
								const Str31 zoneName,			/*I*/
								const Str31 serverName,			/*I*/
								AliasHandle *alias) 			/*O*/
= {0x7009,0xA823};


/* given an alias handle and fromFile, resolve the alias, update the
   alias record and return aliased filename and wasChanged flag. */

pascal OSErr ResolveAlias(const FSSpec *fromFile,			/*I*/
								AliasHandle alias,			/*O*/
								FSSpec *target,				/*O*/
								Boolean *wasChanged)		/*O*/
= {0x7003,0xA823};
					
								
/* given an alias handle and an index specifying requested alias information
	type, return the information from alias record as a string. */
	
pascal OSErr GetAliasInfo(const AliasHandle alias,			/*I*/
								AliasInfoType index,		/*I*/
								Str63 theString)			/*O*/
= {0x7007,0xA823};
								

/* given a file spec, return target file spec if input file spec is an alias.
	It resolves the entire alias chain or one step of the chain.  It returns
	info about whether the target is a folder or file; and whether the input
	file spec was an alias or not.  */
	
pascal OSErr ResolveAliasFile(FSSpec	*theSpec,			/*I/O*/	
								Boolean	resolveAliasChains, /*I*/
								Boolean	*targetIsFolder,	/*O*/
								Boolean	*wasAliased)		/*O*/
= {0x700C,0xA823};


/****						Low Level Routines			****/

/* given an alias handle and fromFile, match the alias and return
	aliased filename(s) and needsUpdate flag */
	
pascal OSErr MatchAlias(const FSSpec *fromFile,	/*I*/
								unsigned long rulesMask,		/*I*/
								const AliasHandle alias,		/*I*/
								short *aliasCount,				/*I/O*/
								FSSpecPtr aliasList, 			/*O*/
								Boolean *needsUpdate,			/*O*/
								AliasFilterProcPtr aliasFilter, /*I*/
								Ptr yourDataPtr)				/*I*/
= {0x7005,0xA823};
					
								
/* given a fromFile-target pair and an alias handle, update the
	alias record pointed to by alias handle to represent target
	as the new alias. */
	
pascal OSErr UpdateAlias (const FSSpec *fromFile,			/*I*/
								const FSSpec *target, 		/*I*/
								AliasHandle alias,			/*I/O*/
								Boolean *wasChanged)		/*O*/
= {0x7006,0xA823};


#ifdef __cplusplus
}
#endif

#endif
