/************************************************************

Created: Wednesday, November 1, 1989
	AppleEvents.h
	C Interface to the Macintosh Libraries


	Copyright Apple Computer, Inc. 1985-1990
	All rights reserved

************************************************************/


#ifndef __APPLEEVENTS__
#define __APPLEEVENTS__

#ifndef __TYPES__
	#include <Types.h>
#endif
#ifndef __APPLETALK__
	#include <AppleTalk.h>
#endif
#ifndef __EVENTS__
	#include <Events.h>
#endif
#ifndef __PPCTOOLBOX__
	#include <PPCToolBox.h>
#endif
#ifndef __PROCESSES__
	#include <Processes.h>
#endif
#ifndef __EPPC__
	#include <EPPC.h>
#endif


#define	kAELongType				'long'
#define	kAEListType				'list'
#define	kAEWildType 			'****'
#define	kAEBooleanType			'bool'
#define	kAETrueType 			'true'
#define	kAEFalseType			'fals'
#define	kAEAliasType			'alis'
#define	kAEDirectObjectKeyword	'----'

#define	kCoreEventClass			'aevt'

#define	kAEOpenApplication		'oapp'
#define	kAEOpenDocuments		'odoc'
#define	kAEPrintDocuments		'pdoc'
#define	kAEQuitApplication		'quit'

#define	kAENoReply			0x00000001		/* Sender doesn't want a reply to msg */
#define	kAEQueueReply		0x00000002		/*Sender wants a reply but won't wait */
#define	kAEWaitReply		0x00000003		/* Sender wants to wait for a reply */

#define	kAENeverInteract	0x00000010		/* Server should not interact with user */
#define	kAECanInteract		0x00000020		/* Server may try to interact with user */
#define	kAEAlwaysInteract	0x00000030		/* Server should always interact with user */

#define	kAECanSwitchLayer	0x00000040		/*  Interaction may switch layer */

#define	kAEDontReconnect	0x00000080		/* don't reconnect if there is a sessClosedErr */


#define	kAEWantReceipt		nReturnReceipt	/* Sender wants a receipt of message */

#define	kAENormalPriority	0x00000000		/* Post message at the end of event queue */
#define	kAEHighPriority 	nAttnMsg		/* Post message at the front of the event
														 queue */

#define	kAutoGenerateRefcon  0

#define	kAEEventNotInTable		0			/* Passed to the general dispatch proc if 
														 the event received was not in the
														 dispatch table */

/* Error messages in response to reading and writing buffer contents */
#define	kAECoercionFail		-1700
#define	kAEDescNotFound		-1701
#define	kAECorruptData		-1702
#define	kAEWrongDataType	-1703
#define	kAENotAEList		-1704
#define	kAEBadListElement	-1705
#define	kAEWrongVersion		-1706
#define	kAENotAppleEvent	-1707

/* Error messages in response to sending/receiving a message */
#define	kAEEventNotFound		-1708				/* The event isn't in the supplied event
														 table */
#define	kAEReplyNotValid		-1709				/* AEResetTimer was passed an invalid reply */
#define	kAEUnknownSendMode		-1710				/* Mode wasn't NoReply, WaitReply,
														 QueueReply */
#define	KAEWaitCanceled			-1711				/* User cancelled out of wait loop for reply
														 or receipt */
#define	kAETimeoutErr			-1712				/* AppleEvent time out */

#define	kAENoUserInteraction	-1713				/* no user interaction allowed */

/******************************************* types *******************************************/

typedef	long			AEList ;
typedef	unsigned long	KeyWord ;
typedef	unsigned long	EventClass ;
typedef	unsigned long	EventID ;
typedef OSType			DescType ;

struct AEAddress {
	long addressKind ;
	union {
		ProcessSerialNumber receiverIDasPSN ;
		OSType receiverIDasSig ;
		long receiverIDasSess ;
		TargetID receiverIDasTargetID ;
		} u ;
	} ;
typedef struct AEAddress AEAddress ;

typedef AEAddress *AEAddressPtr, **AEAddressHndl ;

typedef enum AEArrayType { kAEDataArray, kAEPackedArray, kAEHandleArray, kAEDescArray,
								   kAEKeyDescArray } AEArrayType ;

struct AEDesc {
	DescType descriptorType;
	union {
		Handle asDataHdl ;
		AEList asAEList ;
		} u ;
	} ;
typedef struct AEDesc AEDesc ;

struct AEKeyDesc {
	KeyWord descKey ;
	AEDesc descContent ;
	} ;
typedef struct AEKeyDesc AEKeyDesc ;

#ifdef __cplusplus
extern "C" {
#endif

/********************************** function prototypes *************************************/

pascal OSErr AEinit( ProcPtr dispatchProc,
					ProcPtr coercionProc,
					ProcPtr nonAevtHandler ) ;

pascal void AEQuit( void ) ;

pascal OSErr AEPutPtr(	AEList theList,
						long index,
						DescType typeCode,
						Ptr dataPtr,
						long dataSize ) ;

pascal OSErr AEPutDesc(	AEList theList,
						long index,
						AEDesc theDesc ) ;

pascal OSErr AEPutKeyPtr(	AEList theList,
							KeyWord key,
							DescType typeCode,
							Ptr dataPtr,
							long dataSize ) ;

pascal OSErr AEPutKeyDesc(	AEList theList,
							KeyWord key,
							AEDesc theDesc ) ;

pascal OSErr AEGetNthPtr(	AEList theList,
							long index,
							DescType desiredType,
							KeyWord *key,
							DescType *typeCode,
							Ptr dataPtr,
							long maximSize,
							long *actualSize ) ;

pascal OSErr AEGetNthDesc(	AEList theList,
							long index,
							DescType desiredType,
							KeyWord *key,
							AEDesc *theDesc ) ;

pascal OSErr AEGetKeyPtr(	AEList theList,
							KeyWord key,
							DescType desiredType,
							DescType *typeCode,
							Ptr dataPtr,
							long maximSize,
							long *actualSize ) ;

pascal OSErr AEGetKeyDesc(	AEList theList,
							KeyWord key,
							DescType desiredType,
							AEDesc *theDesc ) ; 

pascal OSErr AEGetArray(	AEList theList,
							AEArrayType arrayType,
							Ptr arrayPtr,
							long bufSize,
							DescType *elemType,
							long *elemSize,
							long *itemCount ) ;

pascal OSErr AEPutArray(	AEList theList,
							AEArrayType arrayType,
							Ptr arrayPtr,
							DescType elemType,
							long elemSize,
							long itemCount ) ;

pascal OSErr AEDeleteIndex(	AEList theList,
							long index ) ;

pascal OSErr AEDeleteKey(	AEList theList,
							KeyWord key ) ;

pascal OSErr AECoerceDesc(	AEDesc desc,
								DescType toType,
								AEDesc *result ) ;

pascal void AEDisposeAEList( AEList theArg ) ;

pascal void AEDisposeAEDesc(	AEDesc theArg ) ;

pascal OSErr AECreateAevt(	EventClass theEventClass,
								EventID theEventID,
								AEAddress *target,
								long theRefCon,
								AEList *result ) ;

pascal void AEGetEventClassAndID(	AEList theAevt,
								EventClass *theEventClass,
								EventID *theEventID ) ;
		/*  Extract the message class and id from the specified message */

pascal void AEGetRefCon(	AEList theAevt,
							long *refcon ) ;
		/*  Extract the message class and id from the specified message */

pascal void AEGetAddress(	AEList theAevt,
								AEAddress *address ) ;
		/*  Extract the message class and id from the specified message */

pascal OSErr AECreateList(	Ptr factoringPtr,
							long factoredSize,
							Boolean isRecord,
							AEDesc *resultList ) ;

pascal long AECountListElems( AEList theList ) ;

pascal OSErr AESend(	AEList theAevt,
						AEList *theReply,
						long sendMode,
						short sendPriority,
						long timeOut,
						ProcPtr idleProc ) ;

pascal OSErr AEResetTimer( AEList reply ) ;
		/*	Convenience routine.  Create and send a 'wait' message from the
			information in the reply. */

pascal void AESetAddressByPSN(	ProcessSerialNumber thePSN,
								AEAddress *theAddress ) ;
		/*  Fill theAddressObject as receiverIDisPSN */

pascal void AESetAddressBySignature(	OSType theSig,
										AEAddress *theAddress ) ;
		/*  Fill theAddressObject as receiverIDisSignature */

pascal void AESetAddressBySessionID(	long theSessionID,
										AEAddress *theAddress );
		/*  Fill theAddressObject as receiverIDisSessionID */

pascal void AESetAddressByTargetID(	TargetID theTargetID,
									AEAddress *theAddress );
		/*  Fill theAddressObject as receiverIDisTargetID */

pascal OSErr AEProcessAppleEvent(	EventRecord eventRec ) ;

pascal OSErr AEGetDispatchTableRsrc( short tableID ) ;
		/* Load an event table from a 'AEDF' resource.  This of course will only be
		 of use to applications using a general dispatch routine rather than doing
		 a direct dispatch to a handler. */

pascal OSErr AECreateDispatchTable(	short numEntries ) ;
		/*	Create an event table with one memory manager call for the number of events
			specified.  If more events are added to the table with AESetupEventTable,
			the table will be grown automatically. */

pascal OSErr AEAddEvtToTable(	EventClass theEventClass,
								EventID theEventID,
								long value ) ;
		/*  Add a single event to the table, expanding if necessary. */

pascal OSErr AEGetTableEntry(	EventClass theEventClass,
								EventID theEventID,
								long *theValue ) ;
		/*  returns the value of the specified event in theValue.
		  	kAEEventNotFound if not found. */


pascal short AEGetInteractLevel(AEList theAevt ) ;
		/* returns kAENeverInteract, kAECanInteract or kAEAlwaysInteract	*/

#ifdef __cplusplus
}
#endif

#endif