/*
	File:		OutlineCalls.h

	Copyright:	 1989-1990 by Apple Computer, Inc., all rights reserved.

*/

#ifndef __OUTLINECALLS__
#define __OUTLINECALLS__

#ifndef __TYPES__
#include <Types.h>
#endif

/*
            ERRORS
*/
#define BITFONT_ERR                 0x8099              /* Outline metrics was called with a bitmap font */

/*
            Selectors
*/
#define IS_OUTLINE                  0x0000              /* Selector for the IsOutline routine */
#define SET_OUTLINEPREFERRED        0x0001              /* Selector for the SetOutlinePreferred routine */

#define OUTLINE_METRICS             0x0008              /* Selector for the OutlineMetrics routine */
#define GET_OUTLINEPREFERRED        0x0009              /* Selector for the GetOutlinePreferred routine */
#define SET_PRESERVE_GLYPHS         0x000A              /* Selector for the SetPreserveGlyph routine */
#define GET_PRESERVE_GLYPHS         0x000B              /* Selector for the GetPreserveGlyph routine */
#define FLUSH_FONTS                 0x000C              /* Selector for the FlushFonts routine */

#define GET_PS_FONT_NAME			0x000F

/****************** <4> ****************/

#define COUNT_FONT_FAMILIES			0x0010
#define GET_FONT_FAMILY_NAME		0x0011
#define NEW_FONT_FAMILY				0x0012
#define FOND_2_FONT_FAMILY			0x0013
#define DISPOSE_FONT_FAMILY			0x0014
#define GET_FONT_FAMILY				0x0015
#define COUNT_FOND_FACES			0x0016
#define GET_FONT_FACE_NAME			0x0017

#define NEW_FONT_FACE				0x0018
#define SFNT_2_FONT_FACE			0x0019
#define DISPOSE_FONT_FACE			0x001A
#define GET_FONT_FACE				0x001B
#define DETACH_FONT_FACE			0x001C

#define COUNT_FONT_TABLES			0x001D
#define GET_FONT_TABLE_INFO			0x001E
#define GET_FONT_TABLE				0x001F
#define GET_FONT_TABLE_PART			0x0020

#define COUNT_FONT_STRINGS			0x0021
#define GET_FONT_STRING_INFO		0x0022
#define GET_FONT_STRING				0x0023
#define COUNT_FONT_MAPPINGS			0x0024
#define GET_FONT_MAPPING_INFO		0x0025
#define APPLY_FONT_MAPPING			0x0026

#define COUNT_FONT_GLYPHS			0x0027
#define GET_GLYPH_PS_NAME			0x0028
#define GET_ALL_GLYPH_PS_NAMES		0x0029

/******************************************************************************
 *
 *	Font Toolbox.h
 *
 *	by Mike Reed
 *	Copyright Apple Computer, Inc. 1990  All rights reserved.
 *
 *****************************************************************************/

typedef enum {
	kNoFontError,
	kBadFont,
	kFontTableNotFound,
	kFontStringNotFound,
	kUnknownMappingFormat,
	kBadMappingIndex,
	kBadFontFaceIndex,
	kBadFontFaceName,
	kCannotLoadFamily,
	kCannotLoadFont,
	kNoStyleSizeMatch,
	kBadFamilyIndex,
	kBadTableFormat,
	kFontMemoryError
} fontError;

typedef struct { void* aFontFamily; } *fontFamily;
typedef struct { void* aFontFace; } *fontFace;

typedef long tableTag;

typedef struct {
	short platform;
	short script;
	short language;
	short meaning;
} fontStringInfo;

#ifdef __cplusplus
extern "C" {
#endif

/*
                          Boolean IsOutline

    IsOutline indicates whether or not the current grafport setting
    with the numer and denom loads a Spline Font.  FMSwapFont is called
    from this function.

    POSSIBLE ERRORS:  returns no errors
*/
extern pascal Boolean IsOutline(Point numer, Point denom)
    = {0x7000,0xA854};


/*
                          SetOutlinePreferred

    Sets a preference to load a matching spline font over an exact sized bitmap match.

    POSSIBLE ERRORS:  returns no errors
*/
extern pascal SetOutlinePreferred(Boolean outlinePreferred)
    = {0x7001,0xA854};

/*
                          Boolean GetOutlinePreferred

    Gets the state of OutlinePreferred.

    POSSIBLE ERRORS:  returns no errors
*/
extern pascal Boolean GetOutlinePreferred()
    = {0x7009,0xA854};

/*
                          OSErr OutlineMetrics

    Uses count, textPtr, numer and denom with the current grafport to
    load a spline font.  It returns yMax, yMin, advance widths, left side bearings and
    Rects.  A nil is passed for metrics not wanted.

    POSSIBLE ERRORS:  BITFONT_ERR      - If outline metrics loads a bitmap font.
*/
extern pascal OSErr OutlineMetrics( short count, Ptr textPtr, Point numer, Point denom, short *yMax,
                                    short *yMin, Fixed *awArray, Fixed *lsbArray, Rect *boundsArray)
	= {0x7008,0xA854};

/*
                          SetPreserveGlyph

    Set a mode to decide whether or not to Preserve the glyph (squishing or not).  With Set Preserve
    Glyph on all character glyphs are rendered at their full height.  With Set Preserve Glyph off
    characters above ascender or below descender will be scaled to fit within those bounds.

    POSSIBLE ERRORS:  returns no errors
*/
extern pascal SetPreserveGlyph(Boolean preserveGlyphs)
    = {0x700A,0xA854};

/*
                          GetPreserveGlyph

    Gets the state of Set Preserve Glyph.

    POSSIBLE ERRORS:  returns no errors
*/
extern pascal Boolean GetPreserveGlyph()
    = {0x700B,0xA854};

/*
                          FlushFonts

    FlushFonts flushed the font managers caches (i.e., width tables, sfnt caches)
	
    POSSIBLE ERRORS:  non-zero       - currently no errors are returned
*/
extern pascal OSErr FlushFonts()
    = {0x700C,0xA854};

/*
                          GetPSFontName

    This dude fills out the Postscript name for the current font
	
    POSSIBLE ERRORS:  non-zero
*/
extern pascal OSErr GetPSFontName(short txFont, short txFace, Str255 psName)
	= {0x700F, 0xA854};


/******* families ********/

extern pascal long CountFontFamilies(void) = {0x7010,0xA854};
extern pascal void GetFontFamilyName(long index, Str255 familyName) = {0x7011,0xA854};
extern pascal fontFamily NewFontFamily(long index, Str255 familyName) = {0x7012,0xA854};
extern pascal fontFamily FOND2FontFamily(Handle fond) = {0x7013,0xA854};
extern pascal void DisposeFontFamily(fontFamily) = {0x7014,0xA854};
extern pascal void GetFontFamily(fontFamily, Str255 familyName) = {0x7015,0xA854};
extern pascal long CountFondFaces(fontFamily) = {0x7016,0xA854};
extern pascal void GetFontFaceName(fontFamily, long index, Str255 fontName) = {0x7017,0xA854};

/******* fonts ********/

extern pascal fontFace NewFontFace(fontFamily, long index, Str255 fontName) = {0x7018,0xA854};
extern pascal fontFace sfnt2FontFace(Handle sfnt) = {0x7019,0xA854};
extern pascal void DisposeFontFace(fontFace) = {0x701A,0xA854};
extern pascal void GetFontFace(fontFace, Str255 fontName) = {0x701B,0xA854};
extern pascal void DetachFontFace(fontFace) = {0x701C,0xA854};

/******* tables ********/

extern pascal long CountFontTables(fontFace) = {0x701D,0xA854};
extern pascal long GetFontTableInfo(fontFace, long index, tableTag*, long* checkSum) = {0x701E,0xA854};
extern pascal long GetFontTable(fontFace, long index, Ptr userCopy) = {0x701F,0xA854};
extern pascal void GetFontTablePart(fontFace, long index, long offset, long length, Ptr userCopy) = {0x7020,0xA854};

/******* strings ********/

extern pascal long CountFontStrings(fontFace) = {0x7021,0xA854};
extern pascal long GetFontStringInfo(fontFace, long, fontStringInfo*) = {0x7022,0xA854};
extern pascal long GetFontString(fontFace, long, Ptr userCopy) = {0x7023,0xA854};

/******* mappings ********/

extern pascal long CountFontMappings(fontFace) = {0x7024,0xA854};
extern pascal long GetFontMappingInfo(fontFace, long index, short* platform, short* specific) = {0x7025,0xA854};
extern pascal long ApplyFontMapping(fontFace, long index, Ptr textStream, short* glyphs, long count) = {0x7026,0xA854};

/******* glyphs ********/

extern pascal long CountFontGlyphs(fontFace) = {0x7027,0xA854};
extern pascal long GetGlyphPSName(fontFace, long index, Str255 psName) = {0x7028,0xA854};
extern pascal void GetAllGlyphPSNames(fontFace fface, Handle names) = {0x7029,0xA854};

#ifdef __cplusplus
}
#endif

#endif