/************************************************************

Created: Thursday, October 19, 1989 at 11:10 PM
	PPCToolBox.h
	C Interface to the Macintosh Libraries


	Copyright Apple Computer, Inc. 1989-90
	All rights reserved

************************************************************/

#ifndef __PPCTOOLBOX__
#define __PPCTOOLBOX__

#ifndef __APPLETALK__
#include <AppleTalk.h>
#endif

/*  service Type  */

enum {RealTime = 1};
enum {StoreAndForward = 2};
enum {DontCare = 3};
enum {Both = 4};

/*  lookup Type  */

enum {NoLocName = 0};                   /* There is no PPCLocName */
enum {NbpName = 1};                     /* Use AppleTalk NBP      */
enum {NasName = 2};                     /* Use Name and Authentication Service */

/*  port Types  */

enum {MacType = 1};                     /* PortType is specified as standars mac creator and type */
enum {StrType = 2};                     /* Port Type is in pascal string format */

/*  Authentication Requests */

enum {Authenticate = 1};                /* Authenticate incomming network requets */
enum {DontAuthenticate = 0};            /* Don't Authenticate */

/*  Values returned for requestType field in PPCInform call */

enum {LocalRequest = 1};
enum {NetworkRequest = 2};

/*  ------------------------------------- 
 -900 to -950 are reserved for PPC     */

enum {notInitErr = -900};
enum {tooManyPortsErr = -901};
enum {nameTypeErr = -902};
enum {noPortErr = -903};
enum {noGlobalsErr = -904};
enum {localOnlyErr = -905};
enum {destPortErr = -906};
enum {sessTableErr = -907};
enum {noSessionErr = -908};
enum {badReqErr = -909};
enum {portNameExistsErr = -910};
enum {noPortTableErr = -911};
enum {userRejectErr = -912};
enum {noUserNameErr = -913};
enum {noToolboxNameErr = -914};
enum {noResponseErr = -915};
enum {portClosedErr = -916};
enum {sessClosedErr = -917};
enum {atalkPhaseErr = -918};            /* apple talk is not phase 2 */
enum {badPortNameErr = -919};
enum {loginCancelErr = -920};
enum {unKnownUserErr = -921};
enum {noDefaultUserErr = -922};
enum {notLoggedInErr = -923};
enum {noUserRefErr = -924};
enum {networkErr = -925};
enum {noInformErr = -926};
enum {authFailErr = -927};
enum {noUserRecErr = -928};
enum {notSupportedErr = -929};
enum {badServiceMethodErr = -930};
enum {badLocNameErr = -931};
enum {guestNotAllowedErr = -932};


/* Structure for  Port Name */
struct PortName
	{
	 unsigned short scriptCode;      /* Language Identifier */
	 Str32			nameStr;         /* pascal name string */
	 unsigned short typeSelector;    /* port's type selector */
	 union
		{
		 Str32		typeStr;         /* pascal type string */
		 struct
			{
			 unsigned long	portCreator;
			 unsigned long	portType;
			} macType;
		} u;
	};

typedef struct PortName PortName;
typedef PortName *PortNamePtr;

/* Structure for Location name */	
struct LocName
	{
	  unsigned short nameType;       /*  NBP or NAS name type (For Now Only NBPNAME) */
	  union
		  {
		    EntityName nbpName;          /* NBP name entity */
			unsigned char nasName[100];  /* For now NAS name entity (To be defined) */
		  } u;
	};

typedef struct LocName LocName;
typedef LocName *LocNamePtr;


#define PPCHeader \
	 Ptr			   qLink;			 /* PPC's Internal Use */\
	 unsigned short    cmdCode; 		   /* Requested PPC command */\
	 unsigned short    intxUse; 		  /* Internal Use */\
	 Ptr			   intxUsePtr;		  /* Internal Use */\
	 ProcPtr		   cmdCompletion;	 /* Completion Routine	*/\
	 OSErr			   cmdResult;		 /* Command Result Code */\
	 unsigned long	   Reserved[5]; 	 /* Reserved for PPC, Don'tuse */ 
	

/* PPCOpen Parameters */
struct PPCOpenParams
	{
	 PPCHeader
	 unsigned short portRefNum; 		/* Port Identifier*/
	 unsigned char  serviceType;       /* service type RealTime or StoreAndorward or Both */
	 unsigned char  resFlag;           /* Must be Set to Zero. Network Requests are always authentcated*/       
	 /* was authRequest. network requets to be Authenticated or not */
	 unsigned char  networkVisible;    /* make this network visible on network */
	 unsigned char  nbpRegistered;      /* The given locname was registered on the network */
	 PortNamePtr    portName;          /* PortName for PPC */
	 LocNamePtr     portLocName;       /* If NBP or NAS Registration is required */
	};

typedef struct PPCOpenParams PPCOpenParams;
typedef PPCOpenParams *PPCOpenParamsPtr;

/* PPCInform Parameters */
struct PPCInformParams
	{
	 PPCHeader
	 unsigned short    portRefNum;		   /* Port Identifier*/
	 unsigned long sessRefNum;       /* Session Reference */
	 Boolean	   autoAccept;       /* if true session will be accepted automatically */
	 unsigned char serviceMethod;    /* service type RealTime or StoreAndorward */ 
	 unsigned char requestType;      /* 1 = Local, 2 = Network */			 
	 unsigned long userData;     /* Any Optional data sent by user */
	 PortNamePtr   portName;         /* Destination portName */ 
	 unsigned char *userName;        /* user requesting the authentication */
	 LocNamePtr    portLocName;      /* NBP or NAS style service location name */ 
	};

typedef struct PPCInformParams PPCInformParams;
typedef PPCInformParams *PPCInformParamsPtr;

/* PPCStart Parameters */
/* Note: If service method is DONTCARE, Actual service used
   will be returned.
 */
struct PPCStartParams
	{
	 PPCHeader
	 unsigned short    portRefNum;		   /* Port Identifier*/
	 unsigned long sessRefNum;       /* Session Reference  */
	 unsigned char serviceMethod;    /* service type RealTime or StoreAndorward or Don't care */
	 unsigned short userRefNum;      /* userRefNum (obtained during login process) */ 
	 unsigned long userData;         /* Any optional data (tool box will not interpret this) */
	 unsigned long rejectInfo;       /* reason for rejecting the session request */
	 PortNamePtr   portName;         /* Destination portName */ 
	 LocNamePtr    portLocName;      /* NBP or NAS style service location name */ 
	};

typedef struct PPCStartParams PPCStartParams;

typedef PPCStartParams *PPCStartParamsPtr;

/* PPCAccept Parameters */
struct PPCAcceptParams
	{
	 PPCHeader
	 unsigned short resFiller;           /* Must be set to zero */
	 unsigned long sessRefNum;          /* Session Idenifier */ 
	};

/* PPCReject Parameters */
typedef struct PPCAcceptParams PPCAcceptParams;
typedef PPCAcceptParams *PPCAcceptParamsPtr;

struct PPCRejectParams
	{
	 PPCHeader
	 unsigned short resFiller;           /* Must be set to zero */
	 unsigned long sessRefNum;      /* Session Idenifier */ 
	 unsigned long rejectInfo;     /* reason for rejecting the session request */

	};
typedef struct PPCRejectParams PPCRejectParams;
typedef PPCRejectParams *PPCRejectParamsPtr;

/* PPCWrite Parameters */
struct PPCWriteParams
	{
	 PPCHeader
	 unsigned short resFiller;           /* Must be set to zero */
	 unsigned long sessRefNum;     /* session Identifier */ 
	 unsigned long blockCreator;   /* Message block creator */ 
	 unsigned long blockType;      /* Messgae block Type */ 
	 unsigned long bufferLength;   /* Length of the message buffer */ 
	 unsigned long actualLength;   /* Actual Length Written */ 
	 unsigned long userData;      /* User can pass anything he wishes */ 
	 unsigned char *bufferPtr;     /* Pointer to message buffer */ 
	 unsigned char more;           /* if true more data will be written */
	};

typedef struct PPCWriteParams PPCWriteParams;
typedef PPCWriteParams *PPCWriteParamsPtr;

/* PPCRead Parameters */	
struct PPCReadParams
	{
	 PPCHeader
	 unsigned short resFiller;           /* Must be set to zero */
	 unsigned long sessRefNum;         /* session Identifier */ 
	 unsigned long blockCreator;   /* Message block creator */ 
	 unsigned long blockType;      /* Messgae block Type */ 
	 unsigned long bufferLength;   /* Length of the message buffer */ 
	 unsigned long actualLength;   /* Actual Length Read */
	 unsigned long userData;      /* User can pass anything he wishes */
	 unsigned char *bufferPtr;     /* Pointer to message buffer */
	 Boolean       more;           /* Is there more data to be read? */
	};
	
typedef struct PPCReadParams PPCReadParams;
typedef PPCReadParams *PPCReadParamsPtr;

/* PPCEnd Parameters */
struct PPCEndParams
	{
	 PPCHeader
	 unsigned short resFiller;           /* Must be set to zero */
	 unsigned long sessRefNum;
	};

typedef struct PPCEndParams PPCEndParams;
typedef PPCEndParams *PPCEndParamsPtr;

struct PPCCloseParams
	{
	 PPCHeader
	 unsigned short    portRefNum;		   /* Port Identifier*/
	};

typedef struct PPCCloseParams PPCCloseParams;
typedef PPCCloseParams *PPCCloseParamsPtr;

/* PPCList Parameters */
typedef struct IPCListPortParams
	{
	 PPCHeader
	 unsigned short resFiller;           /* Must be set to zero */
	 unsigned short startIndex;     /* Start Index */ 
	 unsigned short requestCount;   /* Number of entries to be returned */
	 unsigned short actualCount;    /* Actual Number of entries to be returned */ 
	 PortNamePtr    portName;       /* Request Port name */ 
	 LocNamePtr     portLocName;    /* NBP or NAS type name to locate the Port Location */ 
	 unsigned char  *bufferPtr;     /* Buffer Pointer */ 
	};
	
typedef struct IPCListPortParams IPCListPortParams;
typedef IPCListPortParams *IPCListPortParamsPtr;

/*
 *	Info returned will be of type shown below
 *	for each of the entries as returned by the actualCount.
 *	and bufferPtr should have a buffer of length sizeof(PortInfo) * requestCount
 *
 */

struct PortInfo
{
	unsigned short authRequired;
	PortName portName;
};	

typedef struct PortInfo PortInfo;
typedef PortInfo *PortInfoPtr;

union PPCParamBlk
	{
		PPCOpenParams		  openPB;
		PPCInformParams 	  informPB;
		PPCStartParams		  startPB;
		PPCAcceptParams 	  acceptPB;
		PPCRejectParams 	  rejectPB;
		PPCWriteParams		  writePB;
		PPCReadParams		  readPB;
		PPCEndParams		  endPB;
		PPCCloseParams		  closePB;
		IPCListPortParams	  listPB;
	};

typedef union PPCParamBlk PPCParamBlk;

typedef  PPCParamBlk  *PPCParamBlkPtr;

/*	
typedef struct PPCParams   PPCParamBlk; 	
*/

#ifdef __cplusplus
extern "C" {
#endif

/* PPC Calling Conventions */

pascal OSErr PPCInit(void);

pascal OSErr PPCOpen(PPCOpenParamsPtr PB, Boolean async);

pascal OSErr PPCInform(PPCInformParamsPtr PB, Boolean async);

pascal OSErr PPCStart(PPCStartParamsPtr PB, Boolean async);

pascal OSErr PPCAccept(PPCAcceptParamsPtr PB, Boolean async);

pascal OSErr PPCReject(PPCRejectParamsPtr PB, Boolean async);

pascal OSErr PPCWrite(PPCWriteParamsPtr PB, Boolean async);

pascal OSErr PPCRead(PPCReadParamsPtr PB, Boolean async);

pascal OSErr PPCEnd(PPCEndParamsPtr PB, Boolean async);

pascal OSErr PPCClose(PPCCloseParamsPtr PB, Boolean async);

pascal OSErr IPCListPorts(IPCListPortParamsPtr PB, Boolean async);

pascal OSErr PromptForUserIdentity ( unsigned short *userRef,
									 StringPtr userName,
									 Boolean *guestSelected,
									 Boolean allowGuest,
									 StringPtr prompt);

pascal OSErr DeleteUserIdentity ( unsigned short userRef);

pascal OSErr GetDefaultUser ( unsigned short *userRef,
							  StringPtr userName);

pascal OSErr StartSecureSession ( PPCStartParams *startPb,
								  StringPtr userName,
								  Boolean useDefault,
								  Boolean allowGuest,
								  Boolean *guestSelected,
								  StringPtr prompt);

pascal OSErr PPCBrowser ( const Str255 prompt,
						  const Str255 applListLabel,
						  Boolean defaultSpecified,
						  LocName *theLoc,
						  PortInfo *thePortInfo,
						  ProcPtr portFilter)
						  = {0x303C, 0x0B00, 0xA82B};

/* competion Routine should be declared as
   void completionRoutine(PPCParamBlkPtr PB)
*/

#ifdef __cplusplus
}
#endif

#endif
