/************************************************************

Created: Friday, October 20, 1989 at 8:17 AM
	Packages.h
	C Interface to the Macintosh Libraries


	Copyright Apple Computer, Inc.	1985-1990
	All rights reserved

************************************************************/


#ifndef __PACKAGES__
#define __PACKAGES__

#ifndef __TYPES__
#include <Types.h>
#endif

#ifndef __STANDARDFILE__
#include <StandardFile.h>
#endif

#ifndef __SCRIPT__
#include <Script.h>
#endif

enum {listMgr = 0};             /*list manager*/
enum {dskInit = 2};             /*Disk Initializaton*/
enum {stdFile = 3};             /*Standard File*/
enum {flPoint = 4};             /*Floating-Point Arithmetic*/
enum {trFunc = 5};              /*Transcendental Functions*/
enum {intUtil = 6};             /*International Utilities*/
enum {bdConv = 7};              /*Binary/Decimal Conversion*/
enum {currSymLead = 16};
enum {currNegSym = 32};
enum {currTrailingZ = 64};
enum {currLeadingZ = 128};
enum {zeroCycle = 1};           /*0:00 AM/PM format*/
enum {longDay = 0};             /*day of the month*/
enum {longWeek = 1};            /*day of the week*/
enum {longMonth = 2};           /*month of the year*/
enum {longYear = 3};            /*year*/
enum {supDay = 1};              /*suppress day of month*/
enum {supWeek = 2};             /*suppress day of week*/
enum {supMonth = 4};            /*suppress month*/
enum {supYear = 8};             /*suppress year*/
enum {dayLdingZ = 32};
enum {mntLdingZ = 64};
enum {century = 128};
enum {secLeadingZ = 32};
enum {minLeadingZ = 64};
enum {hrLeadingZ = 128};

enum {verUS = 0};
enum {verFrance = 1};
enum {verBritain = 2};
enum {verGermany = 3};
enum {verItaly = 4};
enum {verNetherlands = 5};
enum {verFrBelgiumLux = 6};			/* French for Belgium & Luxembourg */
enum {verBelgiumLux = 6};			/* (use verFrBelgiumLux instead, less ambiguous) */	
enum {verSweden = 7};
enum {verSpain = 8};
enum {verDenmark = 9};
enum {verPortugal = 10};
enum {verFrCanada = 11};
enum {verNorway = 12};
enum {verIsrael = 13};
enum {verJapan = 14};
enum {verAustralia = 15};
enum {verArabia = 16};
enum {verArabic = 16};			/* synonym for verArabia */			/* <6> */
enum {verFinland = 17};
enum {verFrSwiss = 18};
enum {verGrSwiss = 19};
enum {verGreece = 20};
enum {verIceland = 21};
enum {verMalta = 22};
enum {verCyprus = 23};
enum {verTurkey = 24};
enum {verYugoCroatian = 25};			/* Croatian system for Yugoslavia */
enum {verYugoslavia = 25};				/* (use verYugoCroatian instead, less ambiguous) */
enum {verIndiaHindi = 33};				/* Hindi system for India */
enum {verIndia = 33};					/* (use verIndiaHindi instead, less ambiguous) */
enum {verPakistan = 34};
enum {verLithuania = 41};
enum {verPoland = 42};
enum {verHungary = 43};
enum {verEstonia = 44};
enum {verLatvia = 45};
enum {verLapland = 46};
enum {verFaeroeIsl = 47};
enum {verIran = 48};
enum {verRussia = 49};
enum {verIreland = 50};
enum {verKorea = 51};
enum {verChina = 52};
enum {verTaiwan = 53};
enum {verThailand = 54};
#define minCountry verUS
#define maxCountry verThailand

/* special ScriptCode values */

#define iuSystemScript (-1)     /* system script */
#define iuCurrentScript (-2)    /* current script */

/* special LangCode values */

#define iuSystemCurLang (-2)    /* current (itlbLang) lang for system script */
#define iuSystemDefLang (-3)    /* default (table) lang for system script */
#define iuCurrentCurLang (-4)   /* current (itlbLang) lang for current script */
#define iuCurrentDefLang (-5)   /* default lang for current script */
#define iuScriptCurLang (-6)    /* current (itlbLang) lang for specified script */
#define iuScriptDefLang (-7)    /* default language for a specified script */

/* table selectors for IUGetItlTable */

enum {iuWordSelectTable = 0};
enum {iuWordWrapTable = 1};
enum {iuNumberPartsTable = 2};

enum {shortDate,longDate,abbrevDate};
typedef unsigned char DateForm;

enum {mdy,dmy,ymd,myd,dym,ydm};
typedef unsigned char DateOrders;


struct Intl0Rec {
	char decimalPt; 			/*decimal point character*/
	char thousSep;				/*thousands separator*/
	char listSep;				/*list separator*/
	char currSym1;				/*currency symbol*/
	char currSym2;
	char currSym3;
	unsigned char currFmt;		/*currency format*/
	unsigned char dateOrder;	/*order of short date elements*/
	unsigned char shrtDateFmt;	/*short date format*/
	char dateSep;				/*date separator*/
	unsigned char timeCycle;	/*0 if 24-hour cycle, 255 if 12-hour*/
	unsigned char timeFmt;		/*time format*/
	char mornStr[4];			/*trailing string for first 12-hour cycle*/
	char eveStr[4]; 			/*trailing string for last 12-hour cycle*/
	char timeSep;				/*time separator*/
	char time1Suff; 			/*trailing string for 24-hour cycle*/
	char time2Suff;
	char time3Suff;
	char time4Suff;
	char time5Suff;
	char time6Suff;
	char time7Suff;
	char time8Suff;
	unsigned char metricSys;	/*255 if metric, 0 if not*/
	short intl0Vers;			/*version information*/
};

typedef struct Intl0Rec Intl0Rec;
typedef Intl0Rec *Intl0Ptr, **Intl0Hndl;

struct Intl1Rec {
	Str15 days[7];				/*day names*/
	Str15 months[12];			/*month names*/
	unsigned char suppressDay;	/*0 for day name, 255 for none*/
	unsigned char lngDateFmt;	/*order of long date elements*/
	unsigned char dayLeading0;	/*255 for leading 0 in day number*/
	unsigned char abbrLen;		/*length for abbreviating names*/
	char st0[4];				/*strings for long date format*/
	char st1[4];
	char st2[4];
	char st3[4];
	char st4[4];
	short intl1Vers;			/*version information*/
	short localRtn[1];			/*routine for localizing string comparison*/
};

typedef struct Intl1Rec Intl1Rec;
typedef Intl1Rec *Intl1Ptr, **Intl1Hndl;



#ifdef __cplusplus
extern "C" {
#endif
pascal void InitPack(short packID)
	= 0xA9E5;
pascal void InitAllPacks(void)
	= 0xA9E6;

pascal Handle IUGetIntl(short theID) 
    = {0x3F3C,0x0006,0xA9ED}; 															/* <3> */
pascal void IUSetIntl(short refNum,short theID,Handle intlParam) 
    = {0x3F3C,0x0008,0xA9ED}; 															/* <3> */
pascal void IUDateString(long dateTime,DateForm longFlag,Str255 result) 
    = {0x3F3C,0x0000,0xA9ED}; 															/* <3> */
pascal void IUDatePString(long dateTime,DateForm longFlag,Str255 result,
    Handle intlParam) 
    = {0x3F3C,0x000E,0xA9ED}; 															/* <3> */
pascal void IUTimeString(long dateTime,Boolean wantSeconds,Str255 result) 
    = {0x3F3C,0x0002,0xA9ED}; 															/* <3> */
pascal void IUTimePString(long dateTime,Boolean wantSeconds,Str255 result,
    Handle intlParam) 
    = {0x3F3C,0x0010,0xA9ED}; 															/* <3> */
pascal Boolean IUMetric(void) 
    = {0x3F3C,0x0004,0xA9ED}; 															/* <3> */
void iudatestring(long dateTime,DateForm longFlag,char *result); 
void iudatepstring(long dateTime,DateForm longFlag,char *result,Handle intlParam); 
void iutimestring(long dateTime,Boolean wantSeconds,char *result); 
void iutimepstring(long dateTime,Boolean wantSeconds,char *result,Handle intlParam); 

pascal short IUMagString(Ptr aPtr,Ptr bPtr,short aLen,short bLen) 
    = {0x3F3C,0x000A,0xA9ED}; 															/* <3> */
pascal short IUMagIDString(Ptr aPtr,Ptr bPtr,short aLen,short bLen) 
    = {0x3F3C,0x000C,0xA9ED}; 															/* <3> */
pascal short IUCompString(const Str255 aStr,const Str255 bStr); 
pascal short IUEqualString(const Str255 aStr,const Str255 bStr); 
short iucompstring(char *aStr,char *bStr); 
short iuequalstring(char *aStr,char *bStr); 

pascal void StringToNum(const Str255 theString,long *theNum); 
void 		stringtonum(char *theString,long *theNum); 
pascal void NumToString(long theNum,Str255 theString); 
void 		numtostring(long theNum,char *theString); 

pascal void IULDateString(LongDateTime *dateTime,DateForm longFlag,Str255 result,		/* <3> */
    Handle intlParam)
    = {0x3F3C,0x0014,0xA9ED}; 
pascal void IULTimeString(LongDateTime *dateTime,Boolean wantSeconds,Str255 result,		/* <3> */
    Handle intlParam)
    = {0x3F3C,0x0016,0xA9ED}; 

void iuldatestring(LongDateTime *dateTime,DateForm longFlag,char *result,				/* <8> */
    Handle intlParam);
void iultimestring(LongDateTime *dateTime,Boolean wantSeconds,char *result,				/* <8> */
    Handle intlParam);
pascal void IUClearCache() 																/* <1.2> */
    = {0x3F3C,0x0018,0xA9ED}; 
pascal short IUMagPString(Ptr aPtr,Ptr bPtr,short aLen,short bLen,Handle intlParam) 	/* <1.2> */
    = {0x3F3C,0x001A,0xA9ED}; 
pascal short IUMagIDPString(Ptr aPtr,Ptr bPtr,short aLen,short bLen,Handle intlParam) 	/* <1.2> */
    = {0x3F3C,0x001C,0xA9ED}; 
pascal short IUCompPString(const Str255 aStr,const Str255 bStr,Handle intlParam);		/* <1.2><3> */
pascal short IUEqualPString(const Str255 aStr,const Str255 bStr,Handle intlParam); 		/* <1.2><3> */
pascal short IUScriptOrder(ScriptCode script1,ScriptCode script2)						/* <1.2><1.3> */
    = {0x3F3C,0x001E,0xA9ED}; 
pascal short IULangOrder(LangCode language1,LangCode language2)							/* <1.2><1.3> */
    = {0x3F3C,0x0020,0xA9ED}; 
pascal short IUTextOrder(Ptr aPtr,Ptr bPtr,short aLen,short bLen,ScriptCode aScript,	/* <1.2><1.3> */
	ScriptCode bScript,LangCode aLang,LangCode bLang)
    = {0x3F3C,0x0022,0xA9ED}; 
pascal short IUStringOrder(const Str255 aStr,const Str255 bStr,ScriptCode aScript,		/* <1.2><1.3><3> */
	ScriptCode bScript,LangCode aLang,LangCode bLang);
pascal void IUGetItlTable(ScriptCode script,short tableCode,Handle *itlHandle,
	long *offset, long *length)															/* <1.4> */
	= {0x3F3C,0x0024,0xA9ED}; 
short iucomppstring(char *aStr,char *bStr,Handle intlParam);							/* <8> */
short iuequalpstring(char *aStr,char *bStr,Handle intlParam); 							/* <8> */
short iustringorder(char *aStr,char *bStr,ScriptCode aScript,ScriptCode bScript,		/* <8> */
	LangCode aLang,LangCode bLang);


#ifdef __cplusplus
}
#endif

#endif

