/************************************************************

Created: Friday, October 20, 1989 at 9:08 AM
	Power.h
	C Interface to the Macintosh Libraries


	Copyright Apple Computer, Inc.	1989-1990
	All rights reserved

************************************************************/


#ifndef __POWER__
#define __POWER__

#ifndef __TYPES__
#include <Types.h>
#endif


/* PMgrResultCode values */

enum {pmBusyErr = -13000};          /* PMgr stuck busy */
enum {pmReplyTOErr = -13001};       /* Timed out waiting to begin reply handshake */
enum {pmSendStartErr = -13002};     /* PMgr did not start handshake */
enum {pmSendEndErr = -13003};       /* During send, PMgr did not finish handshake */
enum {pmRecvStartErr = -13004};     /* During receive, PMgr did not start handshake */
enum {pmRecvEndErr = -13005};       /* During receive, PMgr did not finish handshake */

/* bit positions for PowerClockByte */

enum {pmSCCBit = 1};                /*SCC clock*/
enum {pmModemBit = 3};              /*Internal modem power*/
enum {pmSerialBit = 4};             /*Serial drivers power*/
enum {pmVoltBit = 6};               /*-5 volt power*/
enum {pmDevEnableBit = 7};

/* Masks for PowerClockByte */

enum {pmSCCMask = 0x2};
enum {pmModemMask = 0x8};
enum {pmSerialMask = 0x10};
enum {pmVoltMask = 0x40};
enum {pmDevEnableMask = 0x80};

/* Bit positions for ModemByte */

enum {modemOnBit = 0};
enum {ringWakeUpBit = 2};
enum {modemInstalledBit = 3};
enum {ringDetectBit = 4};
enum {modemOnHookBit = 5};

/* masks for ModemByte */

enum {modemOnMask = 0x1};
enum {ringWakeUpMask = 0x4};
enum {modemInstalledMask = 0x8};
enum {ringDetectMask = 0x10};
enum {modemOnHookMask = 0x20};

/* bit positions for BatteryByte */

enum {chargerConnBit = 0};
enum {hiChargeBit = 1};
enum {chargeOverFlowBit = 2};
enum {batteryDeadBit = 3};
enum {batteryLowBit = 4};
enum {connChangedBit = 5};

/* masks for BatteryByte */

enum {chargerConnMask = 0x1};
enum {hiChargeMask = 0x2};
enum {chargeOverFlowMask = 0x4};
enum {batteryDeadMask = 0x8};
enum {batteryLowMask = 0x10};
enum {connChangedMask = 0x20};

/* Commands for PMOp function */

enum {powerCntl = 0x10};            /* Power/clock control */
enum {powerRead = 0x18};            /* Power/clock control */
enum {modemRead = 0x58};            /* Internal modem setup */
enum {batteryRead = 0x68};          /* Battery/charger level and status */
enum {setWakeUp = 0x80};            /* Set Wake-up timer */
enum {disableWakeUp = 0x82};        /* Disable Wake-up timer */
enum {readWakeUp = 0x88};           /* Read Wake-up timer */

/* SleepQRec.sleepQFlags */

enum {noCalls = 1};
enum {noRequest = 2};


typedef char PowerClockByte;
typedef char ModemByte;
typedef char BatteryByte;


typedef long PMResultCode;

struct PMParamBlock {
	short pmCommand;
	short pmCount;
	Ptr pmSendBuff;
	Ptr pmReceiveBuff;
};

typedef struct PMParamBlock PMParamBlock;
typedef PMParamBlock *PMParamBlockPtr;

struct SleepQRec {
	struct SleepQRec *sleepQLink;
	short sleepQType;			/*type = 16*/
	ProcPtr sleepQProc; 		/*Pointer to sleep routine*/
	short sleepQFlags;
};

typedef struct SleepQRec SleepQRec;
typedef SleepQRec *SleepQRecPtr;

#ifdef __cplusplus
extern "C" {
#endif
PMResultCode PMOp(PMParamBlockPtr pBlockPtr)
	= {0x205F,0xA085};
long IdleUpdate(void)
	= 0xA285;
long GetCPUSpeed(void)
	= {0x70FF,0xA485};
void EnableIdle(void)
	= {0x7000,0xA485};
void DisableIdle(void)
	= {0x7001,0xA485};
void SleepQInstall(SleepQRecPtr qRecPtr)
	= {0x205F,0xA28A};
void SleepQRemove(SleepQRecPtr qRecPtr)
	= {0x205F,0xA48A};
void AOn(void)
	= {0x7004,0xA685};
void AOnIgnoreModem(void)
	= {0x7005,0xA685};
void BOn(void)
	= {0x7000,0xA685};
void BOnIgnoreModem(void)
	= {0x7001,0xA685};
void AOff(void)
	= {0x7084,0xA685};
void BOff(void)
	= {0x7080,0xA685};
#ifdef __cplusplus
}
#endif

#endif
