/************************************************************

Created: Friday, October 13, 1989
	Processes.h
	C Interface to the Macintosh Libraries


	Copyright Apple Computer, Inc.	1989-90
	All rights reserved

************************************************************/


#ifndef __PROCESSES__
#define __PROCESSES__

#ifndef __TYPES__
#include <Types.h>
#endif

#ifndef __EVENTS__
#include <Events.h>
#endif

/*************************************************************************
 *					Error codes not yet in libararies
 *************************************************************************/
enum {procNotFound = (-600)};       /* no eligible process with specified descriptor */
enum {memFragErr = (-601)};         /* not enough room to launch app w/special requirements */
enum {appModeErr = (-602)};         /* memory mode is 32-bit, but app not 32-bit clean */
enum {protocolErr = (-603)};        /* app made module calls in improper order */
enum {hardwareConfigErr = (-604)};  /* hardware configuration not correct for call */
enum {appMemFullErr = (-605)};      /* application SIZE not big enough for launch */
enum {appIsDaemon = (-606)};        /* app is BG-only, and launch flags disallow this */

/* ************************************************************************
 *                        AppleEvents contributions
 ************************************************************************

/* AppleEvents destined for kSystemProcess  */

#define aeCreatorType 'crea'
#define aeQuitAll 'quia'
#define aeShutDown 'shut'
#define aeRestart 'rest'

/*  AppleEvents for kSystemProcess  */

#define aeApplicationDied 'obit'

/*  Additional parameter keyword  */

#define aeProcessKeyword 'proc'

/*  Additional parameter types  */

#define aeProcessParamType 'proc'

/* ************************************************************************
 *                            Process identifier.
 ************************************************************************
 Various reserved process serial numbers. 
 */

#define kNoProcess (0)
#define kSystemProcess (1)
#define kCurrentProcess (2)

/*  typedef for unique process identifier 
************************************************************************
 *        Definition of the parameter block passed to _Launch.
 ************************************************************************
 Typedef and flags for launchControlFlags field  */

enum {launchContinue = 0x4000};
enum {launchNoFileFlags = 0x0800};
enum {launchUseMinimum = 0x0400};
enum {launchDontSwitch = 0x0200};
enum {launchAllow24Bit = 0x0100};
enum {launchInhibitDaemon = 0x0080};
enum {launchIsAutomatic = 0x0040};

/*  Set launchBlockID to extendedBlock to specify that extensions exist.
* Set launchEPBLength to extendedBlockLen for compatibility.
 */

#define extendedBlock ((unsigned short)'LC')
#define extendedBlockLen (sizeof(LaunchParamBlockRec) - 12)

/* ************************************************************************
 * Definition of the information block returned by GetProcessInformation.
 ************************************************************************
 Bits in the processMode field 
 */

enum {modeAutomatic = 0x00040000};
enum {modeDeskAccessory = 0x00020000};
enum {modeMultiLaunch = 0x00010000};
enum {modeNeedSuspendResume = 0x00004000};
enum {modeCanBackground = 0x00001000};
enum {modeDoesActivateOnFGSwitch = 0x00000800};
enum {modeOnlyBackground = 0x00000400};
enum {modeGetFrontClicks = 0x00000200};
enum {modeGetAppDiedMsg = 0x00000100};
enum {mode32BitCompatible = 0x00000080};
enum {modeHighLevelEventAware = 0x00000040};
enum {modeLocalAndRemoteHLEvents = 0x00000020};
enum {modeStationeryAware = 0x00000010};

/* typedef for unique process identifier */
struct ProcessSerialNumber {
	unsigned long		HighLongOfPSN;
	unsigned long		LowLongOfPSN;
};

typedef struct ProcessSerialNumber ProcessSerialNumber;
typedef ProcessSerialNumber *ProcessSerialNumberPtr;

/*************************************************************************
 *		Definition of the parameter block passed to _Launch.
 *************************************************************************/

/* Typedef and flags for launchControlFlags field */
typedef unsigned short LaunchFlags;

/* Format for first AppleEvent to pass to new process.  The size of the overall
 * buffer variable: the message body immediately follows the messageLength.
 */
struct AppParameters {
	EventRecord			theMsgEvent;
	unsigned long		eventRefCon;
	unsigned long		messageLength;
/*	char				messageBody[messageLength];		*/
};

typedef struct AppParameters AppParameters;
typedef AppParameters *AppParametersPtr;

/* Parameter block to _Launch */
struct LaunchParamBlockRec {
	StringPtr				name;
	unsigned short			reserved1;
	unsigned short			launchBlockID;
	unsigned long			launchEPBLength;
	unsigned short			launchFileFlags;
	LaunchFlags 			launchControlFlags;
	unsigned short			launchVRefNum;
	unsigned long			launchDirID;
	ProcessSerialNumber 	launchProcessSN;
	unsigned long			launchPreferredSize;
	unsigned long			launchMinimumSize;
	unsigned long			launchAvailableSize;
	AppParametersPtr		launchAppParameters;
};

typedef struct LaunchParamBlockRec LaunchParamBlockRec;
typedef LaunchParamBlockRec *LaunchPBPtr;


/* Record returned by GetProcessInformation */
struct ProcessInfoRec {
	unsigned long			processInfoLength;
	StringPtr				processName;
	ProcessSerialNumber 	processNumber;
	unsigned long			processType;
	unsigned long			processSignature;
	unsigned long			processMode;
	Ptr 					processLocation;
	unsigned long			processSize;
	unsigned long			processFreeMem;
	ProcessSerialNumber 	processLauncher;
	unsigned long			processLaunchDate;
	unsigned long			processActiveTime;
	StringPtr				processFileName;
	unsigned long			processFileDirID;
	unsigned short			processFileVRefNum;
};

typedef struct ProcessInfoRec ProcessInfoRec;
typedef ProcessInfoRec *ProcessInfoRecPtr;

#ifdef __cplusplus
extern "C" {
#endif

/*************************************************************************
 *							Routine prototypes.
 *************************************************************************/
OSErr LaunchApplication(LaunchPBPtr LaunchParams)
	= {0x205F, 0xA9F2};

pascal OSErr LaunchDeskAccessory(StringPtr pFileName, short fileVRefNum, long fileDirID, StringPtr pDAName)
	= {0x3F3C,0x0036,0xA88F};

pascal OSErr GetCurrentProcess(ProcessSerialNumber *psnStorage)
	= {0x3F3C,0x0037,0xA88F};

pascal OSErr GetFrontProcess(ProcessSerialNumber *psnStorage)
	= {0x70FF, 0x2F00, 0x3F3C, 0x0039, 0xA88F};

pascal OSErr GetNextProcess(ProcessSerialNumber *pParamPSN)
	= {0x3F3C,0x0038,0xA88F};

pascal OSErr GetProcessInformation(ProcessSerialNumberPtr pPSN, ProcessInfoRecPtr pStorage)
	= {0x3F3C,0x003A,0xA88F};

pascal OSErr SetFrontProcess(ProcessSerialNumberPtr pPSN)
	= {0x3F3C,0x003B,0xA88F};

pascal OSErr WakeUpProcess(ProcessSerialNumberPtr pPSN)
	= {0x3F3C,0x003C,0xA88F};

pascal OSErr SameProcess(ProcessSerialNumberPtr pPSN1, ProcessSerialNumberPtr pPSN2, Boolean *pResult)
	= {0x3F3C,0x003D,0xA88F};

#ifdef __cplusplus
}
#endif

#endif __PROCESSES__
