/************************************************************

Created: Friday, October 20, 1989 at 3:07 PM
	Sound.h
	C Interface to the Macintosh Libraries


	Copyright Apple Computer, Inc.	1986-1990
	All rights reserved

************************************************************/


#ifndef __SOUND__
#define __SOUND__

#ifndef __TYPES__
#include <Types.h>
#endif

enum {swMode = -1};
enum {ftMode = 1};
enum {ffMode = 0};
#define synthCodeRsrc 'snth'
#define soundListRsrc 'snd '

/* synthesizer numbers for SndNewChannel */

enum {noteSynth = 1};                   /*note synthesizer*/
enum {waveTableSynth = 3};              /*wave table synthesizer*/
enum {sampledSynth = 5};                /*sampled sound synthesizer*/

/* Param2 values */

#define twelthRootTwo 1.05946309434
enum {infiniteTime = 0x7FFFFFFF};

/* command numbers for SndDoCommand */

enum {nullCmd = 0};
enum {initCmd = 1};
enum {freeCmd = 2};
enum {quietCmd = 3};
enum {flushCmd = 4};
enum {reInitCmd = 5};

enum {waitCmd = 10};
enum {pauseCmd = 11};
enum {resumeCmd = 12};
enum {callBackCmd = 13};
enum {syncCmd = 14};
enum {emptyCmd = 15};

enum {tickleCmd = 20};
enum {requestNextCmd = 21};
enum {howOftenCmd = 22};
enum {wakeUpCmd = 23};
enum {availableCmd = 24};
enum {versionCmd = 25};
enum {totalLoadCmd = 26};
enum {loadCmd = 27};

enum {scaleCmd = 30};
enum {tempoCmd = 31};

enum {noteCmd = 40};
enum {restCmd = 41};
enum {freqCmd = 42};
enum {ampCmd = 43};
enum {timbreCmd = 44};

enum {waveTableCmd = 60};
enum {phaseCmd = 61};

enum {soundCmd = 80};
enum {bufferCmd = 81};
enum {rateCmd = 82};
enum {continueCmd = 83};
enum {doubleBufferCmd = 84};
enum {getRateCmd = 85};

enum {sizeCmd = 90};	
enum {convertCmd = 91};

enum {waveInitChannelMask = 0x07};
enum {waveInitChannel0 = 0x04};
enum {waveInitChannel1 = 0x05};
enum {waveInitChannel2 = 0x06};
enum {waveInitChannel3 = 0x07};

enum {initSRateMask = 0x0030};
enum {initStereoMask = 0x00C0};
enum {initChanLeft = 0x0002};           /*left stereo channel */
enum {initChanRight = 0x0003};          /*right stereo channel*/
enum {initSRate22k = 0x0020};           /*22k sampling rate*/
enum {initSRate44k = 0x0030};           /*44k sampling rate*/
enum {initMono = 0x0080};               /*monophonic channel*/
enum {initStereo = 0x00C0};             /*stereo channel*/
enum {initNoInterp = 0x0004};           /*no Linear Interpolation with the SR Convert*/
enum {initNoSRC = 0x0008};
enum {initNoMultiChannel = 0x1000};
enum {initMACE3 = 0x0300};
enum {initMACE6 = 0x0400};

enum {initChan0 = 4};                   /*channel 0 - wave table only*/
enum {initChan1 = 5};                   /*channel 1 - wave table only*/
enum {initChan2 = 6};                   /*channel 2 - wave table only*/
enum {initChan3 = 7};                   /*channel 3 - wave table only*/

enum {stdSH = 0x00};
enum {extSH = 0xFF};
enum {cmpSH = 0xFE};
enum {notCompressed = 0};
enum {twoToOne = 1};
enum {eightToThree = 2};
enum {threeToOne = 3};
enum {sixToOne = 4};
enum {outsideCmpSH = 0};
enum {insideCmpSH = 1};
enum {aceSuccess = 0};
enum {aceMemFull = 1};
enum {aceNilBlock = 2};
enum {aceBadComp = 3};
enum {aceBadEncode = 4};
enum {aceBadDest = 5};
enum {aceBadCmd = 6};
enum {sixToOnePacketSize = 8};
enum {threeToOnePacketSize = 16};
enum {stateBlockSize = 64};
enum {leftOverBlockSize = 32};

enum {firstSoundFormat = 0x0001};			/* general sound format */
enum {secondSoundFormat = 0x0002};			/* special sampled sound format */
enum {setPtrBit = 0x8000};

enum {stdQLength = 128};
enum {dataPointerFlag = 0x8000};

typedef long Time;							/* in half milliseconds */

typedef unsigned char Wave[256];
typedef Wave *WavePtr;

enum {sysBeepDisable = 0x0000};
enum {sysBeepEnable = 0x0001};
											/* unitTypes for AudioSelection.unitType */
enum {unitTypeNoSelection = 0xFFFF};
enum {unitTypeSeconds = 0x0000};

enum {sMgrBadParam = -213};					/* invalid argument */
enum {sMgrResNotFound = -214};

typedef unsigned char FreeWave[30001];

struct FFSynthRec {
	short mode;
	Fixed count;
	FreeWave waveBytes;
};

typedef struct FFSynthRec FFSynthRec;
typedef FFSynthRec *FFSynthPtr;

struct Tone {
	short count;
	short amplitude;
	short duration;
};

typedef struct Tone Tone;
typedef Tone Tones[5001];

struct SWSynthRec {
	short mode;
	Tones triplets;
};

typedef struct SWSynthRec SWSynthRec;
typedef SWSynthRec *SWSynthPtr;

struct FTSoundRec {
	short duration;
	Fixed sound1Rate;
	long sound1Phase;
	Fixed sound2Rate;
	long sound2Phase;
	Fixed sound3Rate;
	long sound3Phase;
	Fixed sound4Rate;
	long sound4Phase;
	WavePtr sound1Wave;
	WavePtr sound2Wave;
	WavePtr sound3Wave;
	WavePtr sound4Wave;
};

typedef struct FTSoundRec FTSoundRec;
typedef FTSoundRec *FTSndRecPtr;

struct FTSynthRec {
	short mode;
	FTSndRecPtr sndRec;
};

typedef struct FTSynthRec FTSynthRec;
typedef FTSynthRec *FTSynthPtr;

typedef struct SndCommand {
    unsigned short	cmd;
    short			param1;
    long			param2;
}SndCommand;

typedef struct SndChannel *SndChannelPtr;

typedef pascal void (*SndCompletionProcPtr)(void);
typedef pascal Boolean (*SndCallBackProcPtr)(SndChannelPtr chan, SndCommand cmd);
typedef ProcPtr	SndModifierProcPtr;

struct SoundHeader {
    char * samplePtr;                  /*if NIL then samples are in sampleArea*/
    unsigned long length;
    Fixed sampleRate;
    unsigned long loopStart;
    unsigned long loopEnd;
    unsigned char encode;
    unsigned char baseNote;
    char sampleArea[1];
};

typedef struct SoundHeader SoundHeader;
typedef SoundHeader *SoundHeaderPtr;

struct SndChannel {
	SndChannelPtr nextChan;
	Ptr firstMod;
	SndCallBackProcPtr callBack;
	long userInfo;
    long wait;                      /*The following is for internal Sound Manager use only.*/
	SndCommand cmdInProgress;
	short flags;
	short qLength;
    short qHead;                    /*next spot to read or -1 if empty*/
    short qTail;                    /*next spot to write = qHead if full*/
	SndCommand queue[128];
};

typedef struct SndChannel SndChannel;

/*______________________ <hbs 1/10/89> structs for ACE____________________________ */


typedef struct{
	short						stateVar[stateBlockSize] ;
}   StateBlock, *StateBlockPtr;    


typedef struct 
{
	unsigned long				count;
	char						sampleArea[leftOverBlockSize];
}
LeftOverBlock, *LeftOverBlockPtr;


typedef struct
 {
	char 				*samplePtr;			/* if nil then samples are in sample area						*/
	unsigned long 	    numChannels;		/* CAFF, number of channels  mono = 1 							*/
	Fixed				sampleRate;			/* sample rate in Apples Fixed point representation				*/
	unsigned long 		loopStart;			/* loopStart of sound before compression						*/
	unsigned long 		loopEnd;			/* loopEnd of sound before compression							*/
	unsigned char 		encode;				/* data structure used ,  stdSH, extSH, or cmpSH				*/
	unsigned char 		baseNote;			/* same meaning as regular SoundHeader							*/
	unsigned long 		numFrames;			/* length in frames	( packetFrames or sampleFrames				*/
	extended 			AIFFSampleRate;		/* CAFF, IEEE sample rate 										*/
	Ptr 				MarkerChunk;		/* CAFF, sync track												*/
	Ptr 				FutureUse1;			/* reserved by Apple											*/
	Ptr				 	FutureUse2;			/* reserved by Apple											*/
	StateBlockPtr 		StateVars;			/* pointer to State Block 										*/	
	LeftOverBlockPtr 	LeftOverSamples;	/* used to save truncated samples between compression calls   	*/
	unsigned short		compressionID;		/* 0 means no compression, non zero means compressionID			*/
	unsigned short 	    packetSize;			/* CAFF, number of bits in compressed sample packet				*/	
	unsigned short 	    snthID;				/* Resource ID of Sound Manager snth that contains NRT C/E		*/
	unsigned short 	    sampleSize;			/* CAFF, number of bits in non-compressed sample 				*/
	char 				sampleArea[1];		/* space for when samples follow directly						*/		
}	
CmpSoundHeader, *CmpSoundHeaderPtr;

typedef struct
 {
	char 				*samplePtr;			/* if nil then samples are in sample area						*/
	unsigned long 	    numChannels;		/* AIFF, number of channels,  ie mono = 1 						*/
	Fixed				sampleRate;			/* sample rate in Apples Fixed point representation				*/
	unsigned long 		loopStart;			/* same meaning as regular SoundHeader							*/	
	unsigned long 		loopEnd;			/* same meaning as regular SoundHeader							*/
	unsigned char 		encode;				/* data structure used ,  stdSH, extSH, or cmpSH				*/
	unsigned char 		baseNote;			/* same meaning as regular SoundHeader							*/
	unsigned long 		numSampleFrames;	/* length in total number of frames								*/
	extended 			AIFFSampleRate;		/* AIFF, IEEE sample rate 										*/
	Ptr 				MarkerChunk;		/* AIFF, sync track												*/
	Ptr 				InstrumentChunks;	/* AIFF, 														*/
	Ptr 				AESRecording;		/* AIFF, 														*/
	unsigned short 	    sampleSize;			/* AIFF, number of bits in sample 								*/
	unsigned short 		FutureUse1;			/* reserved by Apple											*/	
	unsigned long 		FutureUse2;			/* reserved by Apple											*/	
	unsigned long 		FutureUse3;			/* reserved by Apple											*/	
	unsigned long 		FutureUse4;			/* reserved by Apple											*/
	char 				sampleArea[1];		/* space for when samples follow directly						*/		
}	
ExtSoundHeader, *ExtSoundHeaderPtr;



typedef struct{
	short				smMaxCPULoad;
	short				smNumChannels;
	short				smCurCPULoad;
}
SMStatus, *SMStatusPtr;


typedef struct{ 
	Fixed				scStartTime;
	Fixed				scEndTime;
	Fixed				scCurrentTime;
	Boolean				scChannelBusy;
	Boolean				scChannelDisposed;
	Boolean				scChannelPaused;
	char				scUnused;
	unsigned long		scChannelAttributes;
	long				scCPULoad;
}
SCStatus,*SCStatusPtr;


typedef struct {
	long				dbNumFrames;
	long				dbFlags;
	long				dbUserInfo[2];
	char				dbSoundData[1];
} SndDoubleBuffer, *SndDoubleBufferPtr;


typedef struct{ 
	long				unitType;
	Fixed				selStart;
	Fixed				selEnd;
}AudioSelection, *AudioSelectionPtr;


typedef struct {
	short				dbhNumChannels;
	short				dbhSampleSize;
	short				dbhCompressionID;
	short				dbhPacketSize;
	Fixed				dbhSampleRate;
	SndDoubleBufferPtr	dbhBufferPtr[2];
	ProcPtr				dbhDoubleBack;
}
SndDoubleBufferHeader, *SndDoubleBufferHeaderPtr;

						
enum {dbBufferEmpty = 0x00000000};		/* Constants for SndPlayDoubleBuffer*/	
enum {dbBufferEmptyMask = 0xFFFFFFFE};
enum {dbBufferReady = 0x00000001};
enum {dbBufferExhausted = 0x00000002};
enum {dbLastBuffer = 0x00000004};

struct ModRef {
    unsigned short modNumber;
    long modInit;
};

typedef struct ModRef ModRef;

struct SndListResource {
	short format;
	short numModifiers;
	ModRef modifierPart[1]; 		/*This is a variable length array*/
	short numCommands;
	SndCommand commandPart[1];		/*This is a variable length array*/
	char dataPart[1];				/*This is a variable length array*/
};

typedef struct SndListResource SndListResource;
typedef SndListResource *SndListPtr;

#ifdef __cplusplus
extern "C" {
#endif
pascal OSErr SndDoCommand(SndChannelPtr chan,const SndCommand *cmd,Boolean noWait)
	= 0xA803;
pascal OSErr SndDoImmediate(SndChannelPtr chan,const SndCommand *cmd)
	= 0xA804;
pascal OSErr SndNewChannel(SndChannelPtr *chan,short synth,long init,SndCallBackProcPtr userRoutine)
	= 0xA807;
pascal OSErr SndDisposeChannel(SndChannelPtr chan,Boolean quietNow)
	= 0xA801;
pascal OSErr SndPlay(SndChannelPtr chan,Handle sndHdl,Boolean async)
	= 0xA805;
pascal OSErr SndControl(short id,SndCommand *cmd)
	= 0xA806;


/**/
/*
/*			New Enhanced Sound Manager 7.0 Calls	<18 Aug 1989 NJC>
/*
/*			These will soon be supported by a Glue Library so that all this ugly
/*			shit will disappear and the traps will look like non-indexed traps.
/*
/**/

#define	MOVEL						0x203C	

/* constants for MACE in the dispatcher */
#define MACEToolNum 16
#define sMgrMACEVersion 0
#define sMgrComp3to1 4
#define sMgrExp1to3 8
#define sMgrComp6to1 12
#define sMgrExp1to6 16

/* constants for Sound Manager Extensions in the dispatcher */

#define sMgrToolNum 8
#define sMgrSndSoundManagerVersion 0
#define sMgrSndStartFilePlay 4
#define sMgrSndPauseFilePlay 8
#define sMgrSndStopFilePlay 12
#define sMgrSndChannelStatus 16
#define sMgrSndManagerStatus 20
#define sMgrSndGetSysBeepState 24
#define sMgrSndSetSysBeepState 28
#define sMgrSndPlayDoubleBuffer 32
#define sMgrSndGetBufferStufferLoad 36
#define sMgrSndGetMixerLoad 40


pascal unsigned long 
SndSoundManagerVersion ()
	= {MOVEL,sMgrSndSoundManagerVersion,sMgrToolNum,0xA800};

pascal OSErr
SndStartFilePlay ( SndChannelPtr theChannel, short vRefNum, Str63 fName,
		short resNum, long totalSpace, Ptr theBuffer,
		AudioSelectionPtr theSelection, ProcPtr completionRtn, Boolean async)
	= {MOVEL,sMgrSndStartFilePlay,sMgrToolNum,0xA800};
			

pascal OSErr
SndPauseFilePlay (SndChannelPtr theChannel)
	= {MOVEL,sMgrSndPauseFilePlay,sMgrToolNum,0xA800};

pascal OSErr
SndStopFilePlay (SndChannelPtr theChannel, Boolean noWait)
	= {MOVEL,sMgrSndStopFilePlay,sMgrToolNum,0xA800};
	
pascal OSErr 
SndChannelStatus (SndChannelPtr theChannel, 
				short theLength, SCStatusPtr theStatus)
	= {MOVEL,sMgrSndChannelStatus,sMgrToolNum,0xA800};
	
pascal OSErr 
SndManagerStatus (short theLength, SMStatusPtr theStatus)
	= {MOVEL,sMgrSndManagerStatus,sMgrToolNum,0xA800};
	
pascal void 
SndGetSysBeepState ( short *sysBeepState)
	= {MOVEL,sMgrSndGetSysBeepState,sMgrToolNum,0xA800};

pascal OSErr 
SndSetSysBeepState (short sysBeepState)
	= {MOVEL,sMgrSndSetSysBeepState,sMgrToolNum,0xA800};

pascal OSErr
SndPlayDoubleBuffer ( SndChannelPtr theChannel, SndDoubleBufferHeaderPtr theParams)
	= {MOVEL,sMgrSndPlayDoubleBuffer,sMgrToolNum,0xA800};

pascal short 
SndGetBufferStufferLoad (short channelFeatures)
	= {MOVEL,sMgrSndGetBufferStufferLoad,sMgrToolNum,0xA800};

pascal short 
SndGetMixerLoad (short numChannels)
	= {MOVEL,sMgrSndGetMixerLoad,sMgrToolNum,0xA800};

pascal unsigned long
MACEVersion (void)
	= {MOVEL,sMgrMACEVersion,MACEToolNum,0xA800};

pascal void Comp3to1 ( Ptr inBuffer, Ptr outBuffer, unsigned long  Cnt, Ptr inState,
		Ptr outState, unsigned long  numChannels, unsigned long  whichChannel )
	= {MOVEL,sMgrComp3to1,MACEToolNum,0xA800};

pascal void Exp1to3 ( Ptr inBuffer, Ptr outBuffer, unsigned long  Cnt, Ptr inState, 
		Ptr outState, unsigned long  numChannels, unsigned long  whichChannel)
	= {MOVEL,sMgrExp1to3,MACEToolNum,0xA800};

pascal void Comp6to1 ( Ptr inBuffer, Ptr outBuffer, unsigned long  Cnt, Ptr inState, 
		Ptr outState, unsigned long  numChannels, unsigned long  whichChannel )
	= {MOVEL,sMgrComp6to1,MACEToolNum,0xA800};

pascal void Exp1to6 ( Ptr inBuffer, Ptr outBuffer, unsigned long  Cnt, Ptr inState,
		Ptr outState, unsigned long  numChannels, unsigned long  whichChannel)
	= {MOVEL,sMgrExp1to6,MACEToolNum,0xA800};


pascal void SetSoundVol(short level);
pascal void GetSoundVol(short *level);
pascal void StartSound(Ptr synthRec,long numBytes,SndCompletionProcPtr completionRtn);
pascal void StopSound(void);
pascal Boolean SoundDone(void); 
pascal OSErr SndAddModifier(SndChannelPtr chan,SndModifierProcPtr modifier,
    short id,long init)
    = 0xA802; 
#ifdef __cplusplus
}
#endif

#endif
