/*
	File:		SoundInput.h

	Copyright:	 1986-1990 by Apple Computer, Inc., all rights reserved.

*/

/* Error codes */
#ifndef __SOUNDINPUT__
#define __SOUNDINPUT__


#ifndef __TYPES__
#include <Types.h>
#endif

#define noSoundInHardware		-220
#define badSoundInDevice		-221
#define noBufferSpecified		-222
#define noCompletionRoutine		-223
#define invalidSampleRate		-225
#define invalidSampleSize		-226
#define deviceBusyErr			-227
#define badDeviceName			-228
#define badRefNum				-229
#define floppyNotAllowed		-230
#define unknownInfoType			-231
#define unknownQuality			-232


#define sInToolNum				20	/* tool number of sIn for SndDispVersion call */
#define	MOVEL					0x203C	

/* function selectors for the sInput calls */

#define sInVersion				0
#define sInSndRecord			4
#define sInSndRecordToFile		8
#define sInSignInDevice			12
#define sInSignOutDevice		16
#define sInGetIndexedDevice		20
#define sInOpenDevice			24
#define sInCloseDevice			28
#define sInRecord				32
#define sInRecordToFile			36
#define sInPauseRecording		40
#define sInResumeRecording		44
#define sInStopRecording		48
#define sInGetRecordingStatus	52
#define sInGetDeviceInfo		56
#define sInSetDeviceInfo		60
#define sInMilliSecondsToBytes	64
#define sInBytesToMilliSeconds	68



typedef struct {
		long			sInRefNum;
		long			count;
		long			milliseconds;
		long			bufferLength;
		Ptr		 		sInputBufferPtr;
		ProcPtr 		completionRoutine;
		ProcPtr 		interruptRoutine;
		long			userLong;
		OSErr			error;
} SPB, *SPBPtr;


#ifdef __cplusplus
extern "C" {
#endif

pascal unsigned long 
SPBVersion(void)
	= {MOVEL,sInVersion,sInToolNum,0xA800};

pascal OSErr
SndRecord(ProcPtr filterProc, Point corner, OSType	quality, Handle *sndHandle)
	= {MOVEL,sInSndRecord,sInToolNum,0xA800};

pascal OSErr
SndRecordToFile(ProcPtr filterProc, Point corner, OSType quality, 
				short vRefNum, Str255 fName)
	= {MOVEL,sInSndRecordToFile,sInToolNum,0xA800};

pascal OSErr 
SPBSignInDevice(short deviceRefNum, Str255 deviceName)
	= {MOVEL,sInSignInDevice,sInToolNum,0xA800};

pascal OSErr
SPBSignOutDevice(short deviceRefNum)
	= {MOVEL,sInSignOutDevice,sInToolNum,0xA800};

pascal OSErr 
SPBGetIndexedDevice(short count, Str255 deviceName, Ptr deviceIconPtr)
	= {MOVEL,sInGetIndexedDevice,sInToolNum,0xA800};

pascal OSErr 
SPBOpenDevice(Str255 deviceName, short	permission, long *sInRefNum)
	= {MOVEL,sInOpenDevice,sInToolNum,0xA800};

pascal OSErr 
SPBCloseDevice(long sInRefNum)
	= {MOVEL,sInCloseDevice,sInToolNum,0xA800};

pascal OSErr 
SPBRecord(SPBPtr sInParamPtr, Boolean asynchFlag)
	= {MOVEL,sInRecord,sInToolNum,0xA800};

pascal OSErr 
SPBRecordToFile(short refNum, SPBPtr sInParamPtr, Boolean asynchFlag)
	= {MOVEL,sInRecordToFile,sInToolNum,0xA800};

pascal OSErr 
SPBPauseRecording(long	sInRefNum)
	= {MOVEL,sInPauseRecording,sInToolNum,0xA800};

pascal OSErr 
SPBResumeRecording(long	sInRefNum)
	= {MOVEL,sInResumeRecording,sInToolNum,0xA800};

pascal OSErr 
SPBStopRecording(long	sInRefNum)
	= {MOVEL,sInStopRecording,sInToolNum,0xA800};

pascal OSErr 
SPBGetRecordingStatus(	long			sInRefNum,
						short			*recordingStatus,
						short			*meterLevel,
						unsigned long	*totalSamplesToRecord,
						unsigned long	*numberOfSamplesRecorded,
						unsigned long	*totalMsecsToRecord,
						unsigned long	*numberOfMsecsRecorded )
	= {MOVEL,sInGetRecordingStatus,sInToolNum,0xA800};

pascal OSErr 
SPBGetDeviceInfo(long sInRefNum, OSType infoType, char *infoData)
	= {MOVEL,sInGetDeviceInfo,sInToolNum,0xA800};

pascal OSErr 
SPBSetDeviceInfo(long sInRefNum, OSType infoType, char *infoData)
	= {MOVEL,sInSetDeviceInfo,sInToolNum,0xA800};

pascal OSErr 
SPBMilliSecondsToBytes(long sInRefNum, long *milliSeconds)
	= {MOVEL,sInMilliSecondsToBytes,sInToolNum,0xA800};

pascal OSErr 
SPBBytesToMilliSeconds(long sInRefNum, long *byteCount)
	= {MOVEL,sInBytesToMilliSeconds,sInToolNum,0xA800};

#ifdef __cplusplus
}
#endif

#endif
