/*
	File:		CTBTypes.r

	Contains:	Resource types for the Communications Toolbox

	Copyright:	 1989-1990 by Apple Computer, Inc.
				All rights reserved.

*/

type 'cbnd' {
		integer = $$CountOf(TypeArray) - 1;
		array TypeArray {
				literal longint;								/* Type 				*/
				integer = $$CountOf(IDArray) - 1;
				wide array IDArray {
						integer;								/* Local ID 			*/
						integer;								/* Actual ID			*/
				};
		};
};


/*----------------------------flst  Font Family List----------------------------*/
type 'flst' {
		integer = $$CountOf(Fonts);								/* # of fonts			*/
		array Fonts {
			pstring;											/* 		Font NAME			*/
			align word;
			unsigned hex integer	plain;						/* 		Font Style		*/
			integer;											/* 		Font Size		*/
			integer;											/* 		Font Mode		*/
		};
};

type 'fbnd' as 'cbnd';

type 'tbnd' as 'cbnd';




/* Define flags for "Channels" field of 'caps' resource */
#define cmData				(1 << 0)
#define cmCntl				(1 << 1)
#define cmAttn				(1 << 2)

#define	cmDataNoTimeout	 	(1 << 4)
#define	cmCntlNoTimeout 	(1 << 5)
#define cmAttnNoTimeout 	(1 << 6)

#define cmDataClean			(1 << 8)
#define cmCntlClean			(1 << 9)
#define cmAttnClean			(1 << 10)

//	for end of message field of caps resource
#define cmFlagsEOM			(1 << 0);

/* Connection tool capabilities resource */
type 'caps' {
	integer = $$CountOf (PairsArray);
	
	array PairsArray {

		switch {
		case Service:
			key literal longint 	= 'SERV';
			literal longint
				Datagram 		= 'DGRM',
				Stream 			= 'STRM',
				Record			= 'REC ',
				RecordOrStream	= 'RECS';
		
		case Reliability:
			key literal longint 	= 'RELY';
			longint guaranteed 		= 1, 
					notGuaranteed 	= 0;
			
		case Order:
			key literal longint 	= 'ORDR';
			longint guaranteed 		= 1, 
					notGuaranteed 	= 0;
			
		case EndOfMessage:
			key literal longint 	= 'EOM ';
			hex longint;
			
		case Protocol:
			key literal longint 	= 'PROT';
			literal longint
				TCP 	= 'TCP ',
				ADSP 	= 'ADSP',
				NSP 	= 'NSP ',
				LAT 	= 'LAT ',
				NSPg 	= 'NSPg',
				Serial 	= 'SERD',
				Modem	= 'MODM',
				MacPAD 	= 'PAD ';
				
		case Channels:
			key literal longint 	= 'CHAN';
			hex longint;
				
		case Listen:
			key literal longint 	= 'LSTN';
			literal longint
				synchronousOnly 	= 'SYNC',
				asynchronousOnly 	= 'ASYN',
				both 				= 'BOTH',
				notSupported 		= 0;
				
		case Open:
			key literal longint 	= 'OPEN';
			literal longint
				synchronousOnly 	= 'SYNC',
				asynchronousOnly 	= 'ASYN',
				both 				= 'BOTH',
				notSupported 		= 0;
				
		case Read:
			key literal longint 	= 'READ';
			literal longint
				synchronousOnly 	= 'SYNC',
				asynchronousOnly 	= 'ASYN',
				both 				= 'BOTH',
				notSupported 		= 0;
				
		case Write:
			key literal longint 	= 'WRIT';
			literal longint
				synchronousOnly 	= 'SYNC',
				asynchronousOnly 	= 'ASYN',
				both 				= 'BOTH',
				notSupported = 0;
				
		case Break:
			key literal longint 	= 'BRK ';
			literal longint
				synchronousOnly 	= 'SYNC',
				asynchronousOnly 	= 'ASYN',
				both 				= 'BOTH',
				notSupported 		= 0;
				
		case LowLevelIO:
			key literal longint 	= 'LLIO';
			literal longint
				supported 		= 1,
				notSupported 	= 0;
				
		case Timeout:
			key literal longint 	= 'TOUT';
			literal longint
				supported 		= 1,
				notSupported 	= 0;
				
		case Kill:
			key literal longint 	= 'KILL';
			literal longint
				supported 		= 1,
				notSupported 	= 0;
				
		case Abort:
			key literal longint 	= 'ABRT';
			literal longint
				supported 		= 1,
				notSupported 	= 0;
		};
	};
};
