/*
PROGRAM:	Installer

			 1988-1990 Apple Computer, Inc.  All Rights Reserved

PURPOSE:	This file contains all the resource templates necessary to
			create installer scripts.

HISTORY:
			4/27/90		BKJ		Cleaned up for Developer's Conference release.  Removed all old
								Default Maps stuff.
			4/5/90		BKJ		Added 3 new clause types - reportError, addAuditRecord, checkAuditRecord
			2/14/90		BKJ		Added 'indo' template for ordering disks.  (Happy Valentine's Day)
			12/2/89		BKJ		Changed format1 default maps to rules & default maps
								groups to rule frameworks
			9/12/89		BKJ		Made major changes for support of format1 default maps
			9/12/89		BKJ		Changed 'indg' to support a list of IDs rather than a range
			9/12/89		CIF		Added types for Installer action atoms
			9/11/89		BKJ		Added types for default map groups
			9/11/89		BKJ		Forked off of 6.0.4 final
			7/25/89		BKJ		Added support for international keyboards
			2/17/89				ALPHA 7 Release
			1/12/89				ALPHA 6 Release
			12/19/88			ALPHA 5
			5/26/88		CIF		Created
			9/14/88				ALPHA 2 Release
*/


#include "SysTypes.r"									/* For country codes */

/*----------------------------Installer 3.0 Scripts-----------------------------------*/

#define EvenPaddedString																						\
	pstring;																									\
	align word


#define OSType																									\
	literal longint
	
	
#define RsrcType																								\
	OSType												/* type of a rsrc which must exist in system */			\
		PatchRsrcType = 'PTCH',																					\
		UserFunctionType = 'infn',																				\
		AuditRsrcType = 'audt'


#define RsrcID																									\
	integer												/* ID of a rsrc which must exist in system */			\
		macPlusPatch = 117,																						\
		macSEPatch = 630,																						\
		macIIPatch = 376,																						\
		portablePatch = 890,																					\
		macIIciPatch = 1660,																					\
		oldAuroraPatch = 1657


#define RsrcName																								\
	EvenPaddedString

/*
 --------------------------------- Packages ---------------------------------*/
#define	PackageFlags 																							\
		boolean		notShowsOnCustom, ShowsOnCustom;		/* should show up on Custom screen	 */				\
		boolean		notRemovable, removable;				/* this package can be removed	 */					\
		fill bit[14]										/* Reserved */ 


#define	PartSpec																								\
		RsrcType;											/* Part Type: inpk, infa, or inra */ 				\
		RsrcID												/* Part ID */


type 'inpk' {
		switch {
			case format0:
				key integer = 0;							/* Package Format version */
				PackageFlags;								/* Package Flags */
				unsigned integer;							/* ICMT ID */
				unsigned longint;							/* Package Size */
				EvenPaddedString;								/* Package Name */
				unsigned integer = $$CountOf(PartsList);	/* Parts List */
				wide array PartsList {
					PartSpec;								/* Parts Spec */
				};
	
		};
};


type 'icmt' {												/* Installer comment */
		unsigned hex longint;								/* Creation Date  */
		unsigned hex longint;								/* Version */
		RsrcID;												/* Icon ID */
		EvenPaddedString;									/* Comment Text */
};

/*
 --------------------------------- File Specs ---------------------------------*/
#define FileSpecID																								\
		RsrcID


#define	FileSpecFlags																							\
		boolean		noSearchForFile, SearchForFile;			/* Search (tgt only) if not found in given path*/	\
		boolean		noTypeCrMustMatch, TypeCrMustMatch;		/* Type and creator must match	*/					\
		fill bit[14]										/* currently unused	*/


type	'infs' {														
		literal longint;									/* File Type	*/
		literal longint;									/* File Creator	*/
		unsigned hex longint;								/* Creation Date*/ 
		FileSpecFlags;										/* File Spec Flags */
		EvenPaddedString;									/* Full Path */
};

/*
 --------------------------------- File Atoms ---------------------------------*/
#define	opcodeFlags 																							\
		boolean		noDelRemove, delRemove;					/* Remove file or rsrc if remove clicked */			\
		boolean		noDelInstall, delInstall;				/* Delete target before copy	*/					\
		boolean		noCopy, copy;							/* Copy file or rsrc to destination */				\
		fill bit[3]											/* Reserved */


#define	FileAtomFlags																							\
		opcodeFlags;																							\
		fill bit[5]; 																							\
		boolean		noLeaveAloneIfNewer, leaveAloneIfNewer; /* OK for a newer version of this file to exist */	\
		boolean		noKeepExisting, keepExisting;			/* Keep tgt file if it already exists */			\
		boolean		noUpdateOnly, updateOnly;				/* Only update if target exists	*/					\
		boolean		noRsrcFork, rsrcFork;					/* Apply operation to rsrc fork */					\
		boolean		noDataFork, dataFork					/* Apply operation to data fork	*/					\


type 'infa' {
		switch {
			case format0:
				key integer = 0;							/* File Atom Format version */
				FileAtomFlags;								/* File Atom Flags		  */
				RsrcID;										/* tgt file spec ID */
				RsrcID;										/* src file spec ID */
				unsigned longInt;							/* File Size     */
				EvenPaddedString;								/* Atom Description     */
		};
};

/*
 --------------------------------- Resource Atoms ---------------------------------*/
#define	ResourceAtomFlags																						\
		opcodeFlags;																							\
		fill bit[3];										/* Reserved	*/										\
		boolean		noTgtRequired, tgtRequired;				/* target req or can be created by another inra */	\
		boolean		noKeepExisting, keepExisting;			/* Keep tgt rsrc if it already exists */			\
		boolean		noUpdateOnly, updateOnly;				/* Only update if target exists	*/					\
		boolean		noEvenIf, evenIf;						/* Do it even if rsrc protected */					\
		boolean		srcNeedExist, srcNeedNotExist;			/* Rsrc need not exist on source */					\
		boolean		noByID, byID;							/* Use name or id to find rsrc	*/					\
		boolean		notNameMustMatch, NameMustMatch			/* name must match*/


type 'inra' {
		switch {
			case format0:
				key integer = 0;							/* Resource Atom Format version */
				ResourceAtomFlags;							/* Resource Atom Flags */
				RsrcID;										/* tgt file spec ID */
				RsrcID;										/* src file spec ID */
				RsrcType;									/* Resource Type */
				integer;									/* Source ID */
				integer;									/* Target ID */
				unsigned longInt;							/* Resource Size */
				EvenPaddedString;								/* Atom Description */
				EvenPaddedString;								/* Resource Name */
		};
};
/*
 --------------------------------- Boot Block Atoms ---------------------------------*/
#define	BootBlockAtomFlags																						\
		fill bit[14];																							\
		boolean		notChangeOnInstall, changeOnInstall;	/* change parameter if we're installing */			\
		boolean		notChangeOnRemove, changeOnRemove		/* change parameter if we're removing */ 


#define	BootBlockUpdateFlags																					\
		fill bit[7];																							\
		boolean		replaceBBSysName, 																			\
					saveBBSysName; 																				\
		boolean		replaceBBShellName, 																		\
					saveBBShellName;																			\
		boolean		replaceBBDbg1Name, 																			\
					saveBBDbg1Name; 																			\
		boolean		replaceBBDbg2Name, 																			\
					saveBBDbg2Name; 																			\
		boolean		replaceBBScreenName, 																		\
					saveBBScreenName; 																			\
		boolean		replaceBBHelloName, 																		\
					saveBBHelloName; 																			\
		boolean		replaceBBScrapName,																			\
					saveBBScrapName; 																			\
		boolean		replaceBBCntFCBs, 																			\
					maxBBCntFCBs; 																				\
		boolean		replacebbCntEvts, 																			\
					maxBBCntEvts


type 'inbb' {
		switch {
			case format0:
				key integer = 0;							/* Boot Block Format version */
				BootBlockAtomFlags;							/* Boot Block Atom Flags	  */
						
						/* now case off the type for the boot block value */
				switch {
					case bbUpdate:
						key integer = -1;
						RsrcID;								/* The file spec ID for the source of the 'boot' resource */
						BootBlockUpdateFlags;
						
					case bbID:
						key integer = 1;
						decimal integer;
					
					case bbEntry:
						key integer = 2;
						decimal longint;
						
					case bbVersion:
						key integer = 3;
						decimal integer;
						
					case bbPageFlags:
						key integer = 4;
						decimal integer;
						
					case bbSysName:
						key integer = 5;
						EvenPaddedString;
						
					case bbShellName:
						key integer = 6;
						EvenPaddedString;
						
					case bbDbg1Name:
						key integer = 7;
						EvenPaddedString;
						
					case bbDbg2Name:
						key integer = 8;
						EvenPaddedString;
						
					case bbScreenName:
						key integer = 9;
						EvenPaddedString;
						
					case bbHelloName:
						key integer = 10;
						EvenPaddedString;
						
					case bbScrapName:
						key integer = 11;
						EvenPaddedString;
						
					case bbCntFCBs:
						key integer = 12;
						decimal integer;
						
					case bbCntEvts:
						key integer = 13;
						decimal integer;
						
					case bb128KSHeap:
						key integer = 14;
						decimal longint;
						
					case bb256KSHeap:
						key integer = 15;
						decimal longint;
						
					case bb512KSHeap:
						key integer = 16;
						decimal longint;
						
					case bbSysHeapSize:						/* Note - same as 512 */
						key integer = 16;
						decimal longint;
						
					case bbSysHeapExtra:
						key integer = 18;
						decimal longint;
						
					case bbSysHeapFract:
						key integer = 19;
						decimal longint;
				};
				EvenPaddedString;								/* Boot Block Atom Description */
		};
};


/*
 --------------------------------- Action Atoms ---------------------------------*/
#define	ActionAtomFlags																							\
		fill bit[14]; 																							\
		boolean		dontActOnRemove, 																\
					actOnRemove;				/* Call this procedure when doing a remove */		\
		boolean		dontActOnInstall, 																\
					actOnInstall				/* Call this procedure when doing an Install */


type 'inaa' {
		switch {
			case format0:
				key integer = 0;							/* Action Atom Format version */
				ActionAtomFlags;							/* Action Atom Flags		  */
				PartSpec;									/* Resource type & id for executable resource */
				EvenPaddedString;								/* Atom Description     */
		};
};

/*
 --------------------------------- Audit Trail Atoms ---------------------------------*/
type 'inat' {
		switch {
			case format0:
				key integer = 0;							/* Version */
				FileSpecID;									/* Target File Spec */
				OSType;										/* Selector */
				literal longint;							/* Value */
			};
};

/*
 --------------------------------- Installer Rules ---------------------------------*/

#define Version																									\
	hex byte;												/* System file Major revision in BCD*/				\
	hex byte;												/* System file Minor revision in BCD*/				\
	hex byte	development = 0x20,							/* System file Release stage		*/				\
				alpha = 0x40, beta = 0x60,																		\
				final = 0x80, release = 0x80;																	\
	hex byte												/* Non Final */
	
	
/* Note that if the minimal & maximal target disk sizes are both 0, any volume will match */
/* If the Minimal is > 0 and the maximal = 0, then only the minimum requirement is used */
#define TargetVolReqs										/* Required Target Volume Description */			\
	decimal longint											/* Minimal Target Disk Size */						\
		floppy = 0,																								\
		hdFloppy = 1400,																						\
		hardDisk = 10000;																						\
	decimal longint											/* Maximal Target Disk Size */						\
		floppy = 900,																							\
		hdFloppy = 1500,																						\
		hardDisk = 0
	
	
#define IntegerList																								\
	unsigned integer = $$CountOf(IntegerArray);																	\
	array IntegerArray {																						\
		integer;																								\
	}


#define LongIntList																								\
	unsigned integer = $$CountOf(LongIntArray);																	\
	array LongIntArray {																						\
		longint;																								\
	}


#define PackageList																								\
	IntegerList


#define AssertionList																							\
	IntegerList
	
	
#define GestaltList																								\
	LongIntList


#define	alwaysFalseKey							-1
#define	checkGestaltKey							1
#define checkMinMemoryKey						2
#define	checkFileDataForkExistsKey				3
#define	checkFileRsrcForkExistsKey				4
#define	checkFIleContainsRsrcByIDKey			5
#define	checkFileContainsRsrcByNameKey			6
#define	checkFileVersionKey						7
#define	checkFileCountryCodeKey					8
#define	checkTgtVolSizeKey						9
#define	checkUserFunctionKey					10
#define	checkAllAssertionsKey					11
#define	checkAnyAssertionKey					12
#define checkMoreThanOneAssertionKey			13
#define	addUserDescriptionKey					14
#define	addPackagesKey							15
#define	addAssertionKey							16
#define reportVolErrorKey						17
#define reportSysErrorKey						18
#define addAuditRecKey							19
#define checkAuditRecKey						20
#define checkAnyAuditRecKey						21



/* The Easy Install Rules */
type 'inrl' {
	switch {
		case format0:
			key integer = 0;
			hex integer = 0;											/* Rule Flags for future use */
			unsigned integer = $$CountOf(Rules);
			wide array Rules {
				switch {
					case alwaysFalse:
						key integer = alwaysFalseKey;
						
					case checkGestalt:
						key integer = checkGestaltKey;
						OSType;											/* Gestalt selector */
						GestaltList;									/* Legal results */
					
					case checkMinMemory:
						key integer = checkMinMemoryKey;
						decimal longint;

					case checkFileDataForkExists:
						key integer = checkFileDataForkExistsKey;
						FileSpecID;
						
					case checkFileRsrcForkExists:
						key integer = checkFileRsrcForkExistsKey;
						FileSpecID;
						
					case checkFileContainsRsrcByID:
						key integer = checkFileContainsRsrcByIDKey;
						FileSpecID;
						RsrcType;
						RsrcID;
						
					case checkFileContainsRsrcByName:
						key integer = checkFileContainsRsrcByNameKey;
						FileSpecID;
						RsrcType;
						RsrcName;
						
					case checkFileVersion:
						key integer = checkFileVersionKey;
						FileSpecID;
						Version;
						
					case checkFileCountryCode:
						key integer = checkFileCountryCodeKey;
						FileSpecID;
						integer Country;
						
					case checkTgtVolSize:
						key integer = checkTgtVolSizeKey;
						TargetVolReqs;
						
					case checkATVersion:
						key integer = checkATVersionKey;
						unsigned hex integer;
						
					case checkUserFunction:
						key integer = checkUserFunctionKey;
						RsrcType;
						RsrcID;
						
					case checkAllAssertions:
						key integer = checkAllAssertionsKey;
						AssertionList;
						
					case checkAnyAssertion:
						key integer = checkAnyAssertionKey;
						AssertionList;
						
					case checkMoreThanOneAssertion:
						key integer = checkMoreThanOneAssertionKey;
						AssertionList;
						
					case addUserDescription:
						key integer = addUserDescriptionKey;
						EvenPaddedString;
						
					case addPackages:
						key integer = addPackagesKey;
						PackageList;
						
					case addAssertion:
						key integer = addAssertionKey;
						AssertionList;
						
					case reportVolError:
						key integer = reportVolErrorKey;
						EvenPaddedString;
						
					case reportSysError:
						key integer = reportSysErrorKey;
						EvenPaddedString;
						
					case addAuditRec:
						key integer = addAuditRecKey;
						FileSpecID;
						OSType;
						literal longint;
						
					case checkAuditRec:
						key integer = checkAuditRecKey;
						FileSpecID;
						OSType;
						literal longint;
						
					case checkAnyAuditRec:
						key integer = checkAnyAuditRecKey;
						FileSpecID;
						OSType;
						LongIntList;
						
			};
		};
	};
};


#define RuleList IntegerList
	
/* The rule framework */
type 'infr' {
	switch {
		case format0:
			key integer = 0;								/* Rule framework version */
			unsigned integer = $$CountOf(RuleArray);
			wide array RuleArray {
				boolean pickFirst, pickAll;					/* which packages to select? */						
				fill bit[15];
				RuleList;									/* List of rules */									
			};
	};
};


/* The list of disks we may need, and the order we want them to appear it */
/* This is optional. */
type 'indo' {
		integer = $$Countof(StringArray);
		array StringArray {
				EvenPaddedString;
		};
};
